﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: base_file.h
\brief 通用dll接口定义头文件, 该DLL不依赖Qt，依赖STL。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10 
* please import basedll.dll
*/

#pragma once

#include "base_export.h"
#include <stdio.h>
#include <string>
#include "customtype.h"

namespace ns_train {
/**
* @brief 文件操作类
*/
class BASE_API CPPFile {
public:
	CPPFile();
	CPPFile(const char* i_pFileName);
	~CPPFile();

	/**
	* @brief 设置文件名
	*
	* @param[in] strFileName 文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @return true:成功,false:失败
	*/
	bool setFileName(std::string strFileName);
	/**
	* @brief 获取文件名
	*
	* @return true:成功,false:失败
	*/
	std::string getFileName(void) const;

	/**
	* @brief 以指定模式打开文件
	*
	* @param[in] szMode 文件的打开模式
	*   1) 文件使用方式由r, w, a, t, b， + 六个字符拼成，各字符的含义是：
	*       r(read):    只读
	*       w(write):   只写
	*       a(append):  追加
	*       t(text):    文本文件，可省略不写
	*       b(binary) : 二进制文件
	*       +:          读和写
	*    2) 凡用“r"打开一个文件时，该文件必须已经存在，且只能从该文件读出。
	*    3) 用“w"打开的文件只能向该文件写入。若打开的文件不存在，则以指定的文件名建立该文件，若打开的文件已经存在，则将该文件删去，重建一个新文件。
	*    4) 若要向一个已存在的文件追加新的信息，用“a"方式打开文件。如果指定文件不存在则尝试创建该文件。
	*    5) 在打开一个文件时，如果出错，fopen将返回一个空指针值NULL。在程序中可以用这一信息来判别是否完成打开文件的工作，并作相应的处理。
	*       具体示例：
	*       "r" = "rt": 【打开】一个文本文件，文件必须存在，只允许读
	*       "r+" = "rt+" : 【打开】一个文本文件，文件必须存在，允许读写
	*       "rb" : 【打开】一个二进制文件，文件必须存在，只允许读
	*       "rb+":【打开】一个二进制文件，文件必须存在，允许读写
	*       "w" = “wt" : 【新建】一个文本文件，已存在的文件将内容清空，只允许写
	*       "w+" = "wt+" : 【新建】一个文本文件，已存在的文件将内容清空，允许读写
	*       "wb":【新建】一个二进制文件，已存在的文件将内容清空，只允许写
	*       "wb+":【新建】一个二进制文件，已存在的文件将内容清空，允许读写
	*       "a" = "at" : 【打开或新建】一个文本文件，只允许在文件末尾追写
	*       "a+" = "at+" : 【打开或新建】一个文本文件，可以读，但只允许在文件末尾追写
	*       "ab":【打开或新建】一个二进制文件，只允许在文件末尾追写
	*       "ab+":【打开或新建】一个二进制文件，可以读，但只允许在文件末尾追写
	* @return true:成功,false:失败
	*/
	bool open(const char *szMode);
	/**
	* @brief 关闭文件
	*
	* @return void
	*/
	void close(void);

	/**
	* @brief 文件是否存在
	*
	* @return true:存在,false:不存在
	*/
	bool isFileExist(void) const;
	/**
	* @brief 文件是否已结束
	*
	* @return true:存在,false:不存在
	*/
	bool isEOF(void) const;
	/**
	* @brief 指定文件是否存在
	*
	* @param[in] strFileName 文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @return true:存在,false:不存在
	*/
	static bool isFileExist(std::string strFileName);
	/**
	* @brief 获取文件长度
	*
	* @return 文件长度
	*/
	type_size getSize(void);
	/**
	* @brief 获取指定文件的长度，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	*
	* @param[in] strFileName 文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @return 文件长度
	*/
	static type_size getSize(std::string strFileName);

	/**
	* @brief 从文件中读取指定长度的数据。从文件的当前游标位置开始读取。
	*
	* @param[out] pBuf 读取到的数据将存入该缓冲区
	* @param[in] nMaxCount 最多要读取的字节数。
	* @return 实际读取到的字节数。使用文本方式读取时，可能出现：实际读取到的字节数<nMaxCount。
	*/
	type_size read(char* pBuf, type_size nMaxCount);
	/**
	* @brief 向文件写入指定长度的数据。从文件的当前游标位置开始写入。
	*
	* @param[out] pBuf 读取到的数据将存入该缓冲区
	* @param[in] nCount 要写入的字节数
	* @return 实际读取到的字节数
	*/
	type_size write(const char* pBuf, type_size nCount);

	/**
	* @brief 从文件中读取一行数据。从文件的当前游标位置开始读取。
	*
	* @param[out] pBuf 读取到的数据将存入该缓冲区
	* @param[in] nMaxCount 最多要读取的字节数
	* @return true:成功,false:失败
	*/
	bool readLine(char* o_pBuf, int nMaxCount);
	/**
	* @brief 向文件写入一行数据。接口内部自动添加"\n"。
	*
	* @param[out] pBuf 待写入的数据
	* @return void
	*/
	void writeLine(const char* pBuf);

	/**
	* @brief 将文件游标移动到文件开头。
	*/
	void seekToBegin(void);
	/**
	* @brief 将文件游标移动到文件末尾。
	*/
	void seekToEnd(void);
	/**
	* @brief 将文件游标移动到指定位置。
	* @param[in] nOffset 偏移量
	* @param[in] nFrom  基准位置,取值范围（需要包含stdio.h）:
	*                   SEEK_SET:以文件开头为基准
	*					SEEK_END:以文件末尾为基准
	*					SEEK_CUR:以文件当前位置为基准
	* @return 移动后，文件游标的位置
	*/
	type_size seekToPosition(type_size nOffset, int nFrom);

	/**
	* @brief 获取文件游标的当前位置。
	* @return 文件游标的当前位置
	*/
	type_size currentPosition(void);

	/**
	* @brief 拷贝文件。
	* @param[in] szSrc 源文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @param[in] szDst 目标文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @return true:成功,false:失败，可能的原因为目标文件已存在|没有权限。
	*/
	static bool copy(const char *szSrc, const char* szDst);
	/**
	* @brief 为文件改名。
	* @param[in] szNewName 新文件名，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
	* @return true:成功,false:失败，可能的原因为目标文件已存在|没有权限。
	*/
	bool rename(const char* szNewName);

	/**
	* @brief 删除本文件。
	* @return true:成功,false:失败。
	*/
	bool remove(void);
	/**
	* @brief 重载<<操作符，将 char 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_char& data);
	/**
	* @brief 重载<<操作符，将 type_wchar 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_wchar& data);
	/**
	* @brief 重载<<操作符，将 type_int8 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_int8& data);
	/**
	* @brief 重载<<操作符，将 type_uint8 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_uint8& data);
	/**
	* @brief 重载<<操作符，将 type_int16 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_int16& data);
	/**
	* @brief 重载<<操作符，将 type_uint16 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_uint16& data);
	/**
	* @brief 重载<<操作符，将 type_int32 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_int32& data);
	/**
	* @brief 重载<<操作符，将 type_uint32 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_uint32& data);
	/**
	* @brief 重载<<操作符，将 type_int64 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_int64& data);
	/**
	* @brief 重载<<操作符，将 type_uint64 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_uint64& data);
	/**
	* @brief 重载<<操作符，将 type_float 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_float& data);
	/**
	* @brief 重载<<操作符，将 type_double 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const type_double& data);
	/**
	* @brief 重载<<操作符，将 string 类型的数据持久化到文件。
	* @param[in] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator<<(const std::string& data);

	/**
	* @brief 重载>>操作符，将 char 类型的数据从文件中读出。
	* @param[out] data 待持久化到文件的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_char& data);
	/**
	* @brief 重载>>操作符，将 type_wchar 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_wchar& data);
	/**
	* @brief 重载>>操作符，将 type_int8 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_int8& data);
	/**
	* @brief 重载>>操作符，将 type_uint8 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_uint8& data);
	/**
	* @brief 重载>>操作符，将 type_int16 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_int16& data);
	/**
	* @brief 重载>>操作符，将 type_uint16 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_uint16& data);
	/**
	* @brief 重载>>操作符，将 type_int32 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_int32& data);
	/**
	* @brief 重载>>操作符，将 type_uint32 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_uint32& data);
	/**
	* @brief 重载>>操作符，将 type_int64 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_int64& data);
	/**
	* @brief 重载>>操作符，将 type_uint64 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_uint64& data);
	/**
	* @brief 重载>>操作符，将 type_float 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_float& data);
	/**
	* @brief 重载>>操作符，将 type_double 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(type_double& data);
	/**
	* @brief 重载>>操作符，将 string 类型的数据从文件中读出。
	* @param[out] data 待从文件中读出的数据
	* @return CPPFile&。
	*/
	CPPFile& operator>>(std::string& data);
private:
	FILE*			m_pFile;		// 文件指针
	std::string		m_strFileName;	// 文件名
};
} // namespace ns_train
