﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: comdata.h
\brief 通信用数据结构定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/02 
*/
#include "comdata_export.h"
#include "customtype.h"

namespace ns_train {
enum ETestEnum {
	ETest_InvalidValue = -1,
	ETest_Value1 = 0
};

struct sTest {
	type_int16 nTestId; // Id
	type_uint32 nCode;	// 代码
};

#include "prealign.h"	// 仅需要把通信用的结构体进行取消对齐操作，其他结构体可以排除在外
// 遥测类返回值结构
struct COMDATA_API YCFlag {
#if defined CPP_LITTLE_ENDIAN
	type_uint8	bInvalid : 1,	// 数据不可用标志
				bNotInit : 1,	// 未初始化标志
				bNormal : 1,	// 正常数据标志
				bRemote : 1,	// 遥传标志
				bSet : 1,		// 人工设置标志
				bCalculate : 1,	// 计算标志
				bReplace : 1,	// 被替代标志
				bDoubtVal : 1;	// 可疑数据标志

	type_uint8	bNoFresh : 1,	// 不刷新标志
				bFlash : 1,		// 闪烁标志
				bOverUp : 1,	// 越有效上限
				bOverDown : 1,	// 越有效下限
				bNoAlarm : 1,	// 报警屏蔽/开放状态
				bDoubleCol : 1,	// 双采集点标志
				bNoCollect : 1,	// 该数据无采集点
				bStart : 1;		// 已启动

	type_uint8	bCheck : 1,		// 检修
				bPad0 : 7;		// 保留

#else // !CPP_LITTLE_ENDIAN，交换字节内各域顺序	
	type_uint8	bDoubtVal : 1,	// 可疑数据标志
				bReplace : 1,	// 被替代标志
				bCalculate : 1,	// 计算标志
				bSet : 1,		// 人工设置标志
				bRemote : 1,	// 遥传标志
				bNormal : 1,	// 正常数据标志
				bNotInit : 1,	// 未初始化标志
				bInvalid : 1;	// 数据不可用标志				

	type_uint8	bStart : 1,		// 已启动
				bNoCollect : 1,	// 该数据无采集点
				bDoubleCol : 1,	// 双采集点标志
				bNoAlarm : 1,	// 报警屏蔽/开放状态
				bOverDown : 1,	// 越有效下限
				bOverUp : 1,	// 越有效上限
				bFlash : 1,		// 闪烁标志
				bNoFresh : 1;	// 不刷新标志

	type_uint8	bPad0 : 7,		// 保留
				bCheck : 1;		// 检修
#endif
	YCFlag();
	void HostToNet() {/*无需转换*/ }
	void NetToHost() {/*无需转换*/ }
} DISALIGN;


// 状态型量测数据结构体
struct COMDATA_API s_u16data {
	type_uint32 uDataId;	// 量测Id
	type_uint16 u16Value;	// 量测值
	s_u16data();			// 构造函数
	void HostToNet();
	void NetToHost();
} DISALIGN;

// 双精度量测数据结构体
typedef struct COMDATA_API _s_doubledata {
	type_uint32 uDataId;// 量测Id
	type_double dValue;	// 量测值
	YCFlag	ycFlag;		// 量测标志
	_s_doubledata();	// 构造函数
	void HostToNet();
	void NetToHost();
} DISALIGN s_doubledata;

#include "postalign.h"

}