﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: customtype.h
\brief C++跨平台数据类型定义
       本头文件仅用来演示不同操作系统中定长数据类型的定义方法，请勿作为真正的数据类型进行使用。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/02 
*/

#pragma once
#if !defined(WIN32)
#include <unistd.h>
#endif

//  不使用参数宏定义
#ifndef CPP_UNUSED
#define CPP_UNUSED(x) (void)x;
#endif

#include "ostype.h"

typedef char				type_char;
typedef wchar_t				type_wchar;
typedef signed char			type_int8;
typedef unsigned char		type_uint8;
typedef signed short		type_int16;
typedef unsigned short		type_uint16;
typedef signed int			type_int32;
typedef unsigned int		type_uint32;
typedef float				type_float;
typedef double				type_double;

#if defined(AIX)                /* compiler: xlc(cc,c89) */
	#if defined(__64BIT__)
		typedef signed long			type_int64;
		typedef unsigned long		type_uint64;
	#else
		typedef long long			type_int64;
		typedef unsigned long long	type_uint64;
	#endif

#elif defined(ALPHA)            /* compiler: cxx (True64) */
	typedef signed __int64			type_int64;
	typedef unsigned __int64		type_uint64;

#elif defined(WIN32)            /* compiler: microsoft C++ */
	typedef signed __int64			type_int64;
    typedef unsigned __int64		type_uint64;

#elif defined(SUN)              /* SOLARIS: compiler: cc */
	#if defined(__sparcv9)
		typedef signed long			type_int64;
		typedef unsigned long		type_uint64;
	#else
		typedef signed long long	type_int64;
		typedef unsigned long long	type_uint64;
	#endif

#elif defined(HPUX)				/* HP-UX: compiler: aCC */
	#if defined(__LP64__)
		typedef signed long			type_int64;
		typedef unsigned long		type_uint64;
	#else
		typedef signed long long	type_int64;
		typedef unsigned long long	type_uint64;
	#endif

#elif defined(LINUX)            /* LINUX: compiler: gcc */
	#if defined(__LP64__)
		typedef signed long			type_int64;
		typedef unsigned long		type_uint64;
	#else
		typedef signed long long	type_int64;
		typedef unsigned long long	type_uint64;
	#endif
#else    /* other platform */
	#error 本平台未被支持！

#endif

#ifdef PROJECT_32
typedef type_int32 type_size;
#else
typedef type_int64 type_size;
#endif

typedef type_int8  type_bool;
const type_bool VALUE_FALSE = 0;
const type_bool VALUE_TRUE = 1;

typedef type_int64	type_UtcTime;	// utc时间（世界标准时间），从1582/10/15 00:00:00开始计时，精度100纳秒
typedef type_int64	type_EpochTime;	// epoch时间（unix诞生元年），从1970/01/01 00:00:00开始计时，精度微秒
typedef type_int64	type_TimeGap;	// 两个 type_EpochTime 的差值，精度微秒
#ifdef _USE_32BIT_TIME_T
typedef type_int32	type_time_t;	// 用来代替time_t。std::time_t保存的也是epoch时间，只是精度为秒，
									// 并且std::time_t在32位、64位系统中的长度不固定，因此统一为type_time_t。
#else
typedef type_int64	type_time_t;	// 用来代替time_t。std::time_t保存的也是epoch时间，只是精度为秒，
									// 并且std::time_t在32位、64位系统中的长度不固定，因此统一为type_time_t。
#endif


//! 进程ID类型定义
#ifdef WIN32
	typedef type_uint32 type_Pid;
#else
	typedef pid_t type_Pid;
#endif

/* 定义 NULL */
#ifndef NULL
#	ifdef __cplusplus
#		define NULL    0
#	else
#		define NULL    ((void *)0)
#	endif
#endif



/* DISALIGN: 用来指定对齐方式的宏，在GCC下起作用。
 * 用法：紧靠变量或结构定义之后放置，跟结构体或变量之间以空白符隔开，例如：
 *
 * int x DISALIGN;
 * struct Y {
 *     short a;
 *     int b;
 * } DISALIGN;
 *
 * 它指定变量或结构按1字节对齐、位域按位对齐，实际也就是取消对齐(所以起名DISALIGN)。
 * 为保证同一个结构体在各平台的大小及对齐方式一致，
 * 在AIX下应定义：
 * 		#pragma align(packed)
 * 		... XXX结构体定义代码
 * 		#pragma align(power)
 *
 * 而在WIN32 c++下应定义：
 * 		#pragma pack(push)
 * 		#pragma pack(1)	// 以字节方式对齐
 * 		... XXX结构体定义代码
 * 		#pragma pack(pop)
 *

 */

#ifndef DISALIGN
#	if defined(__GNUG__)	// GCC编译器预定义的宏
#		define DISALIGN __attribute__((packed))
#	else
#		define DISALIGN
#	endif
#endif
