﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: ks07_08_client_protocolobject.h
\brief  通信规约对象类定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ks07_08_dll.dll
*/
#pragma once

#include "base/basedll/base_class.h"
#include "protocolobject.h"
#include "ks07_08_dll_export.h"

#include <list>

struct KS07_08_DLL_API s_IDValue {
	s_IDValue(type_uint32 id, type_double value);
	type_uint32 nId;
	type_double dValue;
};
namespace ns_train {
	class CPPSerial;
}

class KS07_08_DLL_API CClientProtocolObject : public CProtocolObject {
public:
	/**
	* @brief  构造函数
	*/
	CClientProtocolObject();
	/**
	* @brief  析构函数
	*/
	~CClientProtocolObject();
	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	virtual bool open();
	/**
	* @brief  将待发送数据进行组帧并发送
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void makeFrameAndSend();
	/**
	* @brief  接收数据并解帧。
	* @param[in] pSerial 串口对象。
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void recvAndParseFrame();
	/**
	* @brief  添加命令：获取服务器端数据。采用异步方式，即先把命令压栈，等待线程执行完毕并收到服务器端数据后，才在终端中显示。
	* @param[in] uDataId 数据项Id
	* @return 无
	*/
	void addCommand_GetData(type_uint32 uDataId);
	/**
	* @brief  添加命令：更新服务器端数据。采用异步方式，即先把命令压栈。
	* @param[in] uDataId 数据项Id
	* @param[in] dValue 数据
	* @return 无
	*/
	void addCommand_SetData(type_uint32 uDataId, type_double dValue);
	/**
	* @brief  添加命令：获取服务器端的文件。
	* @param[in] str 文件名（不带路径，默认从该目录获取：$PROJECT_DEV_HOME/test/chapter07/ks07_08）
	* @return 无
	*/
	void addCommand_GetFile(const std::string& str);
	/**
	* @brief  设置串口通信对象
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void setSerialObject(ns_train::CPPSerial* pSerial);
private:
	/**
    * @brief  获取帧类型。
    * @return 无
    */
    virtual void parseFrame_GetFrameType();
    /**
    * @brief  搜索同步字
    * @return 无
    */
    virtual void parseFrame_Synchronization();
    /**
    * @brief  获取帧长度。
    * @return 无
    */
    virtual void parseFrame_GetFrameLength();
	/**
	* @brief  获取单个模拟量数据帧(ESERVERFRAME_ONEANALOGDATA)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_OneAnalog();
	/**
	* @brief  获取模拟量数据帧(ESERVERFRAME_ANALOGDATAS)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_Analogs();
	/**
	* @brief  获取文件数据帧(ESERVERFRAME_FILE)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_File();
	/**
    * @brief  解析数据。
    * @return 无
    */
    virtual void parseFrame_Parse();
	/**
   * @brief  解析单个模拟量数据帧(ESERVERFRAME_ONEANALOGDATA)。
   * @return 无
   */
	void parseFrame_OneAnalog();
	/**
	* @brief  解析模拟量数据帧(ESERVERFRAME_ANALOGDATAS)。
	* @return 无
	*/
	void parseFrame_Analogs();
	/**
	* @brief  解析文件数据帧(ESERVERFRAME_FILE)。
	* @return 无
	*/
	void parseFrame_File();
	/**
    * @brief  复位与接收数据相关的变量。
    * @return 无
    */
    virtual void reset_receive();
    /**
    * @brief  复位与发送数据相关的变量。
    * @return 无
    */
    virtual void reset_send();    
	/**
    * @brief  申请内存
    * @return 无
    */
    virtual void makeBuffer();	
	/**
	* @brief  重发次数超限
	* @return 无
	*/
	virtual void reSendOverLimit() {

	}
private:
	ns_train::CPPMutex m_mutex;
	std::list<type_uint32> m_lstGetId;
	std::list<s_IDValue> m_lstSetIdValue;
	std::string m_strFileName;					// 文件名
	type_uint32 m_u32CurrentFrameDataCount;		// 本帧数据个数
	type_uint32 m_u32StartId;					// 起始数据序号
};

