﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolcommand.h
\brief  规约命令类定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ks07_08_dll.dll
*/
#pragma once

#include "ks07_08_dll_export.h"
#include "commandinterface.h"
#include "customtype.h"

#include <string>

namespace ns_train {
	class CPPSerial;
}

class KS07_08_DLL_API CProtocolCommand : public CCommandInterface {
public:
 	/**
	* @brief 构造函数
	*/
	CProtocolCommand();
 	/**
	* @brief 执行
	* @return true:成功,false:失败
	*/
	virtual bool exec() = 0;
 	/**
	* @brief 设置发送缓冲区指针
	* @return true:成功,false:失败
	*/
	void setSendBuffer(char* pSendBuffer) {
		m_pSendBuffer = pSendBuffer;
	}
 	/**
	* @brief 设置串口通信对象
	* @return void
	*/
	void setSerial(ns_train::CPPSerial* pSerial) {
		m_pSerial = pSerial;
	}
protected:
	char* m_pSendBuffer;
	ns_train::CPPSerial* m_pSerial;
};
/**
* @brief 发送确认帧
*/
class KS07_08_DLL_API CCommandAck : public CProtocolCommand {
public:
 	/**
	* @brief 构造函数
	*/
	CCommandAck();
 	/**
	* @brief 执行
	* @return true:成功,false:失败
	*/
	virtual bool exec(); 	
 	/**
	* @brief 设置待确认的帧序号
	* @param[in] n32FrameIndex 待确认的帧序号
	* @return void
	*/
	virtual void setFrameIndex(type_int32 n32FrameIndex) {
		m_n32FrameIndex = n32FrameIndex;
	}
private:
	type_int32 m_n32FrameIndex;					// 数据帧序号,-1表示无效，从0开始为有效帧序号。
};
/**
* @brief 获取服务器端数据命令
*/
class KS07_08_DLL_API CCommandSetData : public CProtocolCommand {
public:
 	/**
	* @brief 构造函数
	*/
	CCommandSetData();
 	/**
	* @brief 执行
	* @return true:成功,false:失败
	*/
	virtual bool exec(); 	
 	/**
	* @brief 更新服务器端数据
	* @param[in] uDataId 数据项Id
	* @param[in] dValue 数据
	* @return void
	*/
	virtual void setData(type_uint32 uDataId, type_double dValue) {
		m_uDataId = uDataId;
		m_dValue = dValue;
	}
private:
	type_uint32 m_uDataId;
	type_double m_dValue;
};
/**
* @brief 获取服务器端数据命令
*/
class KS07_08_DLL_API CCommandGetData : public CProtocolCommand {
public:
 	/**
	* @brief 构造函数
	*/
	CCommandGetData();
 	/**
	* @brief 执行
	* @return true:成功,false:失败
	*/
	virtual bool exec(); 	
 	/**
	* @brief 执行
	* @param[in] uDataId 数据项Id
	* @return void
	*/
	virtual void setDataId(type_uint32 uDataId) {
		m_uDataId = uDataId;
	}
private:
	type_uint32 m_uDataId;
};

/**
* @brief 获取服务器文件
*/
class KS07_08_DLL_API CCommandGetFile : public CProtocolCommand {
public:
 	/**
	* @brief 构造函数
	*/
	CCommandGetFile();
 	/**
	* @brief 执行
	* @return true:成功,false:失败
	*/
	virtual bool exec(); 	
 	/**
	* @brief 执行
	* @param[in] strFileName 文件名Id,不含路径，默认为： "$TRAINDEVHOME/test/chapter07_08/"
	* @return void
	*/
	virtual void setFileName(const std::string& strFileName) {
		m_strFileName = strFileName;
	}
private:
	std::string m_strFileName;
};

