﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolinterface.h
\brief  通信规约接口类定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ks07_08_dll.dll
*/
#pragma once
#include "ks07_08_dll_export.h"

namespace ns_train {
	class CPPSerial;
}
class CCommandInterface;

class KS07_08_DLL_API CProtocolInterface {
public:
    /**
    * @brief  设置串口通信对象
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
	virtual void setSerialObject(ns_train::CPPSerial* pSerial) = 0;
    /**
    * @brief  获得串口通信对象
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
	virtual ns_train::CPPSerial*  getSerialObject() const = 0;
	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	virtual bool open() = 0;
	/**
	* @brief 串口是否已打开
	* @return true:已打开,false:未打开
	*/
	virtual bool isOpen() = 0;
	/**
	* @brief 关闭串口
	* @return true:成功,false:失败
	*/
	virtual bool close() = 0;
	/**
	* @brief 从串口中接收数据
	* @param[out] data 接收缓冲区
	* @param[in] cout 准备接收的字节数
	* @return 从串口中实际接收到的数据字节数
	*/
	virtual int read(
		char* data,
		int count) = 0;
	/**
	* @brief 向串口中发送数据
	* @param[in] data 发送缓冲区
	* @param[in] cout 准备发送的字节数
	* @return 实际向串口中写入的字节数
	*/
	virtual int write(
		char* data,
		int count) = 0;
	/**
    * @brief  将待发送数据进行组帧并发送
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    virtual void makeFrameAndSend() = 0;
    /**
    * @brief  接收数据并解帧。
    * @param[in] pSerial 串口对象。
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    virtual void recvAndParseFrame() = 0;
	/**
	* @brief  添加命令。
	* @param[in] pCommand 命令对象。
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void pushCommand(CCommandInterface* pCommand) = 0;
	/**
	* @brief  获取一个命令。
	* @return 命令队列中第一个对象，如果队列为空，则返回NULL。
	*/
	virtual CCommandInterface* takeCommand() = 0;
};

