﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: ksui_01_server_protocolobject.h
\brief  通信规约对象类定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ksui_01_dll.dll
*/
#pragma once

#include "base/basedll/base_class.h"
#include "protocolobject.h"
#include "customtype.h"
#include "ksui_01_dll_export.h"

#include <string>

namespace ns_train {
	class CPPSerial;
}

class KSUI_01_DLL_API CServerProtocolObject : public CProtocolObject {
public:
	/**
	* @brief  构造函数
	*/
	CServerProtocolObject();
	/**
	* @brief  析构函数
	*/
	~CServerProtocolObject();
	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	virtual bool open();
	/**
	* @brief  将待发送数据进行组帧并发送
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void makeFrameAndSend();
	/**
	* @brief  接收数据并解帧。
	* @param[in] pSerial 串口对象。
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void recvAndParseFrame();
	/**
	* @brief  设置串口通信对象
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void setSerialObject(ns_train::CPPSerial* pSerial);
private:
	/**
	* @brief  将待发送数据进行组帧并发送-一个模拟量数据
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	void makeFrameAndSend_OneAnalog();
	/**
	* @brief  将待发送数据进行组帧并发送-模拟量数据集合
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	void makeFrameAndSend_Analogs();
	/**
	* @brief  将待发送数据进行组帧并发送-文件
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	void makeFrameAndSend_File();
	/**
    * @brief  获取帧类型。
    * @return 无
    */
    virtual void parseFrame_GetFrameType();
    /**
    * @brief  搜索同步字
    * @return 无
    */
    virtual void parseFrame_Synchronization();
    /**
    * @brief  获取帧长度。
    * @return 无
    */
    virtual void parseFrame_GetFrameLength();
	/**
	* @brief  计算“确认”帧(ECLIENTFRAME_ACK)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_Ack();
	/**
	* @brief  计算“设置数据”帧(ECLIENTFRAME_SETDATA)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_SetData();
	/**
	* @brief  计算“获取数据”帧(ECLIENTFRAME_GETDATA)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_GetData();
	/**
	* @brief  计算“获取文件”帧(ECLIENTFRAME_GETFILE)的帧长度。
	* @return 无
	*/
	void parseFrame_GetFrameLength_GetFile();
	/**
    * @brief  解析数据。
    * @return 无
    */
    virtual void parseFrame_Parse();
	/**
	* @brief  解析数据帧：ECLIENTFRAME_ACK
	* @return 无
	*/
	void parseFrame_Ack();
	/**
	* @brief  解析数据帧：ECLIENTFRAME_SETDATA
	* @return 无
	*/
	void parseFrame_SetData();
	/**
	* @brief  解析数据帧：ECLIENTFRAME_GETDATA
	* @return 无
	*/
	void parseFrame_GetData();
	/**
	* @brief  解析数据帧： ECLIENTFRAME_GETFILE
	* @return 无
	*/
	void parseFrame_GetFile();
	/**
    * @brief  复位与接收数据相关的变量。
    * @return 无
    */
    virtual void reset_receive();
    /**
    * @brief  复位与发送数据相关的变量。
    * @return 无
    */
    virtual void reset_send();    
	/**
    * @brief  申请内存
    * @return 无
    */
    virtual void makeBuffer();	
	/**
	* @brief  重发次数超限
	* @return 无
	*/
	virtual void reSendOverLimit();

protected:
	ns_train::CPPMutex m_mutex;
	type_uint32 m_u32CurrentFrameDataCount;		// 本帧数据个数
	type_uint32 m_u32StartId;					// 起始数据序号
	bool m_bSendData;                           // true:有数据等待发送，false：无数据等待发送
	type_uint32 m_u32SendDataId;				// 待发送数据项的序号
	ns_train::CPPTime m_tmLastSendAllData;      // 上次发送全部模拟量数据的时间
	std::string m_strFileName;                  // 客户端请求的文件名
	bool m_bSendFile;                           // true:有文件等待发送，false：无文件等待发送
	type_int32 m_n32FrameIndex;					// 数据帧序号,-1表示无效，从0开始为有效帧序号。
	type_int64 m_n64SendDataCount;				// 已发送的文件内容字节数
};

