﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: observer.h
\brief 观察者类定义。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/

#pragma once

#include "base_class.h"
#include "customtype.h"
#include "datavector.h"
#include "ksui_01_dll_export.h"

#include <list>

#include <QObject>

/**
* @brief  订阅者类定义
*/
class KSUI_01_DLL_API CSubscriber : public QObject {
    Q_OBJECT
public:
    CSubscriber();
    ~CSubscriber();
    /**
    * @brief  设置监听的模拟量数据项Id
    * @param[in] id 数据项Id
    * @return void
    */
    void setAnalogDataId(type_uint32 id) {
        m_id = id;
    }
    /**
    * @brief  获取监听的模拟量数据项Id
    * @return 数据项Id
    */
    type_uint32 getAnalogDataId() {
        return m_id;
    }
    /**
    * @brief  变化数据通知：模拟量
    * @param[in] idValue 数据项
    * @return void
    */
    void notifyChangedAnalog(const SData& idValue);
Q_SIGNALS:
    void sig_dataChanged(const SData& idValue);
private:
    type_uint32 m_id;
};

/**
* @brief  量测观察者类定义
*/
class KSUI_01_DLL_API CMeasurementObserver {
public:
    /**
    * @brief  单体对象访问接口
    * @return 单体对象的指针
    */
    static CMeasurementObserver* instance();
    
    /**
    * @brief  变化数据通知：模拟量
    * @param[in] idValue 数据项
    * @return void
    */
    void notifyChangedAnalog(const SData &idValue);

    /**
    * @brief  向订阅队列中添加订阅者
    * @param[in] pSubscriber 订阅者
    * @return 订阅队列尺寸
    */
    size_t addSubscriber(CSubscriber *pSubscriber);
    /**
    * @brief  从队列中删除订阅者
    * @param[in] pClientThread 客户端线程
    * @return 订阅队列尺寸
    */
    size_t removeSubscriber(CSubscriber * pSubscriber);
private:
    /**
    * @brief  构造函数，定义为私有的目的是防止他人使用该类构造对象。
    * @return 无
    */
    CMeasurementObserver();
    /**
    * @brief  拷贝构造函数，定义为私有的目的是防止编译器调用默认的拷贝构造
    *         函数隐式构造该类的对象。
    * @return 无
    */
    CMeasurementObserver(const CMeasurementObserver&);

    /**
    * @brief  赋值构造函数，定义为私有的目的是防止编译器调用默认的赋值构造
    *         函数隐式构造该类的对象。
    * @return 无
    */
    CMeasurementObserver& operator = (const CMeasurementObserver&);

    /**
    * @brief  析构函数，定义为私有的目的是防止他人用delete语句删除单体对象。
    * @return 无
    */
    virtual ~CMeasurementObserver() {}
private:
    ns_train::CPPMutex m_mutex;             // 保护数据成员的互斥锁
    std::list<SData> m_lstChangeAnalog;     // 变化数据队列：模拟量
    std::list<CSubscriber*> m_lstSubscribers;// 订阅者队列
};


