﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolObject.h
\brief  通信规约对象类定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ksui_01_dll.dll
*/
#pragma once

#include "protocolinterface.h"
#include "base/basedll//base_class.h"
#include "customtype.h"
#include "ksui_01_dll_export.h"

#include <list>

namespace ns_train {
	class CPPSerial;
}

class CCommandInterface;

class KSUI_01_DLL_API CProtocolObject : public CProtocolInterface {
public:
    enum EFRAME_STATUS {
        EFRAME_SYNCHRONIZATION = 0, // 搜索同步字
        EFRAME_GET_FRAMETYPE,       // 搜索帧类型
        EFRAME_GET_FRAMELENGTH,     // 获取帧长度
        EFRAME_PARSEDATA,           // 解析数据
    };
	/**
	* @brief  构造函数
	*/
	CProtocolObject();
	/**
	* @brief  析构函数
	*/
	~CProtocolObject();
	/**
	* @brief  设置串口通信对象
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void setSerialObject(ns_train::CPPSerial* pSerial);
	/**
	* @brief  获得串口通信对象
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual ns_train::CPPSerial*  getSerialObject() const;
	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	bool open();
	/**
	* @brief 串口是否已打开
	* @return true:已打开,false:未打开
	*/
	bool isOpen();
	/**
	* @brief 关闭串口
	* @return true:成功,false:失败
	*/
	bool close();
	/**
	* @brief 从串口中接收数据
	* @param[out] data 接收缓冲区
	* @param[in] cout 准备接收的字节数
	* @return 从串口中实际接收到的数据字节数
	*/
	int read(
		char* data,
		int count);
	/**
	* @brief 向串口中发送数据
	* @param[in] data 发送缓冲区
	* @param[in] cout 准备发送的字节数
	* @return 实际向串口中写入的字节数
	*/
	int write(
		char* data,
		int count);
	/**
	* @brief  读取指定长度的数据
	* @param[in] nAllData 待读取的字节总数，含已经收到的字节数。
	* @return true：成功，false：未读完
	*/
	bool readData(type_int32 nAllData);
	/**
	* @brief  将待发送数据进行组帧并发送
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void makeFrameAndSend() = 0;
	/**
	* @brief  接收数据并解帧。
	* @param[in] pSerial 串口对象。
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void recvAndParseFrame() = 0;
	/**
	* @brief  添加命令。
	* @param[in] pCommand 命令对象。
	* @return true:成功，false：失败。当写入串口失败时，将导致返回false。
	*/
	virtual void pushCommand(CCommandInterface* pCommand);
	/**
	* @brief  获取一个命令。
	* @return 命令队列中第一个对象，如果队列为空，则返回NULL。
	*/
	virtual CCommandInterface* takeCommand();
protected:
	/**
    * @brief  获取帧类型。
    * @return 无
    */
    virtual void parseFrame_GetFrameType() = 0;
    /**
    * @brief  搜索同步字
    * @return 无
    */
    virtual void parseFrame_Synchronization() = 0;
    /**
    * @brief  获取帧长度。
    * @return 无
    */
    virtual void parseFrame_GetFrameLength() = 0;
    /**
    * @brief  解析数据。
    * @return 无
    */
    virtual void parseFrame_Parse() = 0;
    /**
    * @brief  复位与接收数据相关的变量。
    * @return 无
    */
    virtual void reset_receive() = 0;
    /**
    * @brief  复位与发送数据相关的变量。
    * @return 无
    */
    virtual void reset_send() = 0;    
	/**
    * @brief  申请内存
    * @return 无
    */
    virtual void makeBuffer() = 0;	
	/**
    * @brief  释放申请的内存
    * @return 无
    */
    virtual void freeBuffer();	
	/**
    * @brief  从命令队列中取出一个命令并执行
    * @return 无
    */
    virtual void takeOneCommandAndExecute();	
	/**
    * @brief  重发
    * @return 无
    */
    virtual void reSend();	
	/**
    * @brief  重发次数超限
    * @return 无
    */
    virtual void reSendOverLimit() = 0;	
	/**
    * @brief  设置等待客户端Ack（确认帧）时间，以秒为单位。
	* @param[in] nWaitAckTime 等待客户端Ack（确认帧）的时间
    * @return 无
    */
	void setWaitAckTime(int nWaitAckTime) {
		m_nWaitAckTime = nWaitAckTime;
	}
	/**
	* @brief  获取等待客户端Ack（确认帧）时间，以秒为单位。
	* @param[in] nWaitAckTime 等待客户端Ack（确认帧的时间）
    * @return   等待客户端Ack（确认帧）时间，以秒为单位。
    */
	int waitAckTime() {
		return m_nWaitAckTime;
	}
protected:
	ns_train::CPPSerial* m_pSerial;				// 串口通信对象
	type_int32 m_nReceivedCount;				// 已经接收到的数据字节数
	EFRAME_STATUS m_eFrameStatus;               // 数据解帧状态
	type_uint8 m_byFrameType;					// 帧类型
	type_int32 m_nFrameLength;					// 本帧数据长度
	char *m_pReceiveBuf;                        // 接收缓冲区
	char *m_pSendBuf;                           // 发送缓冲区
	char *m_pReSendBuf;                         // 重发缓冲区
	type_int32 m_nResendLength;					// 重发数据长度
	type_int32 m_nResendCount;					// 重发次数
	bool m_bWaitAck;                            // true:需要等待客户端的Ack确认帧，false：无需等待客户端的Ack确认帧
	bool m_bResend;                             // 是否需要重发
	ns_train::CPPMutex m_mutex;					// 互斥锁，用来保护私有数据
	std::list<CCommandInterface*> m_listCommands;// 命令队列
	ns_train::CPPTime m_tmBeginWaitAck;			// 开始等待客户端Ack（确认帧）的时间
	int m_nWaitAckTime;							// 等待客户端Ack（确认帧）时间，以秒为单位。
};
