﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: base_api.h
\brief 通用dll接口定义头文件, 该DLL不依赖Qt。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10 
* please import basedll.dll
*/

#pragma once

#include "base_export.h"
#include <string>
#include <vector>
#include "customtype.h"

namespace ns_train {
    /**
    * @brief 用来演示DLL开发的类
    */
    class BASE_API CClassInNameSpace {
    public:
        CClassInNameSpace() {}
        ~CClassInNameSpace() {}
    public:
        void sayHello(void) {
        }
    };
    // == 程序执行相关 =============================================================================================
    /**
    * @brief 暂停，等待用户输入任意字符后继续。
    *
    * @return void
    */
    BASE_API void pauseOnce(void);

    // == 文件相关 =============================================================================================
    /**
    * @brief 获取指定path的字符串, 如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"
    *         接口内部负责:
    *        1. 将"\\"转换为"/"
    *         2. 自动将环境变量替换为实际路径，环境变量使用$XXX的格式，比如， 输入："$PROJECT_DEV_HOME/src"，输出："d:/xingdianketang/project/gui/src/"
    *
    * @param[in] strPath 指定路径
    * @return 文件名, 全路径
    */
    BASE_API std::string getPath(const std::string &strPath);    
    /**
    * @brief 获取指定path所在的全目录名, 如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx.yy"
    *         接口内部负责:
    *         1. 将"\\"转换为"/"
    *         2. 自动将环境变量替换为实际路径，环境变量使用$XXX的格式，比如， 输入："$PROJECT_DEV_HOME/src/a.txt"，输出："d:/xingdianketang/project/gui/src/"
    *
    * @param[in] strPath 指定路径
    * @return 文件名所在目录
    */
    BASE_API std::string getDirectory(const std::string &strPath);
    /**
    * @brief 获取指定文件名的名称，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx.yy"
    *         接口内部负责:
    *         1. 将"\\"转换为"/"
    *         2. 自动将环境变量替换为实际路径，环境变量使用$XXX的格式，比如， 输入："$PROJECT_DEV_HOME/src/a.txt"，输出："a.txt"
    *
    * @param[in] strFilePath 指定文件（全路径）
    * @return 文件名(a.txt)
    */
    BASE_API std::string getFileName(const std::string &strFilePath);

    /**
    * @brief 获取指定strDirectory的当前子目录名, 如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx/xxx/"
    *   接口内部负责:
    *   1. 将"\\"转换为"/"
    *   2. 自动将环境变量替换为实际路径，环境变量使用$XXX的格式，比如， 输入："$PROJECT_DEV_HOME/src/exchange"，输出："exchange"
    *
    * @param[in] strDirectory 指定路径
    * @return 当前子目录名
    */
    BASE_API std::string getNameOfDirectory(const std::string &strDirectory);

    /**
    * @brief 检查指定目录或文件是否存在, 如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx/xxx/"，比如："$PROJECT_DEV_HOME/src/exchange"
    *   接口内部负责:
    *   1. 将"\\"转换为"/"
    *   2. 自动将环境变量替换为实际路径
    *
    * @param[in] strDirectory 指定路径/文件
    * @return true:目录存在，false：目录不存在
    */
    BASE_API bool isFileExist(const std::string &strDirectory);
    /**
    * @brief 创建指定目录，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx/xxx/"，比如："$PROJECT_DEV_HOME/src/exchange"
    *
    * @param[in] strDirectory 指定路径
    * @return true:创建成功，false：创建失败
    */    
    BASE_API bool mkDir(const std::string &strDirectory);
    /**
    * @brief 删除指定目录，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx/xxx/"，比如："$PROJECT_DEV_HOME/src/exchange"
    *
    * @param[in] strDirectory 指定路径
    * @return true:删除成功，false：删除失败
    */    
    BASE_API bool rmDir(const std::string &strDirectory);
    /**
    * @brief 获取指定文件的尺寸，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx/xxx/xxx.yyy"，比如："$PROJECT_DEV_HOME/src/exchange/aaa.txt"
    *
    * @param[in] strFileName 指定文件
    * @return 文件的尺寸,如果文件不存在或访问失败，则返回-1；其他情况：返回文件的尺寸。
    */    
    BASE_API type_size getFileSize(const std::string &strFileName);

    /**
    * @brief 删除指定字符串首尾的空格
    *
    * @param[in] str 指定字符串
    * @return 已经将首尾空格删除的字符串
    */    
    BASE_API std::string &trim(std::string &str);

    /**
    * @brief 将指定字符串以指定分隔符分隔，分隔后的字符串保存到数组中
    *
    * @param[in] str 指定字符串
    * @param[out] tokens 字符串数组，用来保存分割后的字符串
    * @param[in] delimiters 分隔符
    * @return 得到的字符串个数
    */ 
    BASE_API size_t split(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters = " ");

    // 时间间隔(带毫秒)
    struct STimeGapMS {
        type_uint32	nSecond;
        type_uint16	nMSecond;
    };
    /**
    * @brief 获取当前时间，以毫秒为单位
    * @return
    */
    BASE_API STimeGapMS get_current_time_ms();
    /**
    * @brief 获取当前时间，以秒为单位
    * @return
    */
    BASE_API type_time_t get_current_time();
} // namespace ns_train
