﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: base_filehead.h
\brief 文件头定义类
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date : 2018
*/
#pragma once

#include <string>
#include "base_file.h"
#include "customtype.h"
#include "serialize.h"
namespace ns_train {

// 文件头类
class CFileHead {
public:
	CFileHead()	{
		m_nMainVersion = 0;
		m_nSubVersion = 0;
	}

	CFileHead(type_uint16 nMainVersion, type_uint16 nSubVersion)	{
		m_nMainVersion = nMainVersion;
		m_nSubVersion = nSubVersion;
	}

	// 是否有效
	bool isValid() const	{
		return (0 != m_nMainVersion) && (0 != m_nSubVersion);
	}
    // 设置主版本号
    void setMainVersion(type_uint16 uMainVersion) {
        m_nMainVersion = uMainVersion;
    }

    // 设置子版本号
    void setSubVersion(type_uint16 uSubVersion) {
        m_nSubVersion = uSubVersion;
    }
    // 获取主版本号
    type_uint16 getMainVersion() const	{
        return m_nMainVersion;
    }

    // 获取子版本号
    type_uint16 getSubVersion() const	{
        return m_nSubVersion;
    }
	// 当前文件(程序)版本号是否比传入的版本号旧
	bool isEarlierVersion(type_uint16 nMainVersion, type_uint16 nSubVersion) const	{
		if ((m_nMainVersion < nMainVersion)
			|| (m_nMainVersion == nMainVersion && m_nSubVersion < nSubVersion))	{
			return true;
		}
		else	{
			return false;
		}
	}
	// 当前文件版本号是否比传入的版本号新
	bool isLaterVersion(type_uint16 nMainVersion, type_uint16 nSubVersion) const	{
		if ((m_nMainVersion > nMainVersion)
			|| (m_nMainVersion == nMainVersion && m_nSubVersion >= nSubVersion))	{
			return true;
		}
		else {
			return false;
		}
	}
	// 当前文件主版本号是否比传入的主版本号新
	bool isLaterMainVersion(type_uint16 nMainVersion) const	{
		if (m_nMainVersion > nMainVersion)	{
			return true;
		}
		else	{
			return false;
		}
	}
	// 当前文件版本号是否与传入的版本号相同
	bool isSameVersion(type_uint16 nMainVersion, type_uint16 nSubVersion) const	{
		if ((m_nMainVersion == nMainVersion)
			&& (m_nSubVersion == nSubVersion)) {
			return true;
		}
		else	{
			return false;
		}
	}

	// 将版本号转化为string类型字符串，如版本1.0，转后为"1.0"
	std::string toStdString() const	{
		std::string str;
        char buf[256] = {'0'};
		sprintf(buf, "%d", m_nMainVersion);
		str = buf;
		str += ".";
		sprintf(buf, "%d", m_nSubVersion);
		str += buf;
		return str;
	}

	// 将string类型字符串转化为版本号，如字符串"1.0"，转后为版本1.0
	static CFileHead fromStdString(std::string str)	{
		CFileHead FileHead;
        std::string::size_type index = str.find('.');
		if (index > 0)	{
            FileHead.m_nMainVersion = atoi(str.substr(0, index).c_str());
            FileHead.m_nSubVersion = atoi(str.substr(index + 1, str.length() - index - 1).c_str());
		}
		else	{
			FileHead.m_nMainVersion = 0;
			FileHead.m_nSubVersion = 0;
		}
		return FileHead;
	}

	// 赋值构造
	CFileHead& operator = (const CFileHead& fileHead)	{
		if (this != &fileHead)	{
			this->m_nMainVersion = fileHead.m_nMainVersion;
			this->m_nSubVersion = fileHead.m_nSubVersion;
		}
		return *this;
	}
	// 判断两者是否相等
	bool operator == (const CFileHead& r)	{
		if ((this->m_nMainVersion == r.m_nMainVersion) && (this->m_nSubVersion == r.m_nSubVersion))		{
			return true;
		}
		else		{
			return false;
		}
	}


private:
	type_uint16	m_nMainVersion;		// 主版本号
	type_uint16	m_nSubVersion;		// 次版本号
};

// 序列化文件的自定义属性(二进制)
BASE_API CPPFile& operator<<(CPPFile& file, const CFileHead& attrs);
// 反序列化文件的自定义属性(二进制)
BASE_API CPPFile& operator>>(CPPFile& file, CFileHead& attrs);

//！读取文件的属性-二进制方式
/**
* @param[in] fileName	文件全路径名
* @param[out] fileHead	文件头信息
* @retval ESerializeCode
*/
BASE_API ESerializeCode readFileHeadFromBinary(const std::string& fileName, CFileHead& fileHead);

} // namespace ns_train
