﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: iniconfig.h
\brief ini格式配置文件定义头文件, 请注意：保存的配置文件采用GB2312编码。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
* please import basedll.dll
*/

#pragma once

#include <string>
#include <list>
#include <vector>
#include <map>

#include "base_export.h"
#include "customtype.h"
#include "base_file.h"

namespace ns_train {

// ini格式配置文件处理类。支持“;”开头的文本作为注释
///-----------------------------------------
class BASE_API CPPIniFile {
public:
    CPPIniFile();
	~CPPIniFile();

    /**
    * @brief 设置配置文件的文件名，使用全路径，如果使用环境变量，格式必须为:"$环境变量名/xxx/xxx"。
    * @param[in] strFileName 文件名
    * @return true:成功,false:失败
    */
	type_bool  setFileName(const std::string& strFileName);
  
    /**
    * @brief 读取type_bool类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_nDefault 默认值
    * @param[out] o_bRet true:成功, false:失败
    * @return 数据
    */
	type_bool  getBool(const std::string& strKey, const std::string& strSubKey, type_bool i_nDefault, type_bool* o_bRet=NULL);
    /**
    * @brief 读取type_int32类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_nDefault 默认值
    * @param[out] o_bRet true:成功, false:失败
    * @return 数据
    */
	type_int32  getInt32(const std::string& strKey, const std::string& strSubKey, type_int32 i_nDefault, type_bool* o_bRet=NULL);
    /**
    * @brief 读取浮点类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_fDefault 默认值
    * @param[out] o_bRet true:成功, false:失败
    * @return 数据
    */
	type_double  getReal(const std::string& strKey, const std::string& strSubKey, type_double i_fDefault, type_bool* o_bRet=NULL);
    /**
    * @brief 读取string类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] strDefault 默认值
    * @param[out] o_bRet true:成功, false:失败
    * @return 数据
    */
    std::string getString(const std::string& strKey, const std::string& strSubKey, const std::string& strDefault, type_bool* o_bRet=NULL);
    /**
    * @brief 设置type_bool类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_nValue 子键的值
     * @return true:成功, false:失败
    */	
    type_bool  setBool(const std::string& strKey, const std::string& strSubKey, type_bool i_nValue);
    /**
    * @brief 设置type_int32类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_nValue 子键的值
     * @return true:成功, false:失败
    */	
    type_bool  setInt32(const std::string& strKey, const std::string& strSubKey, type_int32 i_nValue);
    /**
    * @brief 设置浮点类型的键值
    * @param[in] strKey 主键
    * @param[in] strSubKey 子键
    * @param[in] i_fValue 子键的值
     * @return true:成功, false:失败
    */	
	type_bool  setReal(const std::string& strKey, const std::string& strSubKey, type_double i_fValue);
    /**
    * @brief 设置字符串类型的键值, 比如，配置文件内容如下:
    *           [config]
    *           x=xx
    *           y=yy
    *           z=zz
    * @param[in] strKey 主键=config
    * @param[in] strSubKey 子键=x
    * @param[in] strValue 子键的值
     * @return true:成功, false:失败
    */	
    type_bool  setString(const std::string& strKey, const std::string& strSubKey, const std::string& strValue);

private:
	type_bool writeToFile(void);

	type_bool getValueString(const std::string& strKey, const std::string& strSubKey, std::string& o_pValue);
	type_bool setValueString(const std::string& strKey, const std::string& strSubKey, const std::string& strValue);

private:
	CPPFile				m_file;    // 文件对象
	std::vector<std::string> m_vecKey;   // 主键
    std::map<std::string, std::list<std::string> >  m_mapKey2Subkey;// 主键到子键的映射
    std::map<std::string, std::string>  m_mapSubkey2Value;// 子键@主键 到 数值 的映射
	type_bool			m_bIsOpen;  // 配置文件是否已打开
};

}

