﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: service_api.h
\brief 通用dll接口定义头文件, 该DLL不依赖Qt。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
* please import basedll.dll
*/
#pragma once

#include "base_export.h"
namespace ns_train {
/**
* @brief 用来演示DLL开发的类
*/
class BASE_API CPrint {
public:
	CPrint(){}
	~CPrint(){}
public:
	void sayHello(void){
	}
};

/**
* @brief 让程序以服务方式运行
* @param[in] service_name 注册用的服务名，Windows用，Linux不用。
* @param[in] service_showname 显示用的服务名，Windows用，Linux不用。
* @return -1：创建守护程/服务失败，其他：成功
*/
BASE_API int api_start_as_service(const char* service_name, const char* service_showname);
/**
* @brief 注册服务接口。
* @param[in] service_name 注册用的服务名。
* @param[in] service_showname 显示用的服务名。
* @param[in] Service_Description 服务描述。
* @return true：成功，false：失败
*/
BASE_API bool regist(const char* service_name, const char* service_showname, const char* Service_Description);
/**
* @brief 注销服务接口。
* @param[in] service_name 待注销的服务名。
* @return true：成功，false：失败
*/
BASE_API bool unregist(const char* service_name);

/**
* @brief 睡眠
* @param[in] nMSecond  睡眠时间，以毫秒为单位
* @return void
*/
BASE_API void api_sleep(int nMSecond);

// 忽略信号，否则会导致进程在收到操作系统的这些信号时退出。
BASE_API void api_ignore_signal(void);

/**
* @brief 将当前线程运行在指定CPU
* @param[in] cpuId  指定CPU索引
* @return void
*/
BASE_API void api_setCPU(int cpuId);

}
