﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: com_serial.h
\brief 串口通讯类定义头文件, 该DLL不依赖Qt。
       使用方法：
		// s_serial_config config;
		// CSerialComm* comObject = new CSerialComm(config);
		// comObject->Open();
		// comObject->ReadData();
		// comObject->WriteData();
		// comObject->Close();
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
* please import comnunicate.dll
*/
#pragma once
#include "com_export.h"
#include "customtype.h"
namespace ns_train {
// 串口状态
enum ESERIAL_STATE {
	ESERIAL_OPEN = 0,	// 串口打开
	ESERIAL_CLOSE,		// 串口关闭
	ESERIAL_ERROR,		// 串口错误
	ESERIAL_STATE_MAX
};
const int c_nDeviceNameLength = 18;
// 串口通信参数定义
struct s_serial_config {
	s_serial_config();
	type_uint16 u32Baudrate;			// 波特率(串口通信速率)
	type_uint8  byDatabits;			// 数据位数, 如:5, 6, 7, 8
	type_char   chStopbit;			// 停止位, 0 or '0':1位, 1 or '1':1.5位, 2 or '2':2位
	type_char   chParity;			// 校验, 'N': 无, 'O': 奇校验, 'E': 偶校验
	type_char   strDevicename[c_nDeviceNameLength];	// 设备名, 如/dev/tty01,\\.\COM1等
};

class CSerialProxy; // 串口私有数据类
/**
* @brief 串行通讯类。
*			负责把数据从串行通道中接收并放到通道的原始数据区中。
*			根据通道波特率、数据位、终止位、同步异步等进行接收和发送。
*/
class COM_API CPPSerial {
public:
	/**
	* @brief 构造函数
	* @param[in] serialConfig 串口配置参数
	*/
	explicit CPPSerial(const s_serial_config& serialConfig);
	/**
	* @brief 析构函数
	*/
	virtual ~CPPSerial();
	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	bool open();
	/**
	* @brief 串口是否已打开
	* @return true:已打开,false:未打开
	*/
	bool isOpen();
	/**
	* @brief 关闭串口
	* @return true:成功,false:失败
	*/
	bool close();
	/**
	* @brief 从串口中接收数据
	* @param[out] data 接收缓冲区
	* @param[in] count 准备接收的字节数
	* @return 从串口中实际接收到的数据字节数
	*/
	int read(
		char* data,
		int count);
	/**
	* @brief 向串口中发送数据
	* @param[in] data 发送缓冲区
	* @param[in] count 准备发送的字节数
	* @return 实际向串口中写入的字节数
	*/
	int write(
		char* data,
		int count);
	/**
	* @brief 设置串口通信参数
	* @param[in] serialConfig 串口通信参数
	* @return true:设置成功，false:设置失败
	*/
	bool setSerialConfig(const s_serial_config& serialConfig);
	/**
	* @brief 获取串口通信参数
	* @return 串口通信参数
	*/
	s_serial_config getSerialConfig();
	/**
	* @brief 设置波特率
	* @param[in] baudrate 波特率
	* @return true:设置成功，false:设置失败
	*/
	bool setBaudrate(type_uint16 baudrate);
	/**
	* @brief 获取波特率
	* @return 波特率
	*/
	type_uint16 getBaudrate();
private:
	CSerialProxy* m_pData;
};

}
