﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: ksui_01_struct.h
\brief  通信数据帧结构定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/12
* please import ksui_01_dll.dll
*/
#pragma once

#include "customtype.h"

#include "ksui_01_dll_export.h"

static const int MAX_FILE_NAME_LENGTH = 64; // 文件名（不带路径）的最长尺寸
#include "prealign.h"

// 帧头结构体
struct KSUI_01_DLL_API ksui_01_FrameHeader {
    ksui_01_FrameHeader();
    type_uint8	byStart;			// 同步字, 0x68
    type_uint8	byFrameType;		// 帧类型
    type_int32  n32FrameIndex;      // 帧序号
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 确认帧
struct KSUI_01_DLL_API ksui_01_Client_Ack {
    ksui_01_Client_Ack();
    ksui_01_FrameHeader	header; 	// 帧头
    type_int32  n32FrameIndexAck;   // 被确认的帧序号
    type_uint8 checkSum;            // 校验码
    type_uint8 calculateChecksum(); // 计算校验码
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 模拟量数据结构体
struct KSUI_01_DLL_API ksui_01_AnalogData {
    ksui_01_AnalogData();
    type_uint32 uDataId;            // 数据项Id, 从0开始
    type_double dValue;             // 数据的值
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 客户端-设置数据命令结构体
struct KSUI_01_DLL_API ksui_01_Client_SetData {
    ksui_01_Client_SetData();
    ksui_01_FrameHeader	header; 	// 帧头
    ksui_01_AnalogData data;        // 模拟量数据
    type_uint8 checkSum;            // 校验码
    type_uint8 calculateChecksum(); // 计算校验码
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 客户端-获取数据命令结构体
struct KSUI_01_DLL_API ksui_01_Client_GetData {
    ksui_01_Client_GetData();
    ksui_01_FrameHeader	header; 	// 帧头
    type_uint32 uDataId;            // 数据项Id, 从0开始
    type_uint8 checkSum;            // 校验码
    type_uint8 calculateChecksum();       // 计算校验码
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 客户端-获取文件命令结构体
struct KSUI_01_DLL_API ksui_01_Client_GetFile {
    ksui_01_Client_GetFile();
    ksui_01_FrameHeader	header; 	// 帧头
    char szFileName[MAX_FILE_NAME_LENGTH]; // 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapterui/ksui_01
    type_uint8 checkSum;            // 校验码
    type_uint8 calculateChecksum();       // 计算校验码
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 服务器端-单个模拟量数结构体
struct KSUI_01_DLL_API ksui_01_Server_OneAnalogData {
    ksui_01_Server_OneAnalogData();
    ksui_01_FrameHeader	header; 	// 帧头
    ksui_01_AnalogData data;        // 模拟量数据
    type_uint8 checkSum;            // 校验码
    type_uint8 calculateChecksum();       // 计算校验码
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

// 服务器端-批量模拟量数结构体
struct KSUI_01_DLL_API ksui_01_Server_AnalogDatas {
    ksui_01_Server_AnalogDatas();
    ksui_01_FrameHeader	header; 	// 帧头
    type_uint32 u32StartDataId;     // 起始数据序号
    type_uint32 u32CurrentFrameDataCount;	// 本帧数据个数
    // ...                          // 数据
    //type_uint8 checkSum;          // 校验码
    type_uint8 calculateChecksum(type_int32 nFrameLength); // 计算校验码, nFrameLength为整帧数据的长度（含帧头、校验码）。
    type_uint8 getCheckSum(type_int32 nFrameLength); // 获取本帧数据中的校验码, nFrameLength为整帧数据的长度（含帧头、校验码）。
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;
// 服务器端-批量模拟量数结构体
struct KSUI_01_DLL_API ksui_01_Server_File {
    ksui_01_Server_File();
    ksui_01_FrameHeader	header; 	// 帧头
    type_uint32 u32FrameLength;     // 本帧数据总长度
    char szFileName[MAX_FILE_NAME_LENGTH];// 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapterui/ksui_01
    type_uint8  byLastFrame;        // 是否最后一帧
    type_uint64 u64Offset;          // 本帧数据在文件中的起始偏移量
    type_uint32 u32DataLength;      // 本帧文件内容长度
    //type_uint8* pData;            // 文件内容
    //type_uint8 checkSum;          // 校验码
    type_uint8 calculateChecksum(type_int32 nFrameLength); // 计算校验码, nFrameLength为整帧数据的长度（含帧头、校验码）。
    type_uint8 getCheckSum(type_int32 nFrameLength); // 获取本帧数据中的校验码, nFrameLength为整帧数据的长度（含帧头、校验码）。
    void NetToHost();       		// 网络序转到主机序
    void HostToNet();       		// 主机序转到网络序
} DISALIGN;

#include "postalign.h"
