﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: base_api.cpp
\brief	
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9 
*/

#include "base_api.h"
#include <algorithm>
#include <iostream>
#include <vector>
#include "customtype.h"

#ifdef WIN32
#include <io.h>
#include <stdio.h>
#else
#include <unistd.h>
#endif
//#include <sys/types.h>
#include <sys/stat.h>

namespace ns_train {
/////////////////////////////////////////////////////////////////////
void pauseOnce(void) {
    char ch = '\0';
    std::cin >> ch;
}
std::string getPath(const std::string &strInputPath) {
    std::string strPath = strInputPath;
    size_t idx = strPath.find('$');
    if (idx == ((size_t)-1)) {
        return strPath;
    }
    std::replace(strPath.begin(), strPath.end(), '\\', '/');
    size_t idx2 = strPath.find("/"); // 找到第1个'/'，这应该是环境变量后的第一个'/'。
    std::string strEnvironmentVariable = strPath.substr(idx+1, idx2-(idx+1));
    std::string strRight = strPath.substr(idx2, strPath.length()-idx2); // 删掉环境变量字符串，只保留后面剩下的字符。
    char * pStr = getenv(strEnvironmentVariable.c_str());
    if (NULL == pStr) {
        return "";
    }
    strEnvironmentVariable = pStr;
    std::replace(strEnvironmentVariable.begin(), strEnvironmentVariable.end(), '\\', '/'); // 替换环境变量中的'\\'。
    if (strEnvironmentVariable.substr(strEnvironmentVariable.length() - 1, 1) == "/") { // 删除环境变量中最后的'/'
        strEnvironmentVariable = strEnvironmentVariable.substr(0, strEnvironmentVariable.length() - 1);
    }
    strPath = strEnvironmentVariable + strRight;
#ifdef WIN32
    std::replace(strPath.begin(), strPath.end(), '/', '\\');
#endif
    return strPath;
}

std::string getFileName(const std::string &strFilePath) {
    std::string strFileName = getPath(strFilePath);
#ifdef WIN32
    char ch = '\\';
#else
    char ch = '/';
#endif //
    std::string::size_type idx = strFileName.find_last_of(ch);
    if (idx > 0) {
        strFileName = strFileName.substr(idx+1, strFileName.length() - idx - 1);
    }
    return strFileName;
}

std::string getDirectory(const std::string &strPath) {
    std::string strDirectory = getPath(strPath);
    if (0 == strDirectory.length()) {
        return "";
    }
#ifdef WIN32
    char ch = '\\';
#else
    char ch = '/';
#endif //
    size_t idx = strDirectory.find_last_of(ch);
    if (idx == ((size_t)-1)) {
        return strDirectory;
    }
    strDirectory = strDirectory.substr(0, idx);
    return strDirectory;
}

std::string getNameOfDirectory(const std::string &strDirectory) {
    std::string strDirName = getPath(strDirectory);
#ifdef WIN32
    char ch = '\\';
#else
    char ch = '/';
#endif //
    if (strDirName.length() > 1) {
        if (strDirName[strDirName.length() - 1] == ch) {
            strDirName = strDirName.substr(0, strDirName.length() - 1);
        }
    }
    size_t idx = strDirName.find_last_of(ch);
    if (idx == ((size_t)-1)) {
        return std::string();
    }
    else {
        strDirName = strDirName.substr(idx, strDirName.length() - idx - 1);
        return strDirName;
    }
}

bool isFileExist(const std::string &strFile) {
    std::string tStr = getPath(strFile);
    if (access(tStr.c_str(), 0) == -1) {
        return false;
    }
    return true;
}

bool mkDir(const std::string & strDirectory) {
    if (strDirectory.length() == 0)
        return false;
    std::string tPath = getPath(strDirectory);
    if (isFileExist(tPath))
        return true;

#ifdef WIN32
    std::string tStr = "md "; 
    std::replace(tPath.begin(), tPath.end(), '/', '\\');
#else
    std::string tStr = "mkdir -p "; // 级联目录创建的时候需要添加-p参数，单级目录创建也可以使用-p参数。
#endif
    tStr += tPath;
    if (system(tStr.c_str()) > 0) {
        return true;
    }
    return false;
}

//  删除目录(包含子目录)
bool rmDir(const std::string &strDirectory) {
    if (strDirectory.length() == 0) {
        return false;
    }
    std::string tPath = getPath(strDirectory);
    if (!isFileExist(tPath))
        return true;
#ifdef WIN32
    std::string tStr = "rd /S /Q ";
#else
    std::string tStr = "rm -rf ";
#endif
    tStr += tPath;
    if (system(tStr.c_str()) > 0) {
        return true;
    }
    return false;
}

type_size getFileSize(const std::string &strFileName) {
    struct stat buf;
    if (0 == strFileName.length()) {
        return ((size_t)-1);
    }
    std::string strTmpFileName = getPath(strFileName);
    if (!isFileExist(strTmpFileName)) {
        return ((size_t)-1);
    }
    if (-1 == stat(strTmpFileName.c_str(), &buf)) {
        return ((size_t)-1);
    }
    return buf.st_size;
}

std::string& trim(std::string& str) {
    if (str.empty())  {
        return str;
    }
    str.erase(0, str.find_first_not_of(" "));
    str.erase(str.find_last_not_of(" ") + 1);
    return str;
}

size_t split(const std::string& str, std::vector<std::string> &tokens, const std::string &delimiters) {
    std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    std::string::size_type pos = str.find_first_of(delimiters, lastPos);
    while (std::string::npos != pos || std::string::npos != lastPos) {
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        lastPos = str.find_first_not_of(delimiters, pos);
        pos = str.find_first_not_of(delimiters, lastPos);
    }
    return tokens.size();
}
} // namespace ns_train
