﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: api_linux.cpp
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/
#include "base_api.h"
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/fcntl.h>
#include <sys/mman.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include "base_class.h"
namespace ns_train {
int api_start_as_service(const char* /*service_name*/, const char* /*service_showname*/) {

	// aix下用nohup方式运行即可，且无屏幕输出
#if defined(AIX)
	return 0;
#endif
	pid_t processId;
	/* 创建子进程，目的是脱离控制终端 */	
	processId = fork();
	if (processId < 0) {				// 分离进程，创建一个子进程。
		return (-1);					// 如果分离失败，则返回错误。
	}
	else if (processId > 0) {
		exit(0); 						// 关闭父进程
	}
	/* 至此，第一子进程创建完毕，可以准备脱离控制终端 */

	/*
	* 下面将第一子进程成为新的会话组长和进程组长, 并与控制终端分离。
	* setsid()后子进程不受父进程的终端影响，父进程的终端退出，不影响子进程。
	*/
	setsid(); 
	/*
	* 对于非服务方式运行，至创建一次子进程即可。
	* 若无界面，服务方式运行，须再创建子进程，脱离会话组长。
	*/
	{
		processId = fork();
		if (processId < 0) {			// 分离进程，即第二次创建子进程。
			return (-1);				// 分离失败，返回错误
		}
		else if (processId > 0) {
			exit(0); 					// 分离成功，关闭第一次创建的子进程
		}
	}
	/* 至此，已经是真正的后台子进程 */
	/* hpux的后台处理问题*/
#ifndef HPUX	
	/* 
	* 至此是第二子进程，第二子进程不再是会话组长。
	* 关闭已打开的文件描述符，
	* 用fork（）新建的子进程会从父进程那里继承一些已经打开了的文件。
	* 这些被打开的文件可能永远不会被守护进程读或写，但它们一样消耗系统资源，
	* 可能导致所在的文件系统无法卸载。
	*/
	int nFileNum = NOFILE < 64 ? 64 : NOFILE;
	for (int i=0; i<nFileNum; i++)
		close(i);
#endif
	/*
	* 脱离终端窗口后必须重定向标准输入输出，不能简单关闭，
	* 否则程序在调用printf之类函数时会出现严重错误，可能导致进程崩溃。
	* 因为前面已将所有文件描述符关闭，所以此处依次打开stdin, stdout, stderr。
	* 将标准输入stdin、标准输出stdout、标准错误stderr重定向为：/dev/null。 
	*/
	open("/dev/null", O_RDONLY);
	open("/dev/null", O_RDWR);
	open("/dev/null", O_RDWR);
	// -------------------------------------------------------------<
	chdir("/");	// 改变工作目录更改目录防止占用可卸载的文件系统，
				// 也可以换成其他路径。
	/*
	* linux中的 umask 函数主要用于：
	* 在创建新文件或目录时 屏蔽掉新文件或目录不应有的访问允许权限。
	* 文件的访问允许权限共有9种，分别是：r w x r w x r w x
	* （它们分别代表：用户读 用户写 用户执行 组读 组写 组执行 其它读 其它写 其它执行）。
	* 其实这个函数的作用，就是设置允许当前进程创建文件或者目录最大可操作的权限，
	* 比如这里设置为0，它的意思就是0取反再创建文件时权限相与，
	* 也就是：(~0) & mode 等于八进制的值0777 & mode了，
	* 这样就是给后面的代码调用函数mkdir给出最大的权限，
	* 避免了创建目录或文件的权限不确定性。
	*/
	umask(0);				// 重设文件创建掩码
	return 0;
}

void api_sleep(int nMSecond) {
	if (nMSecond < 0) {
		return;
	}

	if (nMSecond >= 1000) {
		sleep(nMSecond / 1000);
		usleep((nMSecond % 1000) * 1000);
	}
	else {
		usleep(nMSecond * 1000);
	}
}

void api_ignore_signal(void) {
    signal(SIGINT, SIG_IGN);			// interrupt
    signal(SIGILL, SIG_IGN);			// illegal instruction - invalid function image
    signal(SIGFPE, SIG_IGN);			// floating point exception
	signal(SIGSEGV, SIG_IGN);			// segment violation
	signal(SIGTERM, SIG_IGN);			// Software termination signal from kill
	signal(SIGABRT, SIG_IGN);			// abnormal termination triggered by abort call
	signal(SIGALRM, SIG_IGN);			// 忽略时钟报警信号

	struct sigaction act;
	memset(&act, 0, sizeof(act));
	act.sa_handler = SIG_IGN;
	sigemptyset(&(act.sa_mask));
#	if defined(SA_ONESHOT)
	act.sa_flags |= SA_ONESHOT;
#	elif defined(SA_RESETHAND)
	act.sa_flags |= SA_RESETHAND;
#	endif

#	ifdef SIGPIPE
	sigaction( SIGPIPE, &act, 0 );
#	endif
	sigaction( SIGABRT, &act, 0 );

	// 如果缓存文件>2G会出现该信号导致进程退出
#ifdef SIGXFSZ 
	sigaction( SIGXFSZ, &act, 0 ); /* File size limit exceeded (4.2 BSD).  */
#endif	

	return;
}

bool regist(const char* service_name, const char* service_showname, const char* service_description) {
	return true;
}
bool unregist(const char* service_name) {
	return true;
}
CPPMutex g_mutexTime;
type_time_t get_current_time() {
    g_mutexTime.lock();
    type_time_t tmNow = time(NULL);
    g_mutexTime.unlock();
    return tmNow;
}
STimeGapMS get_current_time_ms() {
    STimeGapMS tIntervalTime;
    timeval	tTimeVal;
    gettimeofday(&tTimeVal, NULL);
    tIntervalTime.nSecond = tTimeVal.tv_sec;
    tIntervalTime.nMSecond = tTimeVal.tv_usec/1000;
    return tIntervalTime;
}
void api_setCPU(int cpuId) {
	cpu_set_t mask;	
	CPU_ZERO(&mask);	
	CPU_SET(cpuId, &mask);
	pthread_setaffinity_np(pthread_self(), sizeof(mask), &mask);
}
}
