﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: file.cpp
\brief 通用dll接口定义头文件, 该DLL不依赖Qt。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
* please import basedll.dll
*/

#include "base_file.h"
#include <assert.h>
#include <string.h> // C语言的头文件
#include <fstream>
#include "base_api.h"
#include <sys/stat.h>
#ifndef WIN32
	#include <unistd.h>
#else
	#include <io.h>
#endif
#include "host_network_api.h"


namespace ns_train {
CPPFile::CPPFile() {
	m_pFile = NULL;
}

CPPFile::CPPFile(const char* szFileName) {
	m_pFile = NULL;
	m_strFileName = szFileName;
}

CPPFile::~CPPFile() {
	close();
}

bool CPPFile::setFileName(std::string strFileName) {
	if (0 == strFileName.length()) {
		return false;
	}
	m_strFileName = getPath(strFileName.c_str());
	return true;
}

std::string CPPFile::getFileName(void) const {
	return m_strFileName;
}


bool CPPFile::open(const char* szMode) {
#ifdef WIN32
	m_pFile = ::fopen(m_strFileName.c_str(), szMode);
#else
	m_pFile = ::fopen64(m_strFileName.c_str(), szMode);
#endif    
	if (NULL == m_pFile) {
		return false;
	}
	return true;
}

type_size CPPFile::read(char* pBuf, type_size nCount) {
	if (m_pFile == NULL) {
		return -1;
	}
	type_size ret = ::fread(pBuf, 1, nCount, m_pFile);
	return (int)ret;
}

type_size CPPFile::write(const char* pBuf, type_size nCount) {
	if (m_pFile == NULL)
		return -1;
	type_size ret = ::fwrite(pBuf, 1, nCount, m_pFile);
	return ret;
}

bool CPPFile::readLine(char* pBuf, int nMaxCount) {
	if (m_pFile == NULL) {
		return false;
	}
	char* szRet = ::fgets(pBuf, nMaxCount, m_pFile);
	if (szRet == NULL) {
		return false;
	}
	return true;
}

void CPPFile::writeLine(const char* pBuf) {
	if (m_pFile == 0) {
		return;
	}
	::fputs(pBuf, m_pFile);
	::fputs("\n", m_pFile);
}

void CPPFile::close(void) {
	if (NULL != m_pFile) {
		::fclose(m_pFile);
		m_pFile = NULL;
	}
}

//文件是否存在
bool CPPFile::isFileExist(void) const {
	if (::access(m_strFileName.c_str(), 0) == -1) {
		return false;
	}
	return true;
}
//文件是否存在
bool CPPFile::isFileExist(std::string strFileName) {
	std::string strTmpFileName = getPath(strFileName.c_str());
	if (::access(strTmpFileName.c_str(), 0) == -1) {
		return false;
	}
	return true;
}
type_size CPPFile::getSize(void) {
#ifdef WIN32
	long len = 0, oldPos = 0;
#else
	off_t len = 0, oldPos = 0;
#endif   
	if (m_pFile != NULL) { // 如果文件已经打开，则先保存游标的当前位置，计算完文件尺寸后，应恢复游标位置。
#ifdef WIN32
		oldPos = ::ftell(m_pFile);			// 保存文件游标的当前位置
		if (::fseek(m_pFile, 0, SEEK_END) != 0) {
			return -1;
		}
		len = ::ftell(m_pFile);				// 利用文件末尾游标位置计算文件长度
		::fseek(m_pFile, oldPos, SEEK_SET);	// 将游标恢复到之前的位置
#else
	//unix下使用_LARGEFILE_SOURCE以支持大文件的访问，对于>2G文件需要使用ftelllo，fseeko这样的函数，不过这些函数不是标准的POSIX
	//函数必须有_LARGEFILE_SOURCE这样的定义，此定义统一增加到了$IESBASE/ies_base.pri下了
		oldPos = ftello(m_pFile);		//获取原先指针位置
		if (fseeko(m_pFile, 0, SEEK_END) != 0)
			return -1;
		len = ftello(m_pFile);
		fseeko(m_pFile, oldPos, SEEK_SET);
#endif
	}
	else {

	}
	return len;
}
type_size CPPFile::getSize(std::string strFileName) {
	struct stat buf;
	if (0 == strFileName.length()) {
		return ((size_t)-1);
	}
	std::string strTmpFileName = getPath(strFileName);
	if (!isFileExist(strTmpFileName)) {
		return ((size_t)-1);
	}
	if (-1 == stat(strTmpFileName.c_str(), &buf)) {
		return ((size_t)-1);
	}
	return buf.st_size;
}

type_size CPPFile::seekToPosition(type_size nOffset, int nFrom) {
	switch (nFrom) {
	case SEEK_SET:
	case SEEK_END:
	case SEEK_CUR:
		break;
	default:
		return -1;
	}
#ifdef WIN32
	long offset = static_cast<long>(nOffset);
	int nPos = ::fseek(m_pFile, offset, nFrom);
	if (nPos != 0) {
		return nPos;
	}
#else
	int nPos = ::fseeko(m_pFile, nOffset, nFrom);
	if (nPos != 0) {
		return nPos;
	}
#endif
	return currentPosition();
}
type_size CPPFile::currentPosition(void) {
	if (NULL == m_pFile) {
		return 0;
	}
	fpos_t cpos;
	memset(&cpos, 0, sizeof(cpos));
	::fgetpos(m_pFile, &cpos);
#ifdef WIN32
	return (type_size)cpos;
#else
	return (type_size)cpos.__pos;
#endif
}

void CPPFile::seekToBegin(void) {
	::rewind(m_pFile);
}

void CPPFile::seekToEnd(void) {
	seekToPosition(0, SEEK_END);
}

bool CPPFile::copy(const char* szSrc, const char* szDst) {
	std::string strSrcFileName = getPath(szSrc);
	std::string strDstFileName = getPath(szDst);

	std::ifstream fin(strSrcFileName, std::ios::in);
	std::ofstream fout(strDstFileName, std::ios::out | std::ios::trunc);
	if (fin && fout) {
		fout << fin.rdbuf();// rdbuf返回streambuf*, 速度比迭代器拷贝（fread,fwrite）快太多
	   // fout.flush();
		fout.close();
		fin.close(); 
		return true;
	}
	else {
		fout.close();
		fin.close(); 
		return false;
	}
}
bool CPPFile::isEOF(void) const {
	return ::feof(m_pFile);
}
bool CPPFile::rename(const char* szNewName) {
	if (NULL == m_pFile) {
		::fclose(m_pFile);
	}
	std::string strTmpFileName = getPath(szNewName);
	if (::rename(m_strFileName.c_str(), strTmpFileName.c_str()) == 0) {
		m_strFileName = strTmpFileName;
		return true;
	}
	return false;
}

bool CPPFile::remove(void) {
	if (NULL == m_pFile) {
		::fclose(m_pFile);
	}
	if (::remove(m_strFileName.c_str()) == 0) {
		m_strFileName.clear();
		return true;
	}
	return false;
}
CPPFile& CPPFile::operator<<(const type_char& data) {
	type_char d = data;
	write((char*)&d, sizeof(data));
	return *this;
}
CPPFile& CPPFile::operator<<(const type_wchar& data) {
	type_wchar d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_int8& data) {
	type_int8 d = data;
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_uint8& data) {
	type_uint8 d = data;
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_int16& data) {
	type_int16 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_uint16& data) {
	type_uint16 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}
CPPFile& CPPFile::operator<<(const type_int32& data) {
	type_int32 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_uint32& data) {
	type_uint32 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}

CPPFile& CPPFile::operator<<(const type_int64& data) {
	type_int64 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}
CPPFile& CPPFile::operator<<(const type_uint64& data) {
	type_uint64 d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}
CPPFile& CPPFile::operator<<(const type_float& data) {
	type_float d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}
CPPFile& CPPFile::operator<<(const type_double& data) {
	type_double d = data;
	HostToNetwork(&d, sizeof(d)); // 需要转换字节序
	write((char*)&d, sizeof(d));
	return *this;
}
CPPFile& CPPFile::operator<<(const std::string& data) {
	type_uint64 len = data.length();
	HostToNetwork(&len, sizeof(len));
	write((char*)&len, sizeof(len));
	write((char*)data.c_str(), data.length());
	return *this;
}

CPPFile& CPPFile::operator>>(type_char& data) {
	read((char*)&data, sizeof(data));
	return *this;
}
CPPFile& CPPFile::operator>>(type_wchar& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}

CPPFile& CPPFile::operator>>(type_int8& data) {
	read((char*)&data, sizeof(data));
	return *this;
}

CPPFile& CPPFile::operator>>(type_uint8& data) {
	read((char*)&data, sizeof(data));
	return *this;
}

CPPFile& CPPFile::operator>>(type_int16& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}

CPPFile& CPPFile::operator>>(type_uint16& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}
CPPFile& CPPFile::operator>>(type_int32& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}

CPPFile& CPPFile::operator>>(type_uint32& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}

CPPFile& CPPFile::operator>>(type_int64& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}
CPPFile& CPPFile::operator>>(type_uint64& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}
CPPFile& CPPFile::operator>>(type_float& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}
CPPFile& CPPFile::operator>>(type_double& data) {
	read((char*)&data, sizeof(data));
	NetworkToHost(&data, sizeof(data)); // 需要转换字节序
	return *this;
}
CPPFile& CPPFile::operator>>(std::string& data) {
	type_uint64 len = 0;
	read((char*)&len, sizeof(len));
	NetworkToHost(&len, sizeof(len));
	char* buf = new char[len + 1];
	memset(buf, 0, len + 1);
	read(buf, len);
	data = buf;
	delete buf;
	buf = NULL;
	return *this;
}

}
