﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: host_network_api.cpp
\brief C++跨平台数据类型定义
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/02 
*/
#include "host_network_api.h"

///////////////////////////////////////////////////////////////////////////////
namespace ns_train {
void
NetworkToHost(void* Data,			// 输入需要转换的数据，输出转换后的数据
		type_uint16 DataLen) {		// 输入需要转换数据长度
#if defined(CPP_LITTLE_ENDIAN)
	type_uint8 tmp;
	type_uint8* tmpData = (type_uint8*)Data;
	for (int i = 0; i < DataLen / 2; i++) {
		tmp = tmpData[i];
		tmpData[i] = tmpData[DataLen - i - 1];
		tmpData[DataLen - i - 1] = tmp;
	}
#endif
}

type_uint8
NetworkToHost(type_uint8 Data)	{	// 输入需要转换的数据，输出转换后的数据
	return Data;
}

type_uint16							// 返回转换后的数据
NetworkToHost(type_uint16 Data)	{	// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_uint16));
#endif
	return Data;
}

type_int16							// 返回转换后的数据
NetworkToHost(type_int16 Data) {	// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_int16));
#endif
	return Data;
}

type_uint32							// 返回转换后的数据
NetworkToHost(type_uint32 Data)	{	// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_uint32));
#endif
	return Data;
}

type_int32							// 返回转换后的数据
NetworkToHost(type_int32 Data){		// 输入需要转换的数据	
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_int32));
#endif
	return Data;
}

type_float							// 返回转换后的数据
NetworkToHost(type_float Data){		// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN) 
	NetworkToHost(&Data, sizeof(type_float));
#endif
	return Data;
}

type_double							// 返回转换后的数据
NetworkToHost(type_double Data){	// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_double));
#endif
	return Data;
}

type_time_t							// 返回转换后的数据
NetworkToHost(type_time_t Data){	// 输入需要转换的数据
#if defined(CPP_LITTLE_ENDIAN)
	NetworkToHost(&Data, sizeof(type_time_t));
#endif
	return Data;
}

void
HostToNetwork(void* Data,			// 输入需要转换的数据，输出转换后的数据
	type_uint16 DataLen){			// 输入需要转换数据长度
	NetworkToHost(Data, DataLen);
}

type_uint8
HostToNetwork(type_uint8 Data)	{	// 输入需要转换的数据，输出转换后的数据
	return Data;
}

type_uint16							// 返回转换后的数据
HostToNetwork(type_uint16 Data)	{	// 输入需要转换的数据

	return NetworkToHost(Data);
}

//  type_int16字节序转换
type_int16							// 返回转换后的数据
HostToNetwork(type_int16 Data) {	// 输入需要转换的数据
	return NetworkToHost(Data);
}

//  type_uint32字节序转换
type_uint32							// 返回转换后的数据
HostToNetwork(type_uint32 Data)	{	// 输入需要转换的数据
	return NetworkToHost(Data);
}

type_int32							// 返回转换后的数据
HostToNetwork(type_int32 Data) {	// 输入需要转换的数据
	return NetworkToHost(Data);
}

type_float							// 返回转换后的数据
HostToNetwork(type_float Data) {	// 输入需要转换的数据
	return NetworkToHost(Data);
}

type_double							// 返回转换后的数据
HostToNetwork(type_double Data) {	// 输入需要转换的数据
	return NetworkToHost(Data);
}

type_time_t							// 返回转换后的数据
HostToNetwork(type_time_t Data){	// 输入需要转换的数据
	return NetworkToHost(Data);
}

type_uint8
LowByte(type_uint16 data) {
	type_uint8* ptmpdata = (type_uint8*)&data;
#if defined(CPP_LITTLE_ENDIAN)
	return ptmpdata[1];
#else
	return ptmpdata[0];
#endif
}

//  获取type_uint16的高字节
type_uint8
HighByte(type_uint16 data) {
	type_uint8* ptmpdata = (type_uint8*)&data;
#if defined(CPP_LITTLE_ENDIAN)
	return ptmpdata[0];
#else
	return ptmpdata[1];
#endif
}

type_uint16
LowWord(type_uint32 data) {
	type_uint16* ptmpdata = (type_uint16*)&data;
#if defined(CPP_LITTLE_ENDIAN)
	return ptmpdata[1];
#else
	return ptmpdata[0];
#endif
}

type_uint16
HighWord(type_uint32 data) {
	type_uint16* ptmpdata = (type_uint16*)&data;
#ifdef CPP_LITTLE_ENDIAN
	return ptmpdata[0];
#else
	return ptmpdata[1];
#endif
}

type_uint16
MakeWord(type_uint8 bylobyte, type_uint8 byhibyte) {
	type_uint16 tmpdata;
	type_uint8* ptmpdata = (type_uint8*)&tmpdata;
#ifdef CPP_LITTLE_ENDIAN
	ptmpdata[1] = bylobyte;
	ptmpdata[0] = byhibyte;
#else
	ptmpdata[0] = bylobyte;
	ptmpdata[1] = byhibyte;
#endif
	return tmpdata;
}

type_uint32
MakeDWord(type_uint16 wloword, type_uint16 whiword) {
	type_uint32 tmpdata;
	type_uint16* ptmpdata = (type_uint16*)&tmpdata;
#ifdef CPP_LITTLE_ENDIAN
	ptmpdata[1] = wloword;
	ptmpdata[0] = whiword;
#else
	ptmpdata[0] = wloword;
	ptmpdata[1] = whiword;
#endif
	return tmpdata;
}

type_uint32
MakeDWord(type_uint8 byte0, type_uint8 byte1, type_uint8 byte2, type_uint8 byte3) {
	type_uint32 tmpdata;
	type_uint8* ptmpdata = (type_uint8*)&tmpdata;
#ifdef CPP_LITTLE_ENDIAN
	ptmpdata[3] = byte0;
	ptmpdata[2] = byte1;
	ptmpdata[1] = byte2;
	ptmpdata[0] = byte3;
#else
	ptmpdata[0] = byte0;
	ptmpdata[1] = byte1;
	ptmpdata[2] = byte2;
	ptmpdata[3] = byte3;
#endif
	return tmpdata;
}

}