﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: service_windows.cpp
\brief 注册服务的实现文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/

#include "service_windows.h"
#include "service_api.h"
#include <winbase.h> 
#include <stdio.h> 
#include <string.h> 
#pragma comment (lib,"Advapi32.lib")
#define USERCONTROL_STOP			130
namespace ns_train {
typedef BOOL (WINAPI *LP_CHANGESERVICECONFIG_ROUTINE)(SC_HANDLE, DWORD, LPVOID);
SERVICE_STATUS          ServiceStatus;
SERVICE_STATUS_HANDLE   ServiceStatusHandle;
DWORD dwThreadID = 0;

void WINAPI ServiceCtrlHandler(DWORD opcode);
DWORD ServiceInitialization(DWORD argc, LPTSTR* argv, DWORD* specificError);
/**
* @brief 内部启动服务接口，该接口由start_service（）调用。
* @param[in] argc 进程启动时的参数个数，main（）函数中的argc。
* @param[in] argv 进程启动时的参数列表，main（）函数中的argv。。
* @return void
*/
void WINAPI ServiceStart(DWORD argc, LPTSTR* argv);
/**
* @brief 是否已注册
* @param[in] argc 进程启动时的参数个数，main（）函数中的argc。
* @param[in] argv 进程启动时的参数列表，main（）函数中的argv。。
* @return void
*/
bool IsRegistered();
LPSTR SERVICE_NAME = NULL;
LPSTR SERVICE_SHOWNAME = NULL;
////////////////////////////////////////////////////////////////
bool start_service(const char* service_name, const char* service_showname) {
	SERVICE_NAME = LPSTR(service_name);
	SERVICE_SHOWNAME = LPSTR(service_showname);
	SERVICE_TABLE_ENTRY DispatchTable[] = {
	{SERVICE_NAME, ServiceStart},
	{NULL, NULL}
	};

	if (!::StartServiceCtrlDispatcher(DispatchTable)) {
		//char szInfo[128];
		//sprintf(szInfo, " [%s] StartServiceCtrlDispatcher error = %d!", SERVICE_NAME, GetLastError());
		return false;
	}
	else {
		return true;
	}
}
void WINAPI ServiceStart (DWORD argc, LPTSTR *argv) {
	DWORD status; 
	DWORD specificError; 
	//char szInfo[256];

	dwThreadID = GetCurrentThreadId ();
	ServiceStatus.dwServiceType        = SERVICE_WIN32; 
	ServiceStatus.dwCurrentState       = SERVICE_START_PENDING; 
	ServiceStatus.dwControlsAccepted   = SERVICE_ACCEPT_STOP;
	ServiceStatus.dwWin32ExitCode      = 0; 
	ServiceStatus.dwServiceSpecificExitCode = 0; 
	ServiceStatus.dwCheckPoint         = 0; 
	ServiceStatus.dwWaitHint           = 0; 

	ServiceStatusHandle = RegisterServiceCtrlHandler( 
		SERVICE_NAME, 
		ServiceCtrlHandler); 

	if (ServiceStatusHandle == (SERVICE_STATUS_HANDLE)0) { 
		//sprintf(szInfo,"注册 [%s] 服务句柄失败! 错误码: %d!", SERVICE_NAME, GetLastError()); 
		return; 
	} 

	// Initialization code goes here. 
	status = ServiceInitialization(argc,argv, &specificError); 

	// Handle error condition 
	if (status != NO_ERROR) { 
		ServiceStatus.dwCurrentState       = SERVICE_STOPPED; 
		ServiceStatus.dwCheckPoint         = 0; 
		ServiceStatus.dwWaitHint           = 0; 
		ServiceStatus.dwWin32ExitCode      = status; 
		ServiceStatus.dwServiceSpecificExitCode = specificError; 
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus); 
		return; 
	} 

	// Initialization complete - report running status. 
	ServiceStatus.dwCurrentState       = SERVICE_RUNNING; 
	ServiceStatus.dwCheckPoint         = 0; 
	ServiceStatus.dwWaitHint           = 0; 

	if (!SetServiceStatus (ServiceStatusHandle, &ServiceStatus))  { 
		status = GetLastError(); 
		//sprintf(szInfo,"设置 [%s] 运行状态失败! 错误码:%ld!", SERVICE_NAME, status);
	} 

	// This is where the service does its work. 
	//sprintf(szInfo,"[%s] 启动成功!", SERVICE_NAME);

	return; 
} 
 
//  Stub initialization function. 
DWORD ServiceInitialization(DWORD argc, LPTSTR* argv, DWORD *specificError) { 
    argv; 
    argc; 
    specificError; 
    return 0; 
} 

//void StopService () {
//	// 首先打开SCM数据库
//	SC_HANDLE hSCM = ::OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS);
//	if (hSCM != NULL) {
//		SC_HANDLE hService = ::OpenService(hSCM, SERVICE_NAME, SERVICE_USER_DEFINED_CONTROL|SERVICE_QUERY_STATUS);
//		if (hService != NULL)	{
//			if (::QueryServiceStatus (hService, &ServiceStatus) == 0) {
//				//DWORD dwRet = GetLastError ();
//				//char szInfo[128];
//				//sprintf(szInfo,"获取 [%s] 运行状态失败! 错误代码:%d!", SERVICE_NAME, dwRet);
//			}
//			else {
//				if (ServiceStatus.dwCurrentState == SERVICE_RUNNING) {
//					if (::ControlService (hService, USERCONTROL_STOP, &ServiceStatus) == 0)	{
//						//DWORD dwRet = GetLastError ();
//						//char szInfo[128];
//						//sprintf(szInfo,"关闭 [%s] 失败! 错误代码:%d!", SERVICE_NAME, dwRet);
//					}
//					else {
//						//char szInfo[128];
//						//sprintf(szInfo,"关闭 [%s] 成功!", SERVICE_NAME);
//					}
//				}
//			}
//			::CloseServiceHandle (hService);
//		}
//		::CloseServiceHandle (hSCM);
//	}
//}

void WINAPI ServiceCtrlHandler (DWORD opcode) {
	switch (opcode)	{
	case SERVICE_CONTROL_PAUSE:
		ServiceStatus.dwCurrentState = SERVICE_PAUSE_PENDING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		ServiceStatus.dwCurrentState = SERVICE_PAUSED;
		break;
	case SERVICE_CONTROL_CONTINUE:
		ServiceStatus.dwCurrentState = SERVICE_CONTINUE_PENDING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		ServiceStatus.dwCurrentState = SERVICE_RUNNING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		break;
	case SERVICE_CONTROL_INTERROGATE:
		break;
	case SERVICE_CONTROL_STOP:
		ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		ServiceStatus.dwCurrentState = SERVICE_STOPPED;
		PostThreadMessage (dwThreadID, WM_QUIT, 0, 0);
		break;
	case SERVICE_CONTROL_SHUTDOWN:
		ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		ServiceStatus.dwCurrentState = SERVICE_STOPPED;
		PostThreadMessage (dwThreadID, WM_QUIT, 0, 0);
		break;
	case USERCONTROL_STOP:
		ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
		ServiceStatus.dwCurrentState = SERVICE_STOPPED;
		PostThreadMessage (dwThreadID, WM_QUIT, 0, 0);
		break;
	default:
		break;
	}
	SetServiceStatus (ServiceStatusHandle, &ServiceStatus);
}

//  察看服务是否已安装
bool IsRegistered () {
	bool bResult = false;
	// 首先打开SCM数据库
	SC_HANDLE hSCM = ::OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS);

	if (hSCM != NULL) {
		SC_HANDLE hService;
        hService = ::OpenService (hSCM, SERVICE_NAME, SERVICE_QUERY_CONFIG);
		if (hService != NULL) {
			bResult = true;
			::CloseServiceHandle (hService);
		}

		::CloseServiceHandle (hSCM);
	}
	return bResult;
}
bool regist (const char* service_name, const char* service_showname, const char* service_description) {
	SERVICE_NAME = LPSTR(service_name);
	SERVICE_SHOWNAME = LPSTR(service_showname);
	LPSTR Service_Description = LPSTR(service_description);
	// 察看服务是否已安装
	if (IsRegistered ())
		unregist(service_name);

	// 首先打开SCM数据库
	SC_HANDLE hSCM = ::OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS);

	if (hSCM == NULL) {
		//gRunInfo.SendInfo("*** 打开管理控制器失败！！！！");
		return false;
	}

	char szFilePath[1024 + 1] = {"\0"};
	::GetModuleFileName (NULL, (LPSTR)szFilePath, 1024);
	SC_HANDLE hService;
    hService = ::CreateService (hSCM, SERVICE_NAME, SERVICE_SHOWNAME, 
		                           SERVICE_ALL_ACCESS|SERVICE_QUERY_STATUS, SERVICE_WIN32_OWN_PROCESS,
		                           SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL, (LPSTR)szFilePath,
		                           NULL, NULL, (LPCSTR)(""), NULL, NULL);
	if (hService == NULL) {
		::CloseServiceHandle (hSCM);
		//DWORD dwRet = GetLastError ();
		//char szInfo[128];
		//sprintf(szInfo,"创建 [%s] 服务失败! 错误代码:%d!", SERVICE_NAME, dwRet);
		return false;
	}
	HMODULE hModule = ::LoadLibrary (TEXT("Advapi32.dll"));
	if (hModule != NULL) {
		FARPROC pChangeServiceConfig2 = ::GetProcAddress (hModule, "ChangeServiceConfig2A");
		LP_CHANGESERVICECONFIG_ROUTINE pRoute = (LP_CHANGESERVICECONFIG_ROUTINE)pChangeServiceConfig2;
		if (pChangeServiceConfig2 != NULL) {
			SERVICE_DESCRIPTION ServiceDescription;
			memset(&ServiceDescription, 0, sizeof(SERVICE_DESCRIPTION));
			ServiceDescription.lpDescription = Service_Description;
			(pRoute) (hService, SERVICE_CONFIG_DESCRIPTION, &ServiceDescription);
		}
		::FreeLibrary (hModule);
	}
	::CloseServiceHandle (hService);
	::CloseServiceHandle (hSCM);
	return true;
}

bool unregist (const char* service_name) {
	SERVICE_NAME = LPSTR(service_name);
	// 察看服务是否已安装
	if (!IsRegistered())	{
		return true;
	}
	// 首先打开SCM数据库
	SC_HANDLE hSCM = ::OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS);

	if (hSCM == NULL)
		return false;
	SC_HANDLE hService;
	hService = ::OpenService (hSCM, SERVICE_NAME, SERVICE_STOP|DELETE);
	if (hService == NULL) {
		::CloseServiceHandle (hSCM);		
		char szInfo[1024] = {"\0"};
		sprintf(szInfo,"无法打开 [%s] 服务!", SERVICE_NAME);
		MessageBox (NULL, (LPCSTR)szInfo, SERVICE_NAME, MB_OK);
		return false;
	}

	::ControlService (hService, SERVICE_CONTROL_STOP, &ServiceStatus);

	int bDelete = ::DeleteService (hService);
	::CloseServiceHandle (hService);
	::CloseServiceHandle (hSCM);

	if (bDelete) {
		return true;
	}

	char szInfo[1024] = {"\0"};
	sprintf(szInfo,"无法删除 [%s] 服务!", SERVICE_NAME);
	MessageBox (NULL, (LPCSTR)szInfo, SERVICE_NAME, MB_OK);
	return false;
}
}
