﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: time.cpp
\brief 时间类CPPTime的实现
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
*/

#include "base_class.h"
#include <assert.h>
#include <stdlib.h>
#include <time.h>
#include <string>

namespace ns_train {
CPPTime::CPPTime():m_nTime(0) {
}

CPPTime::CPPTime(const CPPTime& tmTime) {
    m_nTime = tmTime.m_nTime;
}

CPPTime::~CPPTime() {
}

CPPTime& CPPTime::operator = (const CPPTime& tmTime) {
    m_nTime = tmTime.m_nTime;
	return *this;
}

void CPPTime::addTime(int nDays, int nHours, int nMins, int nSecs) {
    type_time_t tTimeSpan = nSecs + 60* (nMins + 60* (nHours + 24* nDays));
	m_nTime += tTimeSpan;
}

std::string CPPTime::toStdString(ETimeFormat eType) const {
    char tmpBuf[64] = {'\0'};
    switch (eType) {
    case ETF_SHOW_DATE:
        sprintf(tmpBuf, "%04d-%02d-%02d", getYear(), getMonth(), getDay());
		break;
    case ETF_SHOW_TIME:
        sprintf(tmpBuf, "%02d:%02d:%02d", getHour(), getMinute(), getSecond());
		break;
    case ETF_SHOW_DATETIME:
		sprintf(tmpBuf, "%04d-%02d-%02d %02d:%02d:%02d",
            getYear(), getMonth(), getDay(), getHour(), getMinute(), getSecond());
		break;
	default:
		break;
	}
	return tmpBuf;
}


bool CPPTime::setTime(const char *pTimeString) {
    if (pTimeString == NULL) {
		return false;
    }
	struct tm atm;
    type_time_t tTime;
    type_int32 tYear, tMonth, tDay, tHour, tMinute, tSecond;
#ifdef WIN32
    sscanf_s(pTimeString, "%04d-%02d-%02d %02d:%02d:%02d",
		&tYear, &tMonth, &tDay, &tHour, &tMinute, &tSecond);
#else
    sscanf(pTimeString, "%04d-%02d-%02d %02d:%02d:%02d",
		&tYear, &tMonth, &tDay, &tHour, &tMinute, &tSecond);
#endif
	atm.tm_year = tYear - 1900;
	atm.tm_mon = tMonth - 1;
	atm.tm_mday = tDay;
	atm.tm_hour = tHour;
	atm.tm_min = tMinute;
	atm.tm_sec = tSecond;
	atm.tm_isdst = 0;
    g_mutexTime.lock();
	tTime = mktime(&atm);
    g_mutexTime.unlock();
    if (tTime == -1) {
		return false;
    }
	m_nTime = tTime;
	return true;
}

bool CPPTime::setTime(type_time_t tm) {
    if (tm < 0) {
		return false;
    }
    m_nTime = tm;
	return true;
}

void CPPTime::setCurrentTime() {
	m_nTime = time(NULL);
}

type_time_t CPPTime::getTime() const {
	return m_nTime;
}
//  线程安全
STime CPPTime::getLocalTime() const {
    STime timeInfo;
    static STime tmInfo = {0,0,0, 0,0,0, 0}; // 避免在高频调用时构建该对象。
    time_t tmpTime = time_t(m_nTime);
    if(tmpTime > 0)  {
        CPPMutexLocker mutexLocker(&g_mutexTime);
        struct tm tTime;
#ifdef WIN32
        localtime_s(&tTime, &tmpTime);
#else
        localtime_r(&tmpTime, &tTime);
#endif
        timeInfo.nYear	= tTime.tm_year + 1900;
        timeInfo.nMonth	= tTime.tm_mon + 1;
        timeInfo.nDay	= tTime.tm_mday;
        timeInfo.nHour	= tTime.tm_hour;
        timeInfo.nMinute = tTime.tm_min;
        timeInfo.nSecond = tTime.tm_sec;
    }
    return timeInfo;
}

bool CPPTime::isAM() const {
    return (getHour() < 12);
}

int CPPTime::getYear() const {
    return getLocalTime().nYear;
}

int CPPTime::getMonth() const
{
    return getLocalTime().nMonth;
}

int CPPTime::getDay() const
{
    return getLocalTime().nDay;
}

int CPPTime::getHour() const
{
    return getLocalTime().nHour;
}

int CPPTime::getMinute() const
{
    return getLocalTime().nMinute;
}

int CPPTime::getSecond() const
{
    return getLocalTime().nSecond;
}

int CPPTime::getWeek() const {
    CPPMutexLocker mutexLocker(&g_mutexTime);
    struct tm tTime;
    time_t tmpTime = time_t(m_nTime);
#ifdef WIN32
    localtime_s(&tTime, &tmpTime);
#else
    localtime_r(&tmpTime, &tTime);
#endif
    return tTime.tm_wday;
}

int CPPTime::getDayOfYear() const {
    CPPMutexLocker mutexLocker(&g_mutexTime);
    struct tm tTime;
    time_t tmpTime = time_t(m_nTime);
#ifdef WIN32
    localtime_s(&tTime, &tmpTime);
#else
    localtime_r(&tmpTime, &tTime);
#endif
    return tTime.tm_yday;
}

bool CPPTime::operator == (const CPPTime& tmTm) const {
    return m_nTime == tmTm.m_nTime;
}

bool CPPTime::operator != (const CPPTime& tmTm) const {
    return m_nTime != tmTm.m_nTime;
}

bool CPPTime::operator < (const CPPTime& tmTm) const {
    return m_nTime < tmTm.m_nTime;
}

bool CPPTime::operator <= (const CPPTime& tmTm) const {
    return m_nTime <= tmTm.m_nTime;
}

bool CPPTime::operator > (const CPPTime& tmTm) const {
    return m_nTime > tmTm.m_nTime;
}

bool CPPTime::operator >= (const CPPTime& tmTm) const {
    return m_nTime >= tmTm.m_nTime;
}
}
