﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: com_serialproxy_linux.cpp
\brief 串口通讯类实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/04
*/

#include "com_serialproxy_linux.h"
#include "customtype.h"
#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
/////////////////////////////////////////////////////////////////////////
namespace ns_train {
CSerialProxy::CSerialProxy(const s_serial_config& config) : 
	m_fd(-1), // -1表示设备处于关闭状态
	m_eSerialState(ESERIAL_CLOSE),
	m_serialConfig(config){
}
CSerialProxy::~CSerialProxy() {
	if (m_fd > 0) {
		close();
	}
}

bool CSerialProxy::open() {
	bool bRet = false;
	if (m_fd > 0) {
		return true;
	}
	m_fd = ::open(m_serialConfig.strDevicename, O_RDWR | O_NOCTTY | O_NDELAY);
	if (m_fd < 0) {
		m_eSerialState = ESERIAL_ERROR;
	}
	else {
		isatty(m_fd);	// 判断描述符是否为终端
		int flags = fcntl(m_fd, F_GETFL, 0);
		m_eSerialState = ESERIAL_OPEN;
		bzero(&m_termio1, sizeof(m_termio1));
		bzero(&m_termio2, sizeof(m_termio2));
		tcgetattr(m_fd, &m_termio1);
		bRet = setSerialConfig();
		tcgetattr(m_fd, &m_termio2);
	}
	return bRet;
}

bool CSerialProxy::isOpen() {
	return ((ESERIAL_OPEN == m_eSerialState) ? true : false);
}

bool CSerialProxy::close() {
	tcsetattr(m_fd, TCSADRAIN, &m_termio1);
	if (m_fd > -1) {
		::close(m_fd);
	}
	m_fd = -1;
	m_eSerialState = ESERIAL_CLOSE;
	return true;
}

int CSerialProxy::readData(char* data, int count) {
	if (m_eSerialState != ESERIAL_OPEN) {
		return -1;
	}
	// 对于本机串口
	int nRead = read(m_fd, data, count);
	if (nRead <= 0) {
		return 0;
	}
	return nRead;
}

int CSerialProxy::writeData(char* data, int count) {
	if (m_eSerialState != ESERIAL_OPEN) {
		return -1;
	}
	int nWrite = -1;
	nWrite = write(m_fd, data, count);
	if (nWrite <= 0)
		return 0;

	return nWrite;
}
int CSerialProxy::makeBaudrate(int baudrate) {
	switch (baudrate) {
	case 0:
		return (B0);
	case 50:
		return (B50);
	case 75:
		return (B75);
	case 110:
		return (B110);
	case 134:
		return (B134);
	case 150:
		return (B150);
	case 200:
		return (B200);
	case 300:
		return (B300);
	case 600:
		return (B600);
	case 1200:
		return (B1200);
	case 2400:
		return (B2400);
	case 4800:
		return (B4800);
	case 9600:
		return (B9600);
	case 19200:
		return (B19200);
	case 38400:
		return (B38400);
	default:
		return (B9600);
	}
}
bool CSerialProxy::setSerialConfig() {
	return setSerialConfig(m_serialConfig);
}
bool CSerialProxy::setSerialConfig(const s_serial_config& config) {
	setBPS(config.u32Baudrate);
	setDataBit(config.byDatabits);
	setParityCheck(config.chParity);
	setStopBit(config.chStopbit);
	setParam();
	return true;
}

bool CSerialProxy::setBPS(int baudrate) {
	struct termios options;
	bzero(&options, sizeof(options));

	tcgetattr(m_fd, &options);
	cfsetispeed(&options, makeBaudrate(baudrate));
	cfsetospeed(&options, makeBaudrate(baudrate));
	options.c_cflag |= (CLOCAL | CREAD);// | CRTSCTS
	tcflush(m_fd, TCIOFLUSH);
	tcsetattr(m_fd, TCSANOW, &options);
	return true;
}
void CSerialProxy::setDataBit(int databit) {
	struct termios options;
	bzero(&options, sizeof(options));
	tcgetattr(m_fd, &options);
	options.c_cflag &= ~CSIZE;
	switch (databit) {
	case 5:
		options.c_cflag |= CS5;
		break;
	case 6:
		options.c_cflag |= CS6;
		break;
	case 7:
		options.c_cflag |= CS7;
		break;
	case 8:
	default:
		options.c_cflag |= CS8;
		break;
	}
	tcflush(m_fd, TCIOFLUSH);
	tcsetattr(m_fd, TCSANOW, &options);
}
void CSerialProxy::setStopBit(char stopbit) {
	struct termios options;
	bzero(&options, sizeof(options));
	tcgetattr(m_fd, &options);
	switch (stopbit) {
	case 2:
	case '2':
		options.c_cflag |= CSTOPB;
		break;
	case 0:
	case '0':
	case 1:
	case '1':
	default:
		options.c_cflag &= ~CSTOPB;
		break;
	}
	tcflush(m_fd, TCIOFLUSH);
	tcsetattr(m_fd, TCSANOW, &options);
}

void CSerialProxy::setParityCheck(char parity) {
	struct termios options;
	bzero(&options, sizeof(options));
	tcgetattr(m_fd, &options);
	switch (parity) {
	case 'E':
	case 'e':
		options.c_cflag |= PARENB;
		options.c_cflag &= ~PARODD;
		break;
	case 'O':
	case 'o':
		options.c_cflag |= PARENB;
		options.c_cflag |= PARODD;
		break;
	case 'N':
	case 'n':
	default:
		options.c_cflag &= ~PARENB;
		break;
	}
	tcflush(m_fd, TCIOFLUSH);
	tcsetattr(m_fd, TCSANOW, &options);
}

void CSerialProxy::setParam() {
	struct termios options;
	bzero(&options, sizeof(options));

	tcgetattr(m_fd, &options);
	///	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	options.c_lflag = 0;
	options.c_iflag = IXON | IXOFF;
	///	options.c_oflag &= ~OPOST;
	options.c_oflag = 0;
	tcflush(m_fd, TCIOFLUSH);
	tcsetattr(m_fd, TCSANOW, &options);
}

int CSerialProxy::getBaudrate() {
	return (makeBaudrate(cfgetospeed(&m_termio2)));
}

bool CSerialProxy::setBaudrate(int bps) {
	setBPS(bps);
	tcgetattr(m_fd, &m_termio2);
	return true;
}
}
