﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: com_serialproxy_windows.h
\brief 串口通讯类实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/

#include "com_serialproxy_windows.h"
#include "stdio.h"
/////////////////////////////////////////////////////////////////////////
namespace ns_train {
CSerialProxy::CSerialProxy(const s_serial_config& config) : 
	m_hCom(INVALID_HANDLE_VALUE),
	m_eSerialState(ESERIAL_CLOSE),
	m_serialConfig(config){
}
CSerialProxy::~CSerialProxy() {
	if (m_hCom) {
		CloseHandle(m_hCom);
		m_hCom = INVALID_HANDLE_VALUE;
	}
}

bool CSerialProxy::open() {
	bool bRet = false;
	if (isOpen()) {
		return true;
	}
	if (INVALID_HANDLE_VALUE != m_hCom) {
		return bRet;
	}
	m_hCom = CreateFile((LPCTSTR)m_serialConfig.strDevicename, // 串口名,如/dev/tty01,COM1等
						GENERIC_READ | GENERIC_WRITE, // 用于读写
						0,							// 不共享
						NULL,						// 无安全属性
						OPEN_EXISTING,				// 文件（设备）必须已存在
						FILE_ATTRIBUTE_NORMAL,		// 默认属性
						NULL);						// 复制文件句柄。如果不为零，则指定一个文件句柄。新文件将从这个文件中复制扩展属性
	if (m_hCom == INVALID_HANDLE_VALUE) {
		m_eSerialState = ESERIAL_ERROR;
		bRet = false;
	}
	else { // 打开串口成功
		m_eSerialState = ESERIAL_OPEN;
		bRet = setSerialConfig(); // 设置串口通信参数
	}
	return bRet;
}

bool CSerialProxy::isOpen() {
	return ((ESERIAL_OPEN == m_eSerialState) ? true : false);
}

bool CSerialProxy::close() {
	if (INVALID_HANDLE_VALUE != m_hCom)	{
		CloseHandle(m_hCom);
		m_hCom = INVALID_HANDLE_VALUE;
		m_eSerialState = ESERIAL_CLOSE;
	}
	return true;
}
int CSerialProxy::readData(char* data, int count) {
	DWORD		dwErrorFlags;
	COMSTAT		ComStat;
	DWORD		sReadByteCount = 0;
	BOOL		bReadState = FALSE;
	if (INVALID_HANDLE_VALUE == m_hCom) {
		return -1;
	}
	ClearCommError(m_hCom, &dwErrorFlags, &ComStat); // 查询串口状态
	if (ComStat.cbInQue) {		
		if ((DWORD)count > ComStat.cbInQue) {// 读缓冲区有数据
			count = ComStat.cbInQue;
		}
		bReadState = ReadFile(m_hCom, data, count, &sReadByteCount, NULL);
		if (!bReadState) {
			if (GetLastError() != ERROR_IO_PENDING) {
				sReadByteCount = -1;
			}
		}
	}
	return sReadByteCount;
}

int CSerialProxy::writeData(char* data, int count) {
	DWORD		dwErrorFlags;
	unsigned long sendCount = 0;
	COMSTAT		ComStat;
	if (!m_hCom) {
		return -1;
	}
	ClearCommError(m_hCom, &dwErrorFlags, &ComStat);
	if (!ComStat.cbOutQue) {
		// 发送缓冲区没有数据可以发送
		if (!WriteFile(m_hCom, data, count, &sendCount, NULL)) 	{
			if (GetLastError() != ERROR_IO_PENDING) {
				sendCount = -1;
			}
		}
	}
	return sendCount;
}

bool CSerialProxy::setSerialConfig() {
	return setSerialConfig(m_serialConfig);
}
bool CSerialProxy::setSerialConfig(const s_serial_config& config) {
	DCB		comdcb;
	BOOL bComFlag;
	SetCommMask(m_hCom, EV_RXCHAR);
	SetupComm(m_hCom, 4096, 4096);
	PurgeComm(m_hCom, PURGE_TXABORT | PURGE_RXABORT |
		PURGE_TXCLEAR | PURGE_RXCLEAR);
	GetCommState(m_hCom, &comdcb);
	// 通信参数
	comdcb.BaudRate = config.u32Baudrate;		 // 波特率
	switch (config.chParity) {
	case 'O':
	case 'o':
		comdcb.Parity = ODDPARITY;			 // 奇校验
		break;
	case 'E':
	case 'e':
		comdcb.Parity = EVENPARITY;			 // 偶校验
		break;
	case 'N':
	case 'n':
		comdcb.Parity = NOPARITY;			 // 无校验
		break;
	default:
		comdcb.Parity = NOPARITY;			 // 无校验
		break;
	}

	switch (config.chStopbit) {
	case 1:
	case '1':
		comdcb.StopBits = ONE5STOPBITS;		 // 1.5 位停止位
		break;
	case 2:
	case '2':
		comdcb.StopBits = TWOSTOPBITS;		 // 2位停止位
		break;
	case 0:
	case '0':
	default:
		comdcb.StopBits = ONESTOPBIT;		 // 1位停止位
		break;
	}
	comdcb.ByteSize = config.byDatabits;	 // 数据位
	bComFlag = SetCommState(m_hCom, &comdcb);
	if (!bComFlag) {
		close();
		return false;
	}
	return true;
}
type_uint16 CSerialProxy::getBaudrate() {
	return m_serialConfig.u32Baudrate;
}
bool CSerialProxy::setBaudrate(type_uint16 baudrate) {
	DCB		comdcb;
	GetCommState(m_hCom, &comdcb);
	m_serialConfig.u32Baudrate = baudrate;
	if (comdcb.BaudRate != baudrate) {
		comdcb.BaudRate = baudrate;
	}
	if (SetCommState(m_hCom, &comdcb)) {
		return true;
	}
	close();
	return false;
}
}
