﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: serverthread.cpp
\brief 服务器端线程类实现。该线程负责处理客户端连接及交互。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/
#include "serverthread.h"

#include <iostream>
#include <string.h>

#include "base/basedll/base_class.h"
#include "com_serial.h"
#include "datavector.h"
#include "host_network_api.h"
#include "ks07_05/ks07_05_frametype.h"
#include "service_api.h"

static const int c_nReceiveBufferLength = 12800; // 需要注意:确保内存够用，也就是能够至少容纳一帧接收到的数据。
static const int c_nSendBufferLength = 12800;	 // 需要注意:确保内存够用，也就是能够至少容纳一帧发送的数据。

CServerThread::CServerThread() {
    m_pReceiveBuf = new char[c_nReceiveBufferLength];
    memset(m_pReceiveBuf, 0, c_nReceiveBufferLength*sizeof(char));
    m_pSendBuf = new char[c_nSendBufferLength];
    memset(m_pSendBuf, 0, c_nSendBufferLength*sizeof(char));
}
void CServerThread::run() {

    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return;
    }
    if (NULL == getParam()) {
        return;
    }
    m_pSerial = static_cast<ns_train::CPPSerial*>(getParam()); // 串口通信对象
    if (NULL == m_pSerial) {
        return;
    }

    bool bOk = true;
    bool bOpened = false;
    while (isWorking() && bOk) {
        ns_train::api_sleep(10); // 睡眠时间不易过长，否则影响后续代码。
        keepAlive(); // 请安，汇报心跳
        bOpened = m_pSerial->open(); // 首先打开串口
        if (!bOpened) {
            continue;
        }

        makeFrameAndSend();         // 检查有没有需要发送的命令
   }
    freeBuffer(); // 释放资源
}

void CServerThread::reset_send() {

}
void CServerThread::freeBuffer() {
    delete[] m_pReceiveBuf;
    m_pReceiveBuf = NULL;

    delete[] m_pSendBuf;
    m_pSendBuf = NULL;
}

bool CServerThread::makeFrameAndSend() {

    bool bOk = true;

    ns_train::CPPTime tmNow;
    tmNow.setCurrentTime();
    if ((tmNow.getTime() - m_tmLastSendAllData.getTime()) > 5) { // 每5秒发送一次全部模拟量数据
        bOk &= makeFrameAndSend_Analogs();
    }
    return bOk;
}

bool CServerThread::makeFrameAndSend_Analogs() {
    /*
    *	|同步字0x68	|   【1个字节】- 字节0
    *	|帧类型		|   【1个字节】- 字节1
    *	|起始数据序号	|   【4个字节】- 字节2~5
    *	|本帧数据个数	|   【4个字节】- 字节6~9
    *	|1个数据		|   【8个字节】- 字节10~17
    *	|...		|
    *	|1个数据		|   【8个字节】- 字节n~n+7
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    // 1. 定义通信数据帧中各个字段对应的变量。请注意，要保证变量的数据类型跟通信帧中对应的字段完全一致。
    bool bError = false;
    const type_uint8 uchStart	= 0x68;		// 同步字
    type_uint8 uchFrameType		= type_uint8(ESERVERFRAME_ANALOGDATAS);	// 帧类型
    type_uint32 u32StartDataId = 0;	// 起始数据序号
    type_uint32 u32CurrentFrameDataCount = 0;	// 本帧数据个数
    type_double dValue = 0.0;		// 数据的值

    // 2. 计算该缓冲区能容纳多少有效数据
    const type_int32 nFrameMaxDataCount = (c_nSendBufferLength - sizeof(uchStart) 
                                          - sizeof(uchFrameType) - sizeof(u32StartDataId) 
                                          - sizeof(u32CurrentFrameDataCount) )/ sizeof(dValue); // 一帧最多可发送的数据个数

    // 3. 定义相关变量。
    type_uint32 nAllDataCount = pDataVector->getSize();	// 待发送数据总个数
    type_uint32 nSendDataCount = 0;		// 已经发送的数据个数
    type_int32 nFrameLength = 0;        // 本帧数据总长度
    type_uint8 *pBuf = NULL;            // 发送缓冲区游标指针

    while (nSendDataCount < nAllDataCount) { // 数据都发出去了吗？
        pBuf = (type_uint8*)m_pSendBuf;
        pBuf[0] = uchStart;				// 同步字
        pBuf++;
        pBuf[0] = uchFrameType;			// 帧类型
        pBuf++;
        *(reinterpret_cast<type_uint32*>(pBuf)) = ns_train::HostToNetwork(u32StartDataId);	// 起始数据序号
        pBuf += sizeof(u32StartDataId);
        if ((nAllDataCount-nSendDataCount) < nFrameMaxDataCount) {
            u32CurrentFrameDataCount = nAllDataCount-nSendDataCount;
        }
        else {
            u32CurrentFrameDataCount = nFrameMaxDataCount;
        }
        nFrameLength = static_cast<int>(sizeof(uchStart) + sizeof(uchFrameType) + sizeof(u32StartDataId)  +
                sizeof(u32CurrentFrameDataCount) + u32CurrentFrameDataCount * sizeof(dValue)); // 本帧数据长度
        *(reinterpret_cast<type_uint32*>(pBuf)) = ns_train::HostToNetwork(u32CurrentFrameDataCount); // 本帧数据个数
        pBuf += sizeof(u32CurrentFrameDataCount);

        for (type_uint32 idx = 0; idx < u32CurrentFrameDataCount; idx++) {
            pDataVector->getData(idx+1+ u32StartDataId, dValue);
            *(reinterpret_cast<type_double*>(pBuf)) = ns_train::HostToNetwork(dValue);	// 数据
            pBuf += sizeof(dValue);
        }
        u32StartDataId += u32CurrentFrameDataCount;
        // 数据组帧完毕，开始发送。
        int nWrite = m_pSerial->write(m_pSendBuf, nFrameLength);
        if (nWrite > 0) {
            nSendDataCount += u32CurrentFrameDataCount; // 更新已发送数据个数。
        }
        else {
            m_pSerial->close();
            bError = true;
            return false;
        }
        // ns_train::ii_sleep(1); // 如果数据量比较多，导致循环次数太多，可能导致CPU占用率高，那么可以用睡眠释放CPU。
    }
    m_tmLastSendAllData.setCurrentTime();
    return true;
}
