﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: app.cpp
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/
#include "app.h"
#include <iostream>
#include <string.h>
#include "base/basedll/service_api.h"
#include "base/basedll/base_api.h"
#include "base/basedll/base_class.h"
#include "com_serial.h"
#include "serverthread.h"
#include "datavector.h"

ns_train::CPPMutex g_mutexApp;
CApp* CApp::instance() {
    ns_train::CPPMutexLocker mutexLocker(&g_mutexApp);
    static CApp app;
    return &app;
}

CApp::CApp() : m_bProcRun(true), m_bTerminal(true), m_bHelp(false), m_pSerial(NULL), m_pServerThread(NULL){
}

CApp::~CApp() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
}
void CApp::setArgs(int argc, char* argv[]) {
    /*
    * 此处的默认值即是程序的默认运行状态。
    * 当用户未输入命令参数时，程序以默认方式运行，取决于此处定义的默认值。
    */
    bool bTerminal = false;	 // true:程序以终端（前台）方式运行, false:程序以后台服务方式运行。
    bool bFileMode = false;	 // true:程序启动时从文件加载数据。 false:程序从其他途径加载数据。
    bool bFileName = false;	 // true:用户在命令参数中输入了合法的文件名。false:用户未提供合法的文件名。
    char* szFileName = NULL; // 文件名
    bool bTestState = false; // true:程序以测试态运行。false：程序以正常态运行。

    // 解析命令参数, 所有命令参数均以"-"开头。
    for (int i = 1; i < argc; i++) {
        if (strcmp(argv[i], "-term") == 0) {
            bTerminal = true;
        }
        else if (strcmp(argv[i], "-f") == 0) {
            bFileMode = true;
            if ((i + 1) < argc) { // -f 后面跟的是文件名
                szFileName = argv[i + 1];
                if (szFileName[0] == '-') { // 此处可以自定义解析规则
                    bFileName = false;	// 文件名无效，是另一个命令参数（因为是'-'开头的字符串）。
                }
                else {
                    bFileName = true; // 文件名有效，才允许i+1。
                    i++;
                }
            }
        }
        else if (strcmp(argv[i], "-test") == 0) {
            bTestState = true;
        }
        else if (strcmp(argv[i], "-help") == 0) {
            m_bHelp = true;
        }
#ifdef WIN32
        else if (_stricmp(argv[i], "-regist") == 0) {
            std::cout << ">>> 模块注册。" << std::endl;
            ns_train::regist("ks07_07_server", "ks07_07_server",  "ks07_07_server服务");
            exit(0);
        }
        else if (_stricmp(argv[i], "-unregist") == 0) {
            std::cout << ">>> 模块注销。" << std::endl;
            ns_train::unregist("ks07_07_server");
            exit(0);
        }
#endif
    }
    if (m_bHelp) {
        std::cout << "=======================================" << std::endl;
        std::cout << ">>> ks07_07_server_d -term\t\t模块以终端方式运行。" << std::endl;
        std::cout << ">>> ks07_07_server_d -help\t\t显示帮助信息。" << std::endl;
    }

    if (bTerminal) {
        std::cout << "=======================================" << std::endl;
        std::cout << ">>> 服务器模块启动。" << std::endl;
        std::cout << ">>> 模块以终端方式运行。" << std::endl;
        setTerminalFlag(bTerminal);
    }
    if (bFileMode && bTerminal) {
        if (bFileName) {
            std::cout << ">>> 模块启动时以文件方式加载数据。文件名:" << szFileName << std::endl;
        }
        else {
            std::cout << ">>> 模块启动时以文件方式加载数据。无法获取文件名." << std::endl;
        }
    }
    if (bTestState && bTerminal) {
        std::cout << ">>> 模块运行在测试态。" << std::endl;
    }

}

void CApp::setTerminalFlag(bool bTerminal) {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	m_bTerminal = bTerminal;
}

bool CApp::getTerminalFlag() {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	return m_bTerminal;
}

bool CApp::getRunningFlag() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    return m_bProcRun;
}

void CApp::setRunningFlag(bool bRunningFlag) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_bProcRun = bRunningFlag;
}

void CApp::prepare(void) {
	// 初始化
	ns_train::s_serial_config serialConfig;
#ifdef WIN32
	strcpy(serialConfig.strDevicename, "\\\\.\\COM2");
#else
	strcpy(serialConfig.strDevicename, "/dev/tty01");
#endif
	serialConfig.u32Baudrate = 1200;	// 波特率
	serialConfig.byDatabits = 8;	// 8位数据位
	serialConfig.chParity = 'N';	// 无校验
	serialConfig.chStopbit = 0;		// 1位停止位
	m_pSerial = new ns_train::CPPSerial(serialConfig);
	if (m_bTerminal) {
		std::cout << "打开串口:" << serialConfig.strDevicename << std::endl;
		std::cout << "波特率:" << serialConfig.u32Baudrate << std::endl;
		std::cout << "数据位:" << serialConfig.byDatabits << std::endl;
		std::cout << "校验:" << serialConfig.chParity << std::endl;
	}
}

bool CApp::initialize() {
    /* do initialize work */
    prepare();
    CDataVector* pDataVector = CDataVector::instance(); // 数据容器指针
    if (NULL != pDataVector) {
        pDataVector->initialize();
    }
	m_pServerThread = new CServerThread();
	m_pServerThread->start(m_pSerial);

    return true;
}
/**
* @brief 模拟程序退出前的清理工作。
* @return true：成功，false:失败
*/
void CApp::beforeExit(void) {
    // 先退出线程，再释放套接字资源
    if (NULL != m_pServerThread) {
		m_pServerThread->exitThread();
        delete m_pServerThread;
		m_pServerThread = NULL;
    }
    if (NULL != m_pSerial) {
		m_pSerial->close();
		delete m_pSerial;
		m_pSerial = NULL;
    }
    if (!m_bHelp) {
        std::cout << ">>> 模块退出。" << std::endl;
        //char ch = '\0';
        //std::cin >> ch;
    }
}


void CApp::CommandProc() {
	if (!m_bTerminal) {
		return;
	}
#ifndef WIN32
	system("mesg n");	// 取消系统日志终端输出
#endif
	std::string strInput;
	std::cout << "-------------------------------------------------------" << std::endl;
	std::cout << ">>> 模块正常启动，请勿关闭!" << std::endl;
	CDataVector* pDataVector = CDataVector::instance(); // 数据容器指针
	while (m_bProcRun) {
		printMenu();
		std::cout << "<server -> command> : ";
		std::cin >> strInput;
		if (strInput.compare("help") == 0) {
			printMenu();
		}
		else if (strInput.compare("exit") == 0) {
			setRunningFlag(false);
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "模块正在退出 ..." << std::endl;
			return;
		}
		else if (strInput.compare("data") == 0) {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "数据项的值为:" << std::endl;
			/* do something.*/
			type_double dValue = 0.f;
			for (type_uint32 i = 0; i < pDataVector->getSize(); i++) {
				pDataVector->getData(i + 1, dValue); // 数据项的索引从1开始计数。
				std::cout << "data" << i + 1 << "=" << dValue << std::endl;
			}
		}
		else {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << ">>> 输入命令错误，请输入 \"help\"查询可用命令！" << std::endl;
			printMenu();
		}
	}
}

void CApp::printMenu(void) {
	if (!m_bTerminal) {
		return;
	}

	std::cout << "-------------------------------------------------------" << std::endl;
	std::cout << ">>> 1.帮助                           help" << std::endl;
	std::cout << ">>> 2.退出                           exit" << std::endl;
    std::cout << ">>> 3.显示所有数据                    data" << std::endl;
}    
void CApp::exec(void) {
	if (m_bTerminal) {		// 进程以终端方式运行
		CommandProc();		// 交互命令处理
	}
	else {					// 进程以服务方式运行
		ns_train::api_start_as_service("ks07_07_server", "ks07_07_server");
#ifndef WIN32
		/* 监视工作线程的运行状态 */
		while (getRunningFlag()) {
			ns_train::api_sleep(5000); // 5秒
			if (!m_pServerThread->isWorking() || !m_pServerThread->isAlive()) {
				// todo, 保存到日志
			}
		}
#endif
	}
}
