﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: clientthread.cpp
\brief 数据处理线程类实现。该线程负责持续更新数据。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/

#include "clientthread.h"

#include <iostream>
#include <string.h>

#include "base/basedll/base_file.h"
#include "datavector.h"
#include "service_api.h"
#include "com_serial.h"
#include "host_network_api.h"
#include "ks07_08/ks07_08_frametype.h"
#include "ks07_08/ks07_08_struct.h"

s_IDValue::s_IDValue(type_uint32 id, type_double value) {
    nId = id; dValue = value;
}

static const int c_nReceiveBufferLength = 12800; // 需要注意:确保内存够用，也就是能够至少容纳完整的一帧待发送数据。
static const int c_nSendBufferLength = 12800;	 // 需要注意:确保内存够用，也就是能够至少容纳完整的一帧待发送数据。

CClientThread::CClientThread() {
    m_pReceiveBuf = new char[c_nReceiveBufferLength];
    memset(m_pReceiveBuf, 0, c_nReceiveBufferLength*sizeof(char));
    m_pSendBuf = new char[c_nSendBufferLength];
    memset(m_pSendBuf, 0, c_nSendBufferLength*sizeof(char));
    reset_receive();
}
void CClientThread::run() {
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return;
    }
    if (NULL == getParam()) {
        return;
    }
    m_pSerial = static_cast<ns_train::CPPSerial*>(getParam()); // 串口通信对象
    if (NULL == m_pSerial) {
        return;
    }
    bool bOpened = false;
    while (isWorking()) {
        ns_train::api_sleep(10); // 睡眠时间不易过长，否则影响后续代码。
        keepAlive(); // 请安，发送心跳信号
        bOpened = m_pSerial->open(); // 首先打开串口
        if (!bOpened) {
            continue;
        }

        makeFrameAndSend();         // 检查有没有需要发送的命令

        recvAndParseFrame();        // 接收数据、解帧
	}
    freeBuffer();
}
void CClientThread::freeBuffer() {
    delete[] m_pReceiveBuf;
    m_pReceiveBuf = NULL;

    delete[] m_pSendBuf;
    m_pSendBuf = NULL;
}
void CClientThread::reset_receive() {
    m_nReceivedCount = 0;
    m_byFrameType = ESERVERFRAME_ANALOGDATAS;
    m_eFrameStatus = EFRAME_SYNCHRONIZATION;
    m_nFrameLength = 0;
    m_u32StartId = 0;
    m_u32CurrentFrameDataCount = 0;
}
void CClientThread::reset_send() {

}
bool CClientThread::recvAndParseFrame() {
    /*
    *	|同步字0x68	|   【1个字节】- 字节0
    *	|帧类型		|   【1个字节】- 字节1
    *	|...		|    不同的“帧类型”有不同的帧格式，请查看后续代码
    */

    // 接收数据并解析
    switch (m_eFrameStatus) {
    case EFRAME_SYNCHRONIZATION:  // 搜索同步字
        parseFrame_Synchronization();
        break;
    case EFRAME_GET_FRAMETYPE:    // 确定帧类型
        parseFrame_GetFrameType();
        break;
    case EFRAME_GET_FRAMELENGTH:  // 确定帧长度
        parseFrame_GetFrameLength();
        break;
    case EFRAME_PARSEDATA:        // 接收数据并解析
        parseFrame_Parse();
        break;
    default:
        reset_receive();
        return false;
    }
    return true;
}

bool CClientThread::readData(type_int32 nAllData) {
    // 2. 读取数据
    type_int32 readCount = m_pSerial->read(m_pReceiveBuf + m_nReceivedCount, nAllData - m_nReceivedCount);
    // 3. 没有数据则返回
    if (0 == readCount) { // 没有数据, 等下一个循环再尝试读取。
        return true;
    }
    // 4. 出现错误则将数据复位，返回false
    if (readCount < 0) { // 出现错误
        reset_receive();
        return false;
    }
    // 5. 更新接收到的字节数
    m_nReceivedCount += readCount;
    return true;
}
bool CClientThread::parseFrame_Synchronization() {
    reset_receive();
    const type_uint8 uchStart = 0x68;		// 同步字
    // 1. 设置需要读取的总字节数
    type_int32 nAllData = static_cast<int>(sizeof(uchStart)); // 【同步字】
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) { // 已经读取完毕
		if (m_pReceiveBuf[0] == 0x68) {
			m_eFrameStatus = EFRAME_GET_FRAMETYPE;		// 切换到下一个工作状态
		}
		else {
			reset_receive();
		}
	}
    return true;
}

bool CClientThread::parseFrame_GetFrameType() {
    const type_uint8 uchStart = 0x68;		// 同步字
    // 1. 设置需要读取的总字节数
    type_int32 nAllData = static_cast<int>(sizeof(uchStart)+sizeof(m_byFrameType)); // 【同步字】 + 【帧类型】
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) { // 已经读取完毕
        ks07_08_FrameHeader* pHeader = reinterpret_cast<ks07_08_FrameHeader*>(m_pReceiveBuf);
        pHeader->NetToHost();
        m_byFrameType = pHeader->byFrameType;       // 保存【帧类型】
        m_eFrameStatus = EFRAME_GET_FRAMELENGTH;    // 切换到下一个工作状态
    }
    return true;
}

bool CClientThread::parseFrame_GetFrameLength() {
    switch (m_byFrameType) {
    case ESERVERFRAME_ONEANALOGDATA:
        parseFrame_GetFrameLength_OneAnalog();
        break;
    case ESERVERFRAME_ANALOGDATAS:
        parseFrame_GetFrameLength_Analogs();
        break;
    case ESERVERFRAME_FILE:
        parseFrame_GetFrameLength_File();
        break;
    default:
        reset_receive();
        return false;
    }
    return true;
}
bool CClientThread::parseFrame_GetFrameLength_OneAnalog() {
    /*
    *	|同步字0x68	|   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
    *	|数据项Id	|   字节2~5   - 【4个字节】
    *	|数据值		|   字节6~13  - 【8个字节，double】
    */

    // 1. 设置需要读取的字节数。
    m_nFrameLength = sizeof(ks07_08_Server_OneAnalogData);
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
	return true;
}
bool CClientThread::parseFrame_GetFrameLength_Analogs() {
    /*
    *	|同步字0x68		|   字节0     - 【1个字节】
    *	|帧类型			|   字节1     - 【1个字节，ESERVERFRAME_ANALOGDATAS】
    *	|起始数据序号     |   字节2~5   - 【4个字节】
    *	|本帧数据个数     |   字节6~9   - 【4个字节】
    *	|1个数据         |   字节10~17 - 【8个字节，double】
    *	|...			|
    *	|1个数据         |   字节n~n+7 - 【8个字节，double】
    */
    const type_uint8 uchStart = 0x68; // 同步字
    type_int32 nAllData = 0;
    type_int32 readCount = 0;

    // 1. 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
    nAllData = sizeof(uchStart) + sizeof(m_byFrameType) + sizeof(m_u32StartId) + sizeof(m_u32CurrentFrameDataCount);
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) {
        ks07_08_Server_AnalogDatas* pAnalogDatas = reinterpret_cast<ks07_08_Server_AnalogDatas*>(m_pReceiveBuf);
        pAnalogDatas->NetToHost();
        m_u32StartId = pAnalogDatas->u32StartDataId;
        m_u32CurrentFrameDataCount = pAnalogDatas->u32CurrentFrameDataCount;
        m_nFrameLength = static_cast<type_int32>(sizeof(ks07_08_Server_AnalogDatas) + m_u32CurrentFrameDataCount*sizeof(type_double));
        m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
    }
    return true;
}
bool CClientThread::parseFrame_GetFrameLength_File() {
    /*
    *	|同步字0x68               	|   【1个字节】- 字节0
    *	|帧类型                   	|   【1个字节】- 字节1
    *   |本帧数据总长度				|	【4个字节】- 字节2~5
    *	|文件名                   	|   【MAX_FILE_NAME_LENGTH】- 字节6~69
    *	|是否最后一帧              	|   【1个字节】- 字节70
    *	|本帧数据在文件中的起始偏移量|   【4个字节】- 字节71~74
    *	|本帧文件内容长度           |   【4个字节】- 字节75~78
    *	|本帧文件内容              	|   【N个字节】- 字节79~
    */
    ks07_08_Server_File fileFrame;

    // 1. 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
    type_int32 nAllData = sizeof(ks07_08_Server_File);
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) {
        // 2. 设置需要读取的字节数。
        ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pReceiveBuf);
        m_nFrameLength = static_cast<type_int32>(pFile->u32FrameLength);
        m_nFrameLength = ns_train::NetworkToHost(m_nFrameLength);
        m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
    }
    return true;
}
bool CClientThread::parseFrame_Parse() {
    switch (m_byFrameType) {
    case ESERVERFRAME_ONEANALOGDATA:
        parseFrame_OneAnalog();
        break;
    case ESERVERFRAME_ANALOGDATAS:
        parseFrame_Analogs();
        break;
    case ESERVERFRAME_FILE:
		parseFrame_File();
        break;
    default:
        reset_receive();
        return false;
    }
    return true;
}
bool CClientThread::parseFrame_OneAnalog() {
    /*
    *	|同步字0x68	|   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
    *	|数据项Id	|   字节2~5   - 【4个字节】
    *	|数据值		|   字节6~13  - 【8个字节，double】
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    if (!readData(m_nFrameLength)) {
        return false;
    }

    if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
        return true;
    }

    ks07_08_Server_OneAnalogData* pOneAnalog = reinterpret_cast<ks07_08_Server_OneAnalogData*>(m_pReceiveBuf);
    pOneAnalog->NetToHost();
    if (pOneAnalog->data.uDataId < pDataVector->getSize()) {
        pDataVector->setData(pOneAnalog->data.uDataId + 1, pOneAnalog->data.dValue);
    }
    std::cout << std::endl << "收到服务端数据: dataId = " << pOneAnalog->data.uDataId
              << "（序号从0开始）, value = " << pOneAnalog->data.dValue << std::endl;
    reset_receive();
    return true;
}
bool CClientThread::parseFrame_Analogs() {
    /*
    *	|同步字0x68  |   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ESERVERFRAME_ANALOGDATAS】
    *	|起始数据序号	|   字节2~5   - 【4个字节】
    *	|本帧数据个数	|   字节6~9   - 【4个字节】
    *	|1个数据		|   字节10~17 - 【8个字节，double】
    *	|...		|
    *	|1个数据		|   字节n~n+7 - 【8个字节，double】
    */
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return false;
	}
	
	const type_uint8 uchStart = 0x68;		// 同步字
    type_int32 readCount = 0;
    // 1. 读取数据
    if (!readData(m_nFrameLength)) {
        return false;
    }
    if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
        return true;
    }
    // 5. 当数据收全后，解析并保存数据、切换工作状态
    type_uint32 idx = 0;
    type_char* pBuf = m_pReceiveBuf + sizeof(ks07_08_Server_AnalogDatas);
	type_uint32 uDataId = 0;
    type_double dValue = 0.0;
 	uDataId = m_u32StartId;
	while (idx < m_u32CurrentFrameDataCount) {
        dValue = ns_train::NetworkToHost(*(reinterpret_cast<type_double*>(pBuf)));
		if (uDataId < pDataVector->getSize()) {
			pDataVector->setData(uDataId + 1, dValue);
		}
        idx++;
 		uDataId++;
		pBuf += sizeof(dValue);
    }
    //std::cout << std::endl << "服务器端返回数据: dataId = " << uDataId << "（序号从0开始）, value = " << dValue << std::endl;
    reset_receive();
    return true;
}

bool CClientThread::parseFrame_File() {
    /*
    *	|同步字0x68               	|   【1个字节】- 字节0
    *	|帧类型                   	|   【1个字节】- 字节1
    *   |本帧数据总长度				|	【4个字节】- 字节2~5
    *	|文件名                   	|   【MAX_FILE_NAME_LENGTH】- 字节6~69
    *	|是否最后一帧              	|   【1个字节】- 字节70
    *	|本帧数据在文件中的起始偏移量|   【4个字节】- 字节71~74
    *	|本帧文件内容长度           |   【4个字节】- 字节75~78
    *	|本帧文件内容              	|   【N个字节】- 字节79~
    */


    // 1. 读取数据
    if (!readData(m_nFrameLength)) {
        return false;
    }
    if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
        return true;
    }
    // 5. 当数据收全后，解析并保存数据、切换工作状态
    type_uint32 idx = 0;
    type_char* pBuf = m_pReceiveBuf + sizeof(ks07_08_Server_File);
    ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pReceiveBuf);
    pFile->NetToHost();
    std::string strFileName = "$PROJECT_DEV_HOME/test/chapter07/ks07_08/recv/";
	strFileName += pFile->szFileName;
	strFileName = ns_train::getPath(strFileName.c_str());
    std::string strDirectory = ns_train::getDirectory(strFileName.c_str());
    ns_train::mkDir(strDirectory);
    ns_train::CPPFile file;
    file.setFileName(strFileName);
    std::string strOpenMode = "ab";
    if (0 == pFile->u32Offset) {
        strOpenMode = "wb";
    }
    if (!file.open(strOpenMode.c_str())) {
        return false;
    }
    file.seekToPosition(pFile->u32Offset, SEEK_SET);
    file.write(m_pReceiveBuf+ sizeof(ks07_08_Server_File), pFile->u32DataLength);
    file.close();
    //std::cout << std::endl << "服务器端返回数据: dataId = " << uDataId << "（序号从0开始）, value = " << dValue << std::endl;
    reset_receive();
    return true;
}
void CClientThread::addCommand_GetData(type_uint32 uDataId) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_lstGetId.push_back(uDataId);
}
void CClientThread::addCommand_SetData(type_uint32 uDataId, type_double dValue) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_lstSetIdValue.push_back(s_IDValue(uDataId, dValue));
}
void CClientThread::addCommand_GetFile(const std::string& str) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_strFileName = str;
}
bool CClientThread::makeFrameAndSend() {
    if (NULL == m_pSerial) {
        return false;
    }
    int nWrite = 0;
    bool bSend = false;
    int nSend = 0;
	// 1. 发送-"设置数据"帧
    { // 为了使用自动锁，需要用花括号建立一个局部栈
        /*
        *	|同步字0x68    |   字节0     - 【1个字节】
        *	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_SETDATA】
        *	|数据项Id      |   字节2~5   - 【4个字节】
        *	|数据值        |   字节6~13  - 【8个字节，double】
        */

        ks07_08_Client_SetData* pSetData = reinterpret_cast<ks07_08_Client_SetData*>(m_pSendBuf);
        ns_train::CPPMutexLocker mutexLocker(&m_mutex);
        std::list<s_IDValue>::iterator ite = m_lstSetIdValue.begin();
        if (ite != m_lstSetIdValue.end()) { // 取第一个
            bSend = true;
            pSetData->header.byStart = 0x68;
            pSetData->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_SETDATA);
            pSetData->data.uDataId = (*ite).nId;
            pSetData->data.dValue = (*ite).dValue;
            pSetData->HostToNet();
            nSend = sizeof(ks07_08_Client_SetData);
            nWrite = m_pSerial->write(m_pSendBuf, nSend);
            m_lstSetIdValue.erase(ite);
        }
    }
    // 2. 发送-"获取数据"帧
    if (!bSend) {
        /*
        *	|同步字0x68    |   字节0     - 【1个字节】
        *	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
        *	|数据项Id      |   字节2~5   - 【4个字节】
        */

        ks07_08_Client_GetData* pGetData = reinterpret_cast<ks07_08_Client_GetData*>(m_pSendBuf);
        ns_train::CPPMutexLocker mutexLocker(&m_mutex);
        std::list<type_uint32>::iterator ite = m_lstGetId.begin();
        if (ite != m_lstGetId.end()) {
            bSend = true;
			nSend = 0;
            pGetData->header.byStart = 0x68;
            pGetData->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_GETDATA);
            pGetData->uDataId = *ite;
            pGetData->HostToNet();
            nSend = sizeof(ks07_08_Client_GetData);
            nWrite = m_pSerial->write(m_pSendBuf, nSend);
            m_lstGetId.erase(ite);
        }
    }
    // 3. 发送-“获取文件”帧
    if (!bSend) {
        /*
        *	|同步字0x68    |   字节0     - 【1个字节】
        *	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
        *   |文件名        |   字节2-    - 【MAX_FILE_NAME_LENGTH】
        */
        std::string strFileName;
        {
            ns_train::CPPMutexLocker mutexLocker(&m_mutex);
            strFileName = m_strFileName;
        }
        if (strFileName.length() == 0) {
            return true;
        }
        ks07_08_Client_GetFile* pGetFile = reinterpret_cast<ks07_08_Client_GetFile*>(m_pSendBuf);
        bSend = true;
        nSend = 0;
        pGetFile->header.byStart = 0x68;
        pGetFile->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_GETFILE);
        strcpy(pGetFile->szFileName, strFileName.c_str());

        nSend = static_cast<int>(sizeof(ks07_08_Client_GetFile)) ;
        pGetFile->HostToNet();
        nWrite = m_pSerial->write(m_pSendBuf, nSend);
        {
            ns_train::CPPMutexLocker mutexLocker(&m_mutex);
            m_strFileName.clear();
        }
    }
    return true;
}
