﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: serverthread.cpp
\brief 服务器端线程类实现。该线程负责处理客户端连接及交互。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/
#include "serverthread.h"

#include <iostream>
#include <string.h>

#include "base/basedll/base_class.h"
#include "base/basedll/base_file.h"
#include "com_serial.h"
#include "datavector.h"
#include "host_network_api.h"
#include "ks07_08/ks07_08_frametype.h"
#include "service_api.h"
#include "ks07_08/ks07_08_struct.h"

static const int c_nReceiveBufferLength = 12800; // 需要注意:确保内存够用，也就是能够至少容纳一帧接收到的数据。
static const int c_nSendBufferLength = 12800;	 // 需要注意:确保内存够用，也就是能够至少容纳一帧发送的数据。

CServerThread::CServerThread() {
    m_pReceiveBuf = new char[c_nReceiveBufferLength];
    memset(m_pReceiveBuf, 0, c_nReceiveBufferLength*sizeof(char));
    m_pSendBuf = new char[c_nSendBufferLength];
    memset(m_pSendBuf, 0, c_nSendBufferLength*sizeof(char));
    reset_receive();
    reset_send();
}
void CServerThread::run() {

    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return;
    }
    if (NULL == getParam()) {
        return;
    }
    m_pSerial = static_cast<ns_train::CPPSerial*>(getParam()); // 串口通信对象
    if (NULL == m_pSerial) {
        return;
    }
    bool bOpened = false;
    while (isWorking()) {
        ns_train::api_sleep(10); // 睡眠时间不易过长，否则影响后续代码。
        keepAlive(); // 请安，汇报心跳
        bOpened = m_pSerial->open(); // 首先打开串口
        if (!bOpened) {
            continue;
        }

        makeFrameAndSend();         // 检查有没有需要发送的命令

        recvAndParseFrame();        // 接收数据、解帧
     }
    freeBuffer(); // 释放资源
}
void CServerThread::reset_receive() {
    m_nReceivedCount = 0;
    m_byFrameType = static_cast<type_uint8>(ECLIENTFRAME_INVALID);
    m_eFrameStatus = EFRAME_SYNCHRONIZATION;
    m_nFrameLength = 0;
    m_u32StartId = 0;
    m_u32CurrentFrameDataCount = 0;
}
void CServerThread::reset_send() {
    m_bSendData = false;
    m_bSendFile = false;
}
void CServerThread::freeBuffer() {
    delete[] m_pReceiveBuf;
    m_pReceiveBuf = NULL;

    delete[] m_pSendBuf;
    m_pSendBuf = NULL;
}
bool CServerThread::recvAndParseFrame() {
    /*
    *	|同步字0x68	|   【1个字节】- 字节0
    *	|帧类型		|   【1个字节】- 字节1
    *	|...		|    不同的“帧类型”有不同的帧格式，请查看后续代码
    */

    // 接收数据并解析
    bool bOk = true;
    switch (m_eFrameStatus) {
    case EFRAME_SYNCHRONIZATION:
        bOk = parseFrame_Synchronization();
        break;
    case EFRAME_GET_FRAMETYPE:
        bOk = parseFrame_GetFrameType();
        break;
    case EFRAME_GET_FRAMELENGTH:
        bOk = parseFrame_GetFrameLength();
        break;
    case EFRAME_PARSEDATA:
        parseFrame_Parse();
        break;
    default:
        reset_receive();
        return true;
    }
    return true;
}
bool CServerThread::readData(type_int32 nAllData) {
    // 2. 读取数据
    type_int32 readCount = m_pSerial->read(m_pReceiveBuf + m_nReceivedCount, nAllData - m_nReceivedCount);
    // 3. 没有数据则返回
    if (0 == readCount) { // 没有数据, 等下一个循环再尝试读取。
        return true;
    }
    // 4. 出现错误则将数据复位，返回false
    if (readCount < 0) { // 出现错误
        reset_receive();
        return false;
    }
    // 5. 更新接收到的字节数
    m_nReceivedCount += readCount;
    return true;
}
bool CServerThread::parseFrame_Synchronization() {
    reset_receive();
    const type_uint8 uchStart = 0x68;		// 同步字
    // 1. 设置需要读取的总字节数
    type_int32 nAllData = static_cast<int>(sizeof(uchStart)); // 【同步字】
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) { // 已经读取完毕
		if (m_pReceiveBuf[0] == 0x68) {
			m_eFrameStatus = EFRAME_GET_FRAMETYPE;		// 切换到下一个工作状态
		}
		else {
			reset_receive();
		}
    }
    return true;
}
bool CServerThread::parseFrame_GetFrameType() {
    const type_uint8 uchStart = 0x68;		// 同步字
    // 1. 设置需要读取的总字节数
    type_int32 nAllData = static_cast<int>(sizeof(uchStart) + sizeof(m_byFrameType)); // 【同步字】 + 【帧类型】
    if (!readData(nAllData)) {
        return false;
    }
    // 6. 当数据收全后，解析并保存数据、切换工作状态
    if (m_nReceivedCount == nAllData) { // 已经读取完毕
        ks07_08_FrameHeader* pHeader = reinterpret_cast<ks07_08_FrameHeader*>(m_pReceiveBuf);
        pHeader->NetToHost();
        m_byFrameType = pHeader->byFrameType; // 保存【帧类型】
        m_eFrameStatus = EFRAME_GET_FRAMELENGTH;		// 切换到下一个工作状态
    }
    return true;
}

bool CServerThread::parseFrame_GetFrameLength() {
    switch (m_byFrameType) {
    case ECLIENTFRAME_SETDATA:
        parseFrame_GetFrameLength_SetData();
        break;
    case ECLIENTFRAME_GETDATA:
        parseFrame_GetFrameLength_GetData();
        break;
    case ECLIENTFRAME_GETFILE:
        parseFrame_GetFrameLength_GetFile();
        break;
    default:
        reset_receive();
        return false;
    }
    return true;
}

bool CServerThread::parseFrame_GetFrameLength_SetData() {
    /*
    *	|同步字0x68|   字节0     - 【1个字节】
    *	|帧类型    |   字节1     - 【1个字节，ECLIENTFRAME_SETDATA】
    *	|数据项Id  |   字节2~5   - 【4个字节】
    *	|数据值    |   字节6~13  - 【8个字节，double】
    */

    // 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
    m_nFrameLength = sizeof(ks07_08_Client_SetData);
    m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
    return true;
}

bool CServerThread::parseFrame_Parse() {
    switch (m_byFrameType) {
    case ECLIENTFRAME_SETDATA:
        parseFrame_SetData();
        break;
    case ECLIENTFRAME_GETDATA:
        parseFrame_GetData();
        break;
    case ECLIENTFRAME_GETFILE:
		parseFrame_GetFile();
        break;
    default:
        reset_receive();
        return false;
    }
    return true;
}
bool CServerThread::parseFrame_SetData() {
    /*
    *	|同步字0x68	|   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_SETDATA】
    *	|数据项Id    |   字节2~5   - 【4个字节】
    *	|数据值		|   字节6~13  - 【8个字节，double】
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    if (!readData(m_nFrameLength)) {
        return false;
    }
    if (m_nReceivedCount < m_nFrameLength) {
            return false;
    }
    ks07_08_Client_SetData* pSetData = reinterpret_cast<ks07_08_Client_SetData*>(m_pReceiveBuf);
    pSetData->NetToHost();
    if (pSetData->data.uDataId < pDataVector->getSize()) {
        pDataVector->setData(pSetData->data.uDataId + 1, pSetData->data.dValue);
    }
	reset_receive();
    std::cout << std::endl << "收到客户端数据: dataId = " 
                << pSetData->data.uDataId << "（序号从0开始）, value = " 
                << pSetData->data.dValue << std::endl;
    return true;
}
bool CServerThread::parseFrame_GetFrameLength_GetData() {
    /*
    *	|同步字0x68	|   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
    *	|数据项Id  	|   字节2~5   - 【4个字节】
    */

    // 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
    m_nFrameLength = sizeof(ks07_08_Client_GetData);
    m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
    return true;
}
bool CServerThread::parseFrame_GetFrameLength_GetFile() {
    /*
    *	|同步字0x68	|   字节0     - 【1个字节】
    *	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
    *   |文件名     |   字节2-    - 【MAX_FILE_NAME_LENGTH】
    */
	m_nFrameLength = sizeof(ks07_07_Client_GetFile); 
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
    return true;
}
bool CServerThread::parseFrame_GetData() {
    /*
    *	|同步字0x68|   字节0     - 【1个字节】
    *	|帧类型    |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
    *	|数据项Id  |   字节2~5   - 【4个字节】
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    if (!readData(m_nFrameLength)) {
            return false;
        }
    if (m_nReceivedCount < m_nFrameLength) {
        return false;
    }
	reset_receive();   // 先复位，再赋值，否则数据会被冲掉
    ks07_08_Client_GetData* pGetData = reinterpret_cast<ks07_08_Client_GetData*>(m_pReceiveBuf);
    pGetData->NetToHost();
    m_u32SendDataId = pGetData->uDataId;
    m_bSendData = true;
	std::cout << std::endl << "收到客户端getdata命令: dataId = " << m_u32SendDataId << "（序号从0开始）。 " << std::endl;

    return true;
}
bool CServerThread::parseFrame_GetFile() {
    /*
    *	|同步字0x68    |   字节0     - 【1个字节】
    *	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
    *   |文件名        |   字节2-    - 【MAX_FILE_NAME_LENGTH】
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    if (!readData(m_nFrameLength)) {
            return false;
    }
    if (m_nReceivedCount < m_nFrameLength) {
        return false;
    }
    reset_receive();   // 先复位，再赋值，否则数据会被冲掉
    ks07_08_Client_GetFile* pGetFile = reinterpret_cast<ks07_08_Client_GetFile*>(m_pReceiveBuf);
    pGetFile->NetToHost();
    //m_strFileName = reinterpret_cast<char*>(&pGetFile->u32FileNameLength+sizeof(pGetFile->u32FileNameLength));
	m_strFileName = pGetFile->szFileName;
    m_bSendFile = true;
    std::cout << std::endl << "收到客户端getfile命令: 文件名 = " << m_strFileName << std::endl;

    return true;
}

bool CServerThread::makeFrameAndSend() {

    bool bOk = makeFrameAndSend_OneAnalog(); // 根据客户端请求，返回单个模拟量数据

    bOk = makeFrameAndSend_File(); // 根据客户端请求，返回文件

    ns_train::CPPTime tmNow;
    tmNow.setCurrentTime();
    if ((tmNow.getTime() - m_tmLastSendAllData.getTime()) > 5) { // 每5秒发送一次全部模拟量数据
        bOk &= makeFrameAndSend_Analogs();
    }
    return bOk;
}
bool CServerThread::makeFrameAndSend_OneAnalog() {
    /*
    *	|同步字0x68|   字节0     - 【1个字节】
    *	|帧类型    |   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
    *	|数据项Id  |   字节2~5   - 【4个字节】
    *	|数据值    |   字节6~13  - 【8个字节，double】
    */
    if (!m_bSendData) {
        return true;
    }
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    ks07_08_Server_OneAnalogData* pAnalog = reinterpret_cast<ks07_08_Server_OneAnalogData*>(m_pSendBuf);
    pAnalog->header.byStart = 0x68; // 同步字
    pAnalog->header.byFrameType = static_cast<type_uint8>(ESERVERFRAME_ONEANALOGDATA);
    m_bSendData = false; // 把标志放倒。
    type_double dValue = 0.0;
    int nSend = 0;
    if (m_u32SendDataId < pDataVector->getSize()) {
        pDataVector->getData(m_u32SendDataId + 1, dValue);
		std::cout << std::endl << "发送单个数据给客户端: dataId = " << m_u32SendDataId << "（序号从0开始）, value = " << dValue << std::endl;
        pAnalog->data.uDataId = m_u32SendDataId;
        pAnalog->data.dValue = dValue;
        nSend += sizeof(ks07_08_Server_OneAnalogData);
        pAnalog->HostToNet();
        int nWrite = m_pSerial->write(m_pSendBuf, nSend);
		m_u32SendDataId = 0;
        if (nWrite < 0) {
            m_pSerial->close();
            return false;
        }
    }
    return true;
}
bool CServerThread::makeFrameAndSend_Analogs() {
    /*
    *	|同步字0x68	|   【1个字节】- 字节0
    *	|帧类型		|   【1个字节】- 字节1
    *	|起始数据序号	|   【4个字节】- 字节2~5
    *	|本帧数据个数	|   【4个字节】- 字节6~9
    *	|1个数据		|   【8个字节】- 字节10~17
    *	|...		|
    *	|1个数据		|   【8个字节】- 字节n~n+7
    */
    CDataVector* pDataVector = CDataVector::instance();
    if (NULL == pDataVector) {
        return false;
    }
    // 1. 定义通信数据帧中各个字段对应的变量。请注意，要保证变量的数据类型跟通信帧中对应的字段完全一致。
    bool bError = false;
    ks07_08_Server_AnalogDatas* pAnalogDatas = reinterpret_cast<ks07_08_Server_AnalogDatas*>(m_pSendBuf);
    type_double dValue = 0.0;		// 数据的值

    // 2. 计算该缓冲区能容纳多少有效数据
    const type_int32 nFrameMaxDataCount = (c_nSendBufferLength - sizeof(ks07_08_Server_AnalogDatas))/ sizeof(dValue); // 一帧最多可发送的数据个数

    // 3. 定义相关变量。
    type_uint32 nAllDataCount = pDataVector->getSize();	// 待发送数据总个数
    type_uint32 nSendDataCount = 0;		// 已经发送的数据个数
    type_uint32 u32StartDataId = 0;		// 起始数据序号
    type_uint32 u32CurrentFrameDataCount = 0;   // 本帧数据个数
    type_int32 nFrameLength = 0;        // 本帧数据总长度
    type_uint8 *pBuf = NULL;            // 发送缓冲区游标指针

    while (nSendDataCount < nAllDataCount) { // 数据都发出去了吗？
        pAnalogDatas->header.byStart = 0x68;		// 同步字
        pAnalogDatas->header.byFrameType	= type_uint8(ESERVERFRAME_ANALOGDATAS);	// 帧类型
        pAnalogDatas->u32StartDataId = u32StartDataId;	// 起始数据序号

        if ((nAllDataCount-nSendDataCount) < nFrameMaxDataCount) {
            u32CurrentFrameDataCount = nAllDataCount-nSendDataCount;
        }
        else {
            u32CurrentFrameDataCount = nFrameMaxDataCount;
        }
        pAnalogDatas->u32CurrentFrameDataCount = u32CurrentFrameDataCount;	// 本帧数据个数        
        nFrameLength = static_cast<int>(sizeof(ks07_08_Server_AnalogDatas) + u32CurrentFrameDataCount * sizeof(dValue)); // 本帧数据长度

        pBuf = reinterpret_cast<type_uint8*>(m_pSendBuf+sizeof(ks07_08_Server_AnalogDatas)); // 将指针移动到数据区
        for (type_uint32 idx = 0; idx < u32CurrentFrameDataCount; idx++) {
            pDataVector->getData(idx+1+u32StartDataId, dValue);
            *(reinterpret_cast<type_double*>(pBuf)) = ns_train::HostToNetwork(dValue);	// 数据
            pBuf += sizeof(dValue);
        }
		u32StartDataId += pAnalogDatas->u32CurrentFrameDataCount;
		
        pAnalogDatas->HostToNet(); // 转换字节序。在此行代码之后，不可以继续使用 pAnalogDatas 中的变量。
        // 数据组帧完毕，开始发送。
        int nWrite = m_pSerial->write(m_pSendBuf, nFrameLength);
        if (nWrite > 0) {
            nSendDataCount += u32CurrentFrameDataCount; // 更新已发送数据个数。
        }
        else {
            m_pSerial->close();
            bError = true;
            return false;
        }
        // ns_train::ii_sleep(1); // 如果数据量比较多，导致循环次数太多，可能导致CPU占用率高，那么可以用睡眠释放CPU。
    }
    m_tmLastSendAllData.setCurrentTime();
    return true;
}

bool CServerThread::makeFrameAndSend_File() {
    /*
    *	|同步字0x68               |   【1个字节】- 字节0
    *	|帧类型                   |   【1个字节】- 字节1
    *	|本帧数据总长度             |   【4个字节】- 字节2~5
    *	|文件名                   	|   【MAX_FILE_NAME_LENGTH】- 字节6~69
    *	|是否最后一帧              	|   【1个字节】- 字节70
    *	|本帧数据在文件中的起始偏移量|   【4个字节】- 字节71~74
    *	|本帧文件内容长度           |   【4个字节】- 字节75~78
    *	|本帧文件内容              	|   【N个字节】- 字节79~
    */
	if (!m_bSendFile) {
		return false;
	}
    // 1. 定义通信数据帧中各个字段对应的变量。请注意，要保证变量的数据类型跟通信帧中对应的字段完全一致。
    bool bError = false;
    ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pSendBuf);
    type_double dValue = 0.0;		// 数据的值

   // 2. 计算该缓冲区能容纳多少有效数据
    const type_int32 nFrameMaxDataCount = c_nSendBufferLength - sizeof(ks07_08_Server_File); // 一帧最多可发送的数据个数

    // 3. 定义相关变量。
    ns_train::CPPFile file;
    type_int32 nAllDataCount = 0;	// 待发送数据总个数
    type_int32 nSendDataCount = 0;	// 已经发送的数据个数
    type_int32 nFrameLength = 0;    // 本帧数据总长度
    type_uint8 *pBuf = NULL;        // 发送缓冲区游标指针
    type_uint32 u32CurrentFrameDataCount = 0;   // 本帧数据个数
	int nSend = 0;
	int nWrite = 0;	
    std::string strFileName;
    std::string strDirectory = ns_train::getDirectory("$PROJECT_DEV_HOME/test/chapter07/ks07_08/");
    ns_train::mkDir(strDirectory);
    strFileName = strDirectory + "/" + m_strFileName;
	strFileName = ns_train::getPath(strFileName);
    file.setFileName(strFileName);
    bool bOk = file.open("rb");
    if (!bOk) {
        reset_send();
        return false;
    }
    type_size fileSize = file.getSize();
    strcpy(pFile->szFileName, m_strFileName.c_str()); // 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapter07/ks07_08
    pFile->byLastFrame = 0;     // 是否最后一帧
    pFile->u32Offset = 0;       // 本帧数据在文件中的起始偏移量
    pFile->u32DataLength = 0;   // 本帧文件内容长度

    nAllDataCount = static_cast<int>(fileSize);
    type_size sizeRead = 0;
    file.seekToBegin(); // 将文件游标移动到指定位置

	while (nSendDataCount < nAllDataCount) { // 数据都发出去了吗？
        pFile->header.byStart = 0x68;
        pFile->header.byFrameType = static_cast<type_uint8>(ESERVERFRAME_FILE);
        strcpy(pFile->szFileName, m_strFileName.c_str()); // 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapter07/ks07_08
        if ((nAllDataCount-nSendDataCount) < nFrameMaxDataCount) {
            u32CurrentFrameDataCount = nAllDataCount-nSendDataCount;// 本帧文件内容长度
            pFile->byLastFrame = 1; // 是否最后一帧
        }
        else {
            u32CurrentFrameDataCount = nFrameMaxDataCount;// 本帧文件内容长度
            pFile->byLastFrame = 0; // 是否最后一帧
        }
        pFile->u32DataLength = u32CurrentFrameDataCount;
        pFile->u32Offset = nSendDataCount;   // 本帧数据在文件中的起始偏移量
        nFrameLength = pFile->u32FrameLength = sizeof(ks07_08_Server_File)+pFile->u32DataLength; // 本帧数据总长度

        sizeRead = file.read(m_pSendBuf+ sizeof(ks07_08_Server_File), u32CurrentFrameDataCount); // 读取文件
        if (sizeRead != pFile->u32DataLength) {
            return false;
        }
        pFile->HostToNet(); // 转换字节序。在此行代码之后，不可以继续使用 pAnalogDatas 中的变量。
        // 数据组帧完毕，开始发送。
		for (int nCurrentFrameSend = 0; nCurrentFrameSend < nFrameLength;) {
			nSend = nFrameLength - nCurrentFrameSend;
			nWrite = m_pSerial->write(m_pSendBuf + nCurrentFrameSend, nSend);
			
			if (nWrite < 0) {
				file.close();
				m_pSerial->close();
				reset_send();
				return false;
			}
			else {
				nCurrentFrameSend += nWrite;
			}
		}
		nSendDataCount += u32CurrentFrameDataCount;	// 更新已发送数据个数。		
		// ns_train::ii_sleep(1); // 如果数据量比较多，导致循环次数太多，可能导致CPU占用率高，那么可以用睡眠释放CPU。
    }
    file.close();
	m_bSendFile = false;
    m_tmLastSendAllData.setCurrentTime();
    return true;
}
