﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: serverthread.h
\brief 服务器端处理线程类定义。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/

#pragma once

#include "base_class.h"

#include <string>
namespace ns_train {
	class CPPSerial;
}

class CServerThread : public ns_train::CPPThread{
public:
    CServerThread();

    /**
    * @brief  线程主循环函数。内部必须采用while(isWorking())作为主循环。
    * @return 无
    */
    virtual void run() override;
private:
    /**
    * @brief  将待发送数据进行组帧并发送
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool makeFrameAndSend();
    /**
    * @brief  将待发送数据进行组帧并发送-一个模拟量数据
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool makeFrameAndSend_OneAnalog();
    /**
    * @brief  将待发送数据进行组帧并发送-模拟量数据集合
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool makeFrameAndSend_Analogs();
    /**
    * @brief  将待发送数据进行组帧并发送-文件
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool makeFrameAndSend_File();
    /**
    * @brief  接收数据并解帧。
    * @param[in] pSerial 串口对象。
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool recvAndParseFrame();
    /**
    * @brief  读取指定长度的数据
    * @param[in] nAllData 待读取的字节总数，含已经收到的字节数。
    * @return true：成功，false：未读完
    */
    bool readData(type_int32 nAllData);
    /**
    * @brief  获取帧类型。
    * @return 无
    */
    bool parseFrame_GetFrameType();
    /**
    * @brief  搜索同步字
    * @return 无
    */
    bool parseFrame_Synchronization();
    /**
    * @brief  获取帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength();
    /**
    * @brief  计算“设置数据”帧(ECLIENTFRAME_SETDATA)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_SetData();
    /**
    * @brief  计算“获取数据”帧(ECLIENTFRAME_GETDATA)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_GetData();
    /**
    * @brief  计算“获取文件”帧(ECLIENTFRAME_GETFILE)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_GetFile();
    /**
    * @brief  解析数据。
    * @return 无
    */
    bool parseFrame_Parse();
    /**
    * @brief  解析数据帧：ECLIENTFRAME_SETDATA
    * @return 无
    */
    bool parseFrame_SetData();
    /**
    * @brief  解析数据帧：ECLIENTFRAME_GETDATA
    * @return 无
    */
    bool parseFrame_GetData();
    /**
    * @brief  解析数据帧： ECLIENTFRAME_GETFILE
    * @return 无
    */
    bool parseFrame_GetFile();
    /**
	* @brief  复位与接收数据相关的变量。
	* @return 无
	*/
	void reset_receive();
	/**
	* @brief  复位与发送数据相关的变量。
	* @return 无
	*/
	void reset_send();
    /**
    * @brief  释放申请的内存
    * @return 无
    */
    void freeBuffer();
private:
    enum EFRAME_STATUS {
        EFRAME_SYNCHRONIZATION = 0, // 搜索同步字
        EFRAME_GET_FRAMETYPE,       // 搜索帧类型
        EFRAME_GET_FRAMELENGTH,     // 获取帧长度
        EFRAME_PARSEDATA,           // 解析数据
    };
    ns_train::CPPMutex m_mutex;
    ns_train::CPPSerial* m_pSerial;
    type_int32 m_nReceivedCount;				// 已经接收到的数据字节数
    EFRAME_STATUS m_eFrameStatus;               // 数据解帧状态
    type_uint8 m_byFrameType;					// 帧类型
    type_int32 m_nFrameLength;					// 帧长度
    type_uint32 m_u32CurrentFrameDataCount;		// 本帧数据个数
    type_uint32 m_u32StartId;					// 起始数据序号
    char *m_pReceiveBuf;                        // 接收缓冲区
    char *m_pSendBuf;                           // 发送缓冲区
    bool m_bSendData;                           // true:有数据等待发送，false：无数据等待发送
    type_uint32 m_u32SendDataId;				// 待发送数据项的序号
    ns_train::CPPTime m_tmLastSendAllData;      // 上次发送全部模拟量数据的时间
    std::string m_strFileName;                  // 客户端请求的文件名，不带路径。
    bool m_bSendFile;                           // true:有文件等待发送，false：无文件等待发送
};


