﻿/*! 
/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolcommand.cpp
\brief 规约命令实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang)
\Date 2018/10
*/
#include "ksui_01/protocolcommand.h"
#include "base/basedll/base_class.h"
#include "com_serial.h"
#include "ksui_01/ksui_01_struct.h"
#include "ksui_01/ksui_01_frametype.h"

#include <list>
#include <iostream>

CProtocolCommand::CProtocolCommand() {
	m_pSerial = NULL;
	m_pSendBuffer = NULL;
}

///////////////////////////////////////////////////
CCommandAck::CCommandAck() {
	m_n32FrameIndex = -1;
}

bool CCommandAck::exec() { // 发送-"确认"帧
	int nWrite = 0;
	int nSend = 0;
	/*
	*	|同步字0x68		|   字节0		- 【1个字节】
	*	|帧类型			|   字节1		- 【1个字节，ECLIENTFRAME_ACK】
	*	|帧序号			|   字节2~5		- 【4个字节】
	*	|待确认的帧序号	|   字节6~9		- 【4个字节】
	*   |校验码			|   字节10		- 【1个字节】
	*/
	if (NULL == m_pSerial) {
		return false;
	}
	ksui_01_Client_Ack* pAck = reinterpret_cast<ksui_01_Client_Ack*>(m_pSendBuffer);
	pAck->header.byStart = 0x68;
	pAck->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_ACK);
	pAck->header.n32FrameIndex = 0; // 未启用
	pAck->n32FrameIndexAck = m_n32FrameIndex;
	pAck->checkSum = pAck->calculateChecksum();
	pAck->HostToNet();
	nSend = sizeof(ksui_01_Client_Ack);
	nWrite = m_pSerial->write(m_pSendBuffer, nSend);
	std::cout << "发送确认帧，帧序号=" << m_n32FrameIndex << std::endl;
	return true;
}
///////////////////////////////////////////////////
CCommandSetData::CCommandSetData() {
	m_uDataId = 0;
	m_dValue = 0;
}

bool CCommandSetData::exec() { // 发送-"设置数据"帧
	int nWrite = 0;
	int nSend = 0;
	/*
	*	|同步字0x68    |   字节0     - 【1个字节】
	*	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_SETDATA】
	*	|帧序号		   |   字节2~5	 - 【4个字节】
	*	|数据项Id      |   字节6~9   - 【4个字节】
	*	|数据值        |   字节10~17 - 【8个字节，double】
	*   |校验码		   |   字节18	 - 【1个字节】
	*/
	if (NULL == m_pSerial) {
		return false;
	}
	ksui_01_Client_SetData* pSetData = reinterpret_cast<ksui_01_Client_SetData*>(m_pSendBuffer);
	pSetData->header.byStart = 0x68;
	pSetData->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_SETDATA);
	pSetData->data.uDataId = m_uDataId;
	pSetData->data.dValue = m_dValue;
	pSetData->checkSum = pSetData->calculateChecksum();
	pSetData->HostToNet();
	nSend = sizeof(ksui_01_Client_SetData);
	nWrite = m_pSerial->write(m_pSendBuffer, nSend);
	return true;
}

///////////////////////////////////////////////////
CCommandGetData::CCommandGetData() {
	m_uDataId = 0;
}

bool CCommandGetData::exec() { // 发送-"获取数据"帧
	/*
	*	|同步字0x68    |   字节0     - 【1个字节】
	*	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|数据项Id      |   字节2~5   - 【4个字节】
	*/
	if (NULL == m_pSerial) {
		return false;
	}
	
	int nWrite = 0;
	int nSend = 0;
	ksui_01_Client_GetData* pGetData = reinterpret_cast<ksui_01_Client_GetData*>(m_pSendBuffer);
	nSend = 0;
	pGetData->header.byStart = 0x68;
	pGetData->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_GETDATA);
	pGetData->uDataId = m_uDataId;
	pGetData->checkSum = pGetData->calculateChecksum();
	pGetData->HostToNet();
	nSend = sizeof(ksui_01_Client_GetData);
	nWrite = m_pSerial->write(m_pSendBuffer, nSend);
	return true;
}

///////////////////////////////////////////////////
CCommandGetFile::CCommandGetFile() {
}

bool CCommandGetFile::exec() {// 发送-“获取文件”帧	

	/*
	*	|同步字0x68    |   字节0     - 【1个字节】
	*	|帧类型        |   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|文件名长度    |   字节2~5   - 【4个字节】
	*   |文件名        |   字节6-    - 【长度由前一个字段决定】
	*/
	int nWrite = 0;
	int nSend = 0;
	if (NULL == m_pSerial) {
		return false;
	}
	if (m_strFileName.length() == 0) {
		return true;
	}
	ksui_01_Client_GetFile* pGetFile = reinterpret_cast<ksui_01_Client_GetFile*>(m_pSendBuffer);
	nSend = 0;
	pGetFile->header.byStart = 0x68;
	pGetFile->header.byFrameType = static_cast<type_uint8>(ECLIENTFRAME_GETFILE);
	strcpy(pGetFile->szFileName, m_strFileName.c_str());

	nSend = static_cast<int>(sizeof(ksui_01_Client_GetFile));
	pGetFile->checkSum = pGetFile->calculateChecksum();
	pGetFile->HostToNet();
	nWrite = m_pSerial->write(m_pSendBuffer, nSend);
	return true;
}
