﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: event.cpp
\brief 事件管理类/事件监听类
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/

#include "base_class.h"
#include <algorithm>
#include "base_api.h"
#include "service_api.h"
namespace ns_train {

////////////////////////////////////////////////////
//  CPPEventManager
CPPEventManager::CPPEventManager() {
}

CPPEventManager::~CPPEventManager() {
}

bool CPPEventManager::addEvent(type_int32 nEventId) {
    CPPMutexLocker tMutex(&m_mutex, false);
    if (tMutex.tryLock(5000)) {
        m_eventsQueue.push(nEventId);
		return true;
	}
	return false;
}

void CPPEventManager::clearQueue() {
    CPPMutexLocker tMutex(&m_mutex);
    if (tMutex.tryLock(5000)) {
        std::queue<type_int32> q;
        m_eventsQueue = q;
    }
}

type_int32 CPPEventManager::getOneEvent() {
    type_int32 nEventId = -1;
    CPPMutexLocker tMutex(&m_mutex, false);
    if (tMutex.tryLock(5000)) {
        if (m_eventsQueue.size() > 0) {
            nEventId = m_eventsQueue.front();
            m_eventsQueue.pop();
        }
	}
    return nEventId;
}

////////////////////////////////////////////////////////
//  CPPEventListener
CPPEventListener::CPPEventListener() {
}

CPPEventListener::CPPEventListener(CPPEventManager *pEventManager) {
    setEventManager(pEventManager);
}

bool CPPEventListener::setEventManager(CPPEventManager *pEventManager) {
    m_pEventManager = pEventManager;
    return true;
}
type_int32 CPPEventListener::waitForEvent(type_uint32 nTimeout) {
    type_int32 nEventId = -1;
    type_uint32 nWaitTime = 0;
    STimeGapMS tBeginTime = get_current_time_ms();
    STimeGapMS tEndTime;

    // type_bool bFlag = VALUE_TRUE;
    while (true) {
        nEventId = m_pEventManager->getOneEvent();
        if (nEventId >= 0) {
            return nEventId;
		}

        if (nTimeout == 0) {
            return -1;
        }
        api_sleep(10);
		nWaitTime += 10;
        if (nWaitTime >= nTimeout) {
            return -1;
		}

        if (nTimeout > 0) {
            tEndTime = get_current_time_ms();
			if ((tEndTime.nSecond - tBeginTime.nSecond)*1000 +
                tEndTime.nMSecond - tBeginTime.nMSecond >= nTimeout) {
                return -1;
			}
		}
        else {
            return -1;
		}
	}
}
}

