﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: ini.cpp
\brief ini格式配置文件定义实现文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
*/
#include "base_ini.h"
#include <algorithm>

#include "base_api.h"

namespace ns_train {
static const type_int32 c_bufLen = 10240;
CPPIniFile::CPPIniFile() :m_bIsOpen(false) {
}

CPPIniFile::~CPPIniFile() {
}

//  OK
type_bool CPPIniFile::setFileName(const std::string &strName) {
    if (strName.length() == 0) {
        return false;
    }
    std::string strFileName = getPath(strName);
    char pTmp[c_bufLen] = {'\0'};
    std::string tStr;
    m_file.setFileName(strFileName);
    if (!m_file.open("r"))	{	// 文件不存在则返回false
        m_file.close();
        return false;
    }
    // 文件长度=0则返回false
    type_size t_nFileLen = m_file.getSize();
    if (t_nFileLen <= 0)  {
        m_file.close();
        return false;
    }
    m_bIsOpen = true;
    m_vecKey.clear();
    m_mapKey2Subkey.clear();
    m_mapSubkey2Value.clear();
    type_size idx = 0;
    std::string strKey;
    std::string strSubkey;
    std::string strValue;
    std::string str;
    // 将文件内容读入行链表
    while (m_file.readLine(pTmp, c_bufLen)) {
        tStr = pTmp;
        if (tStr.at(tStr.length() - 1) == '\n') {
            tStr = tStr.substr(0, tStr.length()-1);
        }
        //tStr = trim(tStr); // 删除首尾的空格
        if (tStr.at(0) == ';') {			// 注释
            m_vecKey.push_back(tStr);
        }
        else if ((tStr.at(0) == '[') &&
                (tStr.at(tStr.length()-1) == ']')) { // 主键
            tStr = tStr.substr(1, tStr.length()-2);
            if (find(m_vecKey.begin(), m_vecKey.end(), tStr) == m_vecKey.end()) {
                m_vecKey.push_back(tStr);
                strKey = tStr;
            }
        }
        else if (strKey.length()>0 && (idx = tStr.find("="))>0) { // 子键 
            strSubkey = tStr.substr(0, idx);
            strValue = tStr.substr(idx + 1, tStr.length() - idx - 1);
            m_mapKey2Subkey[strKey].push_back(strSubkey);
            str = strSubkey + "@" + strKey;
            m_mapSubkey2Value[str] = strValue;
        }
    }
    m_file.close();
    return true;
}


type_bool CPPIniFile::getBool(const std::string &strKey, const std::string &strSubkey, type_bool i_bDefault, type_bool* o_bRet) {
    std::string t_strResult;
    if (o_bRet != NULL) {
        *o_bRet = true;
    }
    // 抽取结果字符串
    if (!getValueString(strKey, strSubkey, t_strResult))  {
        if (o_bRet != NULL) {
            *o_bRet = false;
        }
    }
    // 将结果字符串转换为整形数
    if (t_strResult == "Y" || t_strResult == "y") {
        return true;
    }
    else if (t_strResult == "N" || t_strResult == "n") {
        return false;
    }
    else {
        if (o_bRet != NULL) {
            *o_bRet = false;
        }
        return i_bDefault;
    }
}
//  OK
type_bool CPPIniFile::setBool(const std::string &strKey, const std::string &strSubkey, type_bool i_bValue) {
    if ((0 == strKey.length()) || (0 == strSubkey.length())) {
        return false;
    }
    std::string strValue;

    if (i_bValue) {
        strValue = "Y";
    }
    else {
        strValue = "N";
    }
    if (!setValueString(strKey, strSubkey, strValue)) {
        return false;
    }
    return writeToFile();
}
//  OK
type_int32 CPPIniFile::getInt32(const std::string &strKey, const std::string &strSubkey, type_int32 i_nDefault, type_bool* o_bRet) {
    std::string t_strResult;
    if (o_bRet != NULL) {
        *o_bRet = true;
    }
    // 抽取结果字符串
    if (!getValueString(strKey, strSubkey, t_strResult)) {
        if (o_bRet != NULL) {
            *o_bRet = false;
        }
        return i_nDefault;
    }
    // 将结果字符串转换为整形数
    return atoi(t_strResult.c_str());
}
//  ok
type_bool CPPIniFile::setInt32(const std::string &strKey, const std::string &strSubkey, type_int32 i_nValue) {
    if ((0 == strKey.length()) || (0 == strSubkey.length())) {
        return false;
    }
    std::string strValue;
    char buffer[128] = {'\0'};
    sprintf(buffer, "%d", i_nValue);
    strValue = buffer;
    if (!setValueString(strKey, strSubkey, strValue)) {
        return false;
    }
    return writeToFile();
}
//  ok
type_double CPPIniFile::getReal(const std::string &strKey, const std::string &strSubkey, type_double i_fDefault, type_bool* o_bRet) {
    std::string t_strResult;
    if (o_bRet != NULL) {
        *o_bRet = true;
    }
    // 抽取结果字符串
    if (!getValueString(strKey, strSubkey, t_strResult)) {
        if (o_bRet != NULL) {
            *o_bRet = false;
        }
        return i_fDefault;
    }
    // 将结果字符串转换为浮点数
    return atof(t_strResult.c_str());
}

type_bool CPPIniFile::setReal(const std::string &strKey, const std::string &strSubkey, type_double i_fValue) {
    if ((0 == strKey.length()) || (0 == strSubkey.length())) {
        return false;
    }
    std::string strValue;
    char buffer[128] = { '\0' };
    sprintf(buffer, "%f", i_fValue);
    strValue = buffer;
    if (!setValueString(strKey, strSubkey, strValue)) {
        return false;
    }
    return writeToFile();
}

std::string CPPIniFile::getString(const std::string &strKey, const std::string &strSubkey, const std::string &strDefault, type_bool* o_bRet) {
    std::string tStr;
    if (o_bRet != NULL) {
        *o_bRet = true;
    }
    // 抽取结果字符串
    if (!getValueString(strKey, strSubkey, tStr)) {
        if (o_bRet != NULL) {
            *o_bRet = false;
        }
        return strDefault;
    }
    return tStr;
}
//  ok
type_bool CPPIniFile::setString(const std::string &strKey, const std::string &strSubkey, const std::string &strValue) {
    if ((0 == strKey.length()) || (0 == strSubkey.length())) {
        return false;
    }

    if (!setValueString(strKey, strSubkey, strValue)) {
        return false;
    }

    return writeToFile();
}


//  OK
type_bool CPPIniFile::writeToFile(void) {
    if (!m_file.open("w")) {
        return false;
    }
    // 将缓存区写入文件
    m_file.seekToBegin();
    std::string str;
    std::string strKey;
    std::string strSubkey;
    bool bKey = false;
    std::map<std::string, std::list<std::string> >::iterator iteMap;
    std::list<std::string>::iterator iteLst;
    std::map<std::string, std::string>::iterator iteMap2;
    for (size_t i = 0; i < m_vecKey.size(); ++i) {
        bKey = false;
        strKey = m_vecKey[i];
        if (';' != str[0]) {
            bKey = true;
            str = "[" + strKey + "]";
        }
        else {
            str = strKey;
        }
        str += "\n";
        m_file.write(str.c_str(), str.length());
        if (!bKey) {
            continue;
        }
        iteMap = m_mapKey2Subkey.find(strKey);
        if (iteMap == m_mapKey2Subkey.end()) {
            continue;
        }
        iteLst = iteMap->second.begin();
        while (iteLst != iteMap->second.end()) {
            strSubkey = *iteLst;
            str = strSubkey + "@" + strKey;
            iteMap2 = m_mapSubkey2Value.find(str);
            if (iteMap2 != m_mapSubkey2Value.end()) {
                str = strSubkey + "=";
                str += (*iteMap2).second;
                str += "\n";
                m_file.write(str.c_str(), str.length());
            }
            iteLst++;
        }
    }
    m_file.close();
    return true;
}

//  OK
type_bool CPPIniFile::getValueString(const std::string &strKey, const std::string &strSubkey, std::string &o_pValue) {
    std::map<std::string, std::string>::iterator iteMap2;
    std::string  str = strSubkey + "@" + strKey;
    iteMap2 = m_mapSubkey2Value.find(str);
    if (iteMap2 != m_mapSubkey2Value.end()) {
        o_pValue = (*iteMap2).second;
        return true;
    }
    return false;
}

//  OK
type_bool CPPIniFile::setValueString(const std::string &strKey, const std::string &strSubkey, const std::string &strValue) {
    if ((0 == strKey.length()) || (0 == strKey.length())) {
        return false;
    }
    std::map<std::string, std::list<std::string> >::iterator iteMap;
    std::list<std::string>::iterator iteLst;
    iteMap = m_mapKey2Subkey.find(strKey);
    if (iteMap == m_mapKey2Subkey.end()) {
        m_vecKey.push_back(strKey);
        std::list<std::string> lst;
        lst.push_back(strSubkey);
        m_mapKey2Subkey[strKey] = lst;
    }
    else {
        std::list<std::string> lst = m_mapKey2Subkey[strKey];
        if (find(lst.begin(), lst.end(), strSubkey) == lst.end()) {
            lst.push_back(strSubkey);
        }
        m_mapKey2Subkey[strKey] = lst;
    }
    std::string str = strSubkey + "@" + strKey;
    m_mapSubkey2Value[str] = strValue;   
    return true;
}

}
