﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: thread_windows.cpp
\brief 线程类 windows 平台实现
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
*/

#include "base_class.h"
#include <windows.h>
#include "service_api.h"

namespace ns_train {
extern "C" {static DWORD WINAPI start_thread(LPVOID pParam);}

class CPPThreadData {
public:

    CPPThreadData():m_hThread(NULL), m_dwThreadID(0){
    }

    ~CPPThreadData() {
        closeHandle();
    }
    void closeHandle() {
        if (m_hThread != NULL) 	{
           // ExitThread(0);
           // TerminateThread(m_hThread, 0);
            CloseHandle(m_hThread); // 关闭句柄
            m_hThread = NULL;
        }
        m_dwThreadID = 0;
    }
    bool init(CPPThread *pThread) 	{
        m_hThread = CreateThread(NULL, 0, start_thread, (void*)pThread, 0, &m_dwThreadID);
        if (m_hThread == NULL) {
            return false;
        }
        else {
            return true;
        }
    }
    int getThreadId() {
        return (int)m_dwThreadID;
    }
    static void start(CPPThread *pThread) {
        pThread->run();
        pThread->closeThreadHandle();
    }
private:
    HANDLE m_hThread;
    DWORD m_dwThreadID;
};

extern "C" {
    static DWORD WINAPI start_thread(LPVOID pParam) 	{
        CPPThreadData::start((CPPThread*)pParam);
        return 0;
    }
}

CPPThread::CPPThread() : m_pParam(NULL), m_bWorking(false),m_bFinished(true), m_tAliveTimeLimit(ETHREAD_ALIVE_MAXTIME) {
    m_pThreadData = new CPPThreadData;
}

CPPThread::~CPPThread() {
    exitThread();
    delete m_pThreadData;
}

bool CPPThread::start(void* pParam) {
    if (pParam != NULL) {
        m_pParam = pParam;
    }
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bFinished = false; // 应先设置为false再调用init（），否则将导致run()还没有退出时，exitThread()却先执行完毕了。正确的顺序是exitThread（）一直等待run（）完全退出后才返回。
        m_bWorking = true;   // 线程的主循环中，以while(isWorking())进行判断，因此在启动线程之前，应将m_bWorking设置为true。
    }
    bool bOK = m_pThreadData->init(this);
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bWorking = bOK;
    }
    if (bOK) {
        keepAlive();
    }
    return bOK;
}
bool CPPThread::isAlive(void) {
    type_time_t tCurrent = get_current_time();
    CPPMutexLocker locker(&m_mtxAliveTime);
    return ((tCurrent - m_tAlive) < m_tAliveTimeLimit) ? true : false;
}
void CPPThread::keepAlive() {
    type_time_t tCurrent = get_current_time();
    ns_train::CPPMutexLocker locker(&m_mtxAliveTime);
    m_tAlive = tCurrent;
}
type_time_t CPPThread::getAliveTime(void) {
    ns_train::CPPMutexLocker locker(&m_mtxAliveTime);
    return m_tAlive;
}
bool CPPThread::isWorking() {
    m_mtxRunning.lock();
    bool bWorking = m_bWorking;
    m_mtxRunning.unlock();
    return bWorking;
}

void CPPThread::closeThreadHandle() {
   m_pThreadData->closeHandle();
   {
       CPPMutexLocker locker(&m_mtxRunning);
       m_bWorking = false;
       m_bFinished = true;
   }
}

void CPPThread::exitThread() {
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bWorking = false;
    }
    while (!m_bFinished) { // 等待线程完全退出。
        api_sleep(10);
    }
    closeThreadHandle();
}  
int CPPThread::getThreadId() {
    return m_pThreadData->getThreadId();
}
void CPPThread::initializeThreadId(void) {

}

}
