﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: com_serialproxy_windows.h
\brief 串口通讯类实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/
#pragma once

#include "com_serial.h"
#include "windows.h"
/////////////////////////////////////////////////////////////////////////
namespace ns_train {

class CSerialProxy {
public:
	/**
	* @brief 构造函数
	*/
	CSerialProxy(const s_serial_config& config);
	/**
	* @brief 析构函数
	*/
	~CSerialProxy();

	/**
	* @brief 打开串口
	* @return true:打开成功,false:打开失败
	*/
	bool open();
	/**
	* @brief 串口是否已打开
	* @return true:已打开,false:未打开
	*/
	bool isOpen();
	/**
	* @brief 关闭串口
	* @return true:关闭成功,false:关闭失败
	*/
	bool close();
	/**
	* @brief 设置串口通信参数。使用m_serialConfig设置通信参数。
	* @return true:设置成功，false:设置失败
	*/
	bool setSerialConfig();
	/**
	* @brief 设置串口通信参数
	* @param[in] serialConfig 串口通信参数
	* @return true:设置成功，false:设置失败
	*/
	bool setSerialConfig(const s_serial_config& serialConfig);
	/**
	* @brief 获取串口通信参数
	* @return 串口通信参数
	*/
	s_serial_config getSerialConfig() {
		return m_serialConfig;
	}
	/**
	* @brief 设置波特率
	* @param[in] baudrate 波特率
	* @return true:设置成功，false:设置失败
	*/
	bool setBaudrate(type_uint16 baudrate);
	/**
	* @brief 获取串口通信参数
	* @return 串口通信参数
	*/
	type_uint16 getBaudrate();
	/**
	* @brief 从串口中接收数据
	* @param[out] data 接收缓冲区
	* @param[in] cout 准备接收的字节数
	* @return 从串口中实际接收到的数据字节数
	*/
	int readData(
		char* data,
		int count);
	/**
	* @brief 向串口中发送数据
	* @param[in] data 发送缓冲区
	* @param[in] cout 准备发送的字节数
	* @return 实际向串口中写入的字节数
	*/
	int writeData(
		char* data,
		int count); 
private:
	HANDLE m_hCom; // 句柄	
	ESERIAL_STATE m_eSerialState; // 串口状态	
	s_serial_config m_serialConfig;// 串口通信参数
};

}
