﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: app.cpp
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/
#include "app.h"
#include <iostream>
#include <string>
#include <string.h>
#include "base/basedll/service_api.h"
#include "base/basedll/base_api.h"
#include "base/basedll/base_class.h"
#include "base/basedll/host_network_api.h"
#include "base/communicate/com_serial.h"

#include "customtype.h"

ns_train::CPPMutex g_mutexApp;
CApp* CApp::instance() {
    ns_train::CPPMutexLocker mutexLocker(&g_mutexApp);
    static CApp app;
    return &app;
}

CApp::CApp() : m_bProcRun(true), m_bTerminal(true), m_bHelp(false), m_pSerial(NULL){
}

CApp::~CApp() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
}
void CApp::setArgs(int argc, char* argv[]) {
    /*
    * 此处的默认值即是程序的默认运行状态。
    * 当用户未输入命令参数时，程序以默认方式运行，取决于此处定义的默认值。
    */
    bool bTerminal = false;	 // true:程序以终端（前台）方式运行, false:程序以后台服务方式运行。

    // 解析命令参数, 所有命令参数均以"-"开头。
    for (int i = 1; i < argc; i++) {
        if (strcmp(argv[i], "-term") == 0) {
            bTerminal = true;
        }
        else if (strcmp(argv[i], "-help") == 0) {
            m_bHelp = true;
        }
#ifdef WIN32
        else if (_stricmp(argv[i], "-regist") == 0) {
            std::cout << ">>> 模块注册。" << std::endl;
            ns_train::regist("ks07_03_client", "ks07_03_client",  "ks07_03_client服务");
            exit(0);
        }
        else if (_stricmp(argv[i], "-unregist") == 0) {
            std::cout << ">>> 模块注销。" << std::endl;
            ns_train::unregist("ks07_03_client");
            exit(0);
        }
#endif
    }
    if (m_bHelp) {
        std::cout << "=======================================" << std::endl;
        std::cout << ">>> ks07_03_client -term\t\t模块以终端方式运行。" << std::endl;
        std::cout << ">>> ks07_03_client -help\t\t显示帮助信息。" << std::endl;
    }

    if (bTerminal) {
        std::cout << "=======================================" << std::endl;
        std::cout << ">>> 服务器模块启动。" << std::endl;
        std::cout << ">>> 模块以终端方式运行。" << std::endl;
        setTerminalFlag(bTerminal);
    }
}

void CApp::setTerminalFlag(bool bTerminal) {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	m_bTerminal = bTerminal;
}

bool CApp::getTerminalFlag() {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	return m_bTerminal;
}

bool CApp::getRunningFlag() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    return m_bProcRun;
}

void CApp::setRunningFlag(bool bRunningFlag) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_bProcRun = bRunningFlag;
}

void CApp::prepare(void) {
    // 初始化
    ns_train::s_serial_config serialConfig;
#ifdef WIN32
    strcpy(serialConfig.strDevicename, "\\\\.\\COM1");
#else
    strcpy(serialConfig.strDevicename, "/dev/tty01");
#endif
	serialConfig.u32Baudrate = 1200;	// 波特率
	serialConfig.byDatabits = 8;	// 8位数据位
	serialConfig.chParity = 'N';	// 无校验
	serialConfig.chStopbit = 0;		// 1位停止位
	m_pSerial = new ns_train::CPPSerial(serialConfig);
    bool bOpen = m_pSerial->open();
    if (m_bTerminal) {
        std::cout << "打开串口:" << serialConfig.strDevicename << std::endl;
        std::cout << "波特率:" << serialConfig.u32Baudrate << std::endl;
        std::cout << "数据位:" << serialConfig.byDatabits << std::endl;
        std::cout << "校验:" << serialConfig.chParity << std::endl;
		if (bOpen) {
			std::cout << "打开串口成功。" << std::endl;
		}
		else {
			std::cout << "打开串口失败。" << std::endl;
		}
	}
}

bool CApp::initialize() {
    /* do initialize work */
    prepare();

    return true;
}
/**
* @brief 模拟程序退出前的清理工作。
* @return true：成功，false:失败
*/
void CApp::beforeExit(void) {
    m_pSerial->close();
    if (!m_bHelp) {
        std::cout << ">>> 模块退出。" << std::endl;
        //char ch = '\0';
        //std::cin >> ch;
    }    
}

void CApp::exec(void) {
	// 因为案例的需要，删除人机交互的处理部分，仅读取数据。
    while (getRunningFlag()) {
           ns_train::api_sleep(10); // 接收数据时，周期要短，降低延迟。提高读取速率，及时读取数据。
           recvData();
	}
}

void CApp::recvData(void) {
    type_uint32 uDataId = 0;    // 数据项的Id
    type_double dValue = 0.f;   // 数据值
    int readCount = 0;
    const int nAllData = 2*(sizeof(uDataId) + sizeof(dValue)); // 有两组数据要读取
    char buf[nAllData] = {'\0'};
    bool bError = false;
    if (!m_pSerial->isOpen()) {
        m_pSerial->open();
	}
    while (readCount < nAllData) {
        int nRead = m_pSerial->read(buf + readCount, nAllData - readCount);
        if (nRead >= 0) {
            readCount += nRead;
        }
        else {
            m_pSerial->close();
            bError = true;
            break;
        }
    }
    if (bError) {
        return;
    }
    char *pBuf =  buf;
    // 解析第1个数据项序号、数据值
    uDataId = ns_train::NetworkToHost(*((type_uint32*)pBuf));
    pBuf += sizeof(uDataId);
    dValue = ns_train::NetworkToHost(*((type_double*)pBuf));
    pBuf += sizeof(dValue);
    std::cout << "id1 = " << uDataId << ", value1 = " << dValue << std::endl;
    // 解析第2个数据项序号、数据值
    uDataId = ns_train::NetworkToHost(*((type_uint32*)pBuf));
    pBuf += sizeof(uDataId);
    dValue = ns_train::NetworkToHost(*((type_double*)pBuf));
    pBuf += sizeof(dValue);
    std::cout << "id2 = " << uDataId << ", value2 = " << dValue << std::endl;
}
