﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: datavector.h
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/
#pragma once
#include "customtype.h"
#include "base_class.h"

struct SData {
    type_uint32 nId;        // 数据项Id
    type_double dataValue;  // 数据当前值
    type_double maxValue;   // 数据最大值
    type_double minValue;   // 数据最小值
    SData();
};

/**
* @brief 数据容器类。用来模拟对双精度浮点数数组的操作。
*/
class CDataVector {
public:
    /**
   * @brief  单体对象访问接口
   * @return 单体对象的指针
   */
    static CDataVector *instance();

    /**
    * @brief 初始化。此处模拟从数据库中读取数据，并非真正从数据库中读取。
    * @return true:成功,false:失败
    */
    bool initialize();
    /**
    * @brief 获取数据个数。
    * @return 数据个数。
    */
    type_uint32 getSize();
    /**
    * @brief 获取指定Id的数据。
    * @param[in] nId 数据Id，从1开始。
    * @param[out] dValue 数据值
    * @return true:成功,false:失败，失败的原因一般是数据Id不存在
    */
    bool getData(type_uint32 nId, type_double &dValue);
    /**
    * @brief 更新指定Id的数据。
    * @param[in] nId 数据id，从1开始
    * @param[in] dNewValue 新的数据值
    * @return true:成功,false:失败，失败的原因一般是数据Id不存在
    */
    bool setData(type_uint32 nId, type_double dNewValue);
private:
    /**
    * @brief  构造函数，定义为私有的目的是防止他人使用该类构造对象。
    * @return 无
    */
    CDataVector();
    /**
    * @brief  拷贝构造函数，定义为私有的目的是防止编译器调用默认的拷贝构造
    *         函数隐式构造该类的对象。
    * @return 无
    */
    CDataVector(const CDataVector&);

    /**
    * @brief  析构函数，定义为私有的目的是防止他人用delete语句删除单体对象。
    * @return 无
    */
    virtual ~CDataVector();
private:
    ns_train::CPPMutex m_mutex;     // 用来保护对数据区m_pDataVector的访问
    SData *m_pDataVector;           // 数据区
    type_uint32 m_nSize;            // 数据个数  
};
