﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolobject.cpp
\brief 通信规约对象实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang)
\Date 2018/10
*/
#include "ks07_08_client_protocolobject.h"

#include <iostream>
#include <string.h>

#include "base/basedll/base_class.h"
#include "base/basedll/base_file.h"
#include "com_serial.h"
#include "host_network_api.h"
#include "ks07_08/datavector.h"
#include "ks07_08/ks07_08_frametype.h"
#include "ks07_08/ks07_08_struct.h"
#include "protocolcommand.h"
#include "service_api.h"

static const int c_nReceiveBufferLength = 12800;	// 需要注意:确保内存够用，也就是能够至少容纳一帧接收到的数据。
static const int c_nSendBufferLength = 12800;		// 需要注意:确保内存够用，也就是能够至少容纳一帧发送的数据。

s_IDValue::s_IDValue(type_uint32 id, type_double value) {
	nId = id; dValue = value;
}

////////////////////////////////////////////////////////////////////////////////////////////
CClientProtocolObject::CClientProtocolObject() {
	makeBuffer();
	reset_receive();
	reset_send();
}

CClientProtocolObject::~CClientProtocolObject() {
}

void CClientProtocolObject::reset_receive() {
	CProtocolObject::reset_receive();
	m_nReceivedCount = 0;
	m_byFrameType = ESERVERFRAME_ANALOGDATAS;
	m_eFrameStatus = EFRAME_SYNCHRONIZATION;
	m_nFrameLength = 0;
	m_u32StartId = 0;
	m_u32CurrentFrameDataCount = 0;
}

void CClientProtocolObject::reset_send() {
	m_bWaitAck = false;
}

void CClientProtocolObject::makeBuffer() {
	m_pReceiveBuf = new char[c_nReceiveBufferLength];
	memset(m_pReceiveBuf, 0, c_nReceiveBufferLength * sizeof(char));
	m_pSendBuf = new char[c_nSendBufferLength];
	memset(m_pSendBuf, 0, c_nSendBufferLength * sizeof(char));
	m_pReSendBuf = new char[c_nSendBufferLength];
	memset(m_pReSendBuf, 0, c_nSendBufferLength * sizeof(char));
}

void CClientProtocolObject::setSerialObject(ns_train::CPPSerial* pSerial) {
	CProtocolObject::setSerialObject(pSerial);
	ns_train::s_serial_config config = pSerial->getSerialConfig();
	config.u32RecvBufferLength = c_nReceiveBufferLength;
	config.u32SendBufferLength = c_nSendBufferLength;
	if (pSerial->isOpen()) {
		close();
	}
	pSerial->setSerialConfig(config);
}
bool CClientProtocolObject::open() {
	if (NULL != m_pSerial) {
		return CProtocolObject::open();
	}
	else {
		return false;
	}
}

void CClientProtocolObject::recvAndParseFrame() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节】
	*	|帧序号		|   字节2~5   - 【4个字节】
	*	|...		|   不同的“帧类型”有不同的帧格式，请查看后续代码
	*/
	// 接收数据并解析
	switch (m_eFrameStatus) {
    case EFRAME_SYNCHRONIZATION:  // 搜索同步字
        parseFrame_Synchronization();
        break;
    case EFRAME_GET_FRAMETYPE:    // 确定帧类型
        parseFrame_GetFrameType();
        break;
    case EFRAME_GET_FRAMELENGTH:  // 确定帧长度
        parseFrame_GetFrameLength();
        break;
    case EFRAME_PARSEDATA:        // 接收数据并解析
		parseFrame_Parse();
		break;
	default:
		reset_receive();
		return;
	}
}

void CClientProtocolObject::parseFrame_Synchronization() {
	reset_receive();
	const type_uint8 uchStart = 0x68;		// 同步字
	// 1. 设置需要读取的总字节数
	type_int32 nAllData = static_cast<int>(sizeof(uchStart)); // 【同步字】
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) { // 已经读取完毕
		if (m_pReceiveBuf[0] == 0x68) {
			m_eFrameStatus = EFRAME_GET_FRAMETYPE;		// 切换到下一个工作状态
		}
		else {
			reset_receive();
		}
	}
}

void CClientProtocolObject::parseFrame_GetFrameType() {
	const type_uint8 uchStart = 0x68;		// 同步字
	// 1. 设置需要读取的总字节数
	type_int32 nAllData = static_cast<int>(sizeof(uchStart) + sizeof(m_byFrameType)); // 【同步字】 + 【帧类型】
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) { // 已经读取完毕
		ks07_08_FrameHeader* pHeader = reinterpret_cast<ks07_08_FrameHeader*>(m_pReceiveBuf);
		pHeader->NetToHost();
		m_byFrameType = pHeader->byFrameType;       // 保存【帧类型】
		m_eFrameStatus = EFRAME_GET_FRAMELENGTH;    // 切换到下一个工作状态
	}
}

void CClientProtocolObject::parseFrame_GetFrameLength() {
	switch (m_byFrameType) {
	case ESERVERFRAME_ONEANALOGDATA:
		parseFrame_GetFrameLength_OneAnalog();
		break;
	case ESERVERFRAME_ANALOGDATAS:
		parseFrame_GetFrameLength_Analogs();
		break;
	case ESERVERFRAME_FILE:
		parseFrame_GetFrameLength_File();
		break;
	default:
		reset_receive();
		return;
	}
}
void CClientProtocolObject::parseFrame_GetFrameLength_OneAnalog() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
	*	|帧序号		|   字节2~5   - 【4个字节】
	*	|数据项Id	|   字节6~9   - 【4个字节】
	*	|数据值		|   字节10~17 - 【8个字节，double】
	*   |校验码		|   字节18	  - 【1个字节】
	*/

	// 1. 设置需要读取的字节数。
	m_nFrameLength = sizeof(ks07_08_Server_OneAnalogData);
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
}

void CClientProtocolObject::parseFrame_GetFrameLength_Analogs() {
	/*
	*	|同步字0x68		|   字节0     - 【1个字节】
	*	|帧类型			|   字节1     - 【1个字节，ESERVERFRAME_ANALOGDATAS】
	*	|帧序号			|   字节2~5   - 【4个字节】
	*	|起始数据序号   |   字节6~9   - 【4个字节】
	*	|本帧数据个数   |   字节10~13 - 【4个字节】
	*	|1个数据        |   字节14~21 - 【8个字节，double】
	*	|...			|
	*	|1个数据        |   字节n~n+7 - 【8个字节，double】
	*   |校验码			|   字节n+8	  - 【1个字节】
	*/
	

	// 1. 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
	type_int32 nAllData = sizeof(ks07_08_Server_AnalogDatas);
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) {
		ks07_08_Server_AnalogDatas* pAnalogDatas = reinterpret_cast<ks07_08_Server_AnalogDatas*>(m_pReceiveBuf);
		pAnalogDatas->NetToHost();
		m_u32StartId = pAnalogDatas->u32StartDataId;
		m_u32CurrentFrameDataCount = pAnalogDatas->u32CurrentFrameDataCount;
		type_uint8 byCheckSum = 0;
		m_nFrameLength = static_cast<type_int32>(sizeof(ks07_08_Server_AnalogDatas) + m_u32CurrentFrameDataCount * sizeof(type_double)+sizeof(byCheckSum));
		m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
	}
}

void CClientProtocolObject::parseFrame_GetFrameLength_File() {
	/*
	*	|同步字0x68               	|   字节0		- 【1个字节】
	*	|帧类型                   	|   字节1		- 【1个字节】
	*	|帧序号						|   字节2~5		- 【4个字节】
	*   |本帧数据总长度				|	字节6~9		- 【4个字节】
	*	|文件名                   	|   字节10~73	- 【MAX_FILE_NAME_LENGTH】
	*	|是否最后一帧              	|   字节74		- 【1个字节】
	*	|本帧数据在文件中的起始偏移量|  字节75~82	- 【8个字节】
	*	|本帧文件内容长度           |   字节83~86	- 【4个字节】
	*	|本帧文件内容              	|   字节87~		- 【N个字节】
	*   |校验码						|   字节87+N	- 【1个字节】
	*/

	ks07_08_Server_File fileFrame;

	// 1. 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
	type_int32 nAllData = sizeof(ks07_08_Server_File);
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) {
		// 2. 设置需要读取的字节数。
		ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pReceiveBuf);
		m_nFrameLength = static_cast<type_int32>(pFile->u32FrameLength);
		m_nFrameLength = ns_train::NetworkToHost(m_nFrameLength);
		m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
	}
}

void CClientProtocolObject::parseFrame_Parse() {
	switch (m_byFrameType) {
	case ESERVERFRAME_ONEANALOGDATA:
		parseFrame_OneAnalog();
		break;
	case ESERVERFRAME_ANALOGDATAS:
		parseFrame_Analogs();
		break;
	case ESERVERFRAME_FILE:
		parseFrame_File();
		break;
	default:
		reset_receive();
		return;
	}
}

void CClientProtocolObject::parseFrame_OneAnalog() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
	*	|帧序号		|   字节2~5   - 【4个字节】
	*	|数据项Id	|   字节6~9   - 【4个字节】
	*	|数据值		|   字节10~17 - 【8个字节，double】
	*   |校验码		|   字节18	  - 【1个字节】
	*/
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
	// 1. 读取数据
    if (!readData(m_nFrameLength)) {
        return;
    }
	if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
		return;
	}
	ks07_08_Server_OneAnalogData* pOneAnalog = reinterpret_cast<ks07_08_Server_OneAnalogData*>(m_pReceiveBuf);
	pOneAnalog->NetToHost();
	type_uint8 byCheckSum = pOneAnalog->calculateChecksum();
	if (byCheckSum != pOneAnalog->checkSum) {
		std::cout << std::endl << "收到服务端数据: 校验错误." << std::endl;
		reset_receive();
		return;
	}
	if (pOneAnalog->data.uDataId < pDataVector->getSize()) {
		pDataVector->setData(pOneAnalog->data.uDataId + 1, pOneAnalog->data.dValue);
	}
	std::cout << std::endl << "收到服务端数据: dataId = " << pOneAnalog->data.uDataId
		<< "（序号从0开始）, value = " << pOneAnalog->data.dValue << std::endl;
	reset_receive();
}

void CClientProtocolObject::parseFrame_Analogs() {
	/*
	*	|同步字0x68		|   字节0     - 【1个字节】
	*	|帧类型			|   字节1     - 【1个字节，ESERVERFRAME_ANALOGDATAS】
	*	|帧序号			|   字节2~5   - 【4个字节】
	*	|起始数据序号	|   字节6~9   - 【4个字节】
	*	|本帧数据个数	|   字节10~13 - 【4个字节】
	*	|1个数据		|   字节14~21 - 【8个字节，double】
	*	|...			|
	*	|1个数据		|   字节n~n+7 - 【8个字节，double】
	*   |校验码			|   字节n+8	  - 【1个字节】
	*/
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}

	// 1. 读取数据
    if (!readData(m_nFrameLength)) {
        return;
    }
	if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
		return;
	}
	// 5. 当数据收全后，解析并保存数据、切换工作状态
	type_uint32 idx = 0;
	type_char* pBuf = NULL;
	type_uint32 uDataId = 0;
	type_double dValue = 0.0;
	ks07_08_Server_AnalogDatas* pAnalogDatas = reinterpret_cast<ks07_08_Server_AnalogDatas*>(m_pReceiveBuf);
	pBuf = m_pReceiveBuf + sizeof(ks07_08_Server_AnalogDatas);
	// 先执行一遍循环，计算校验码
	while (idx < m_u32CurrentFrameDataCount) {
		ns_train::NetworkToHost(pBuf, sizeof(type_double));		
		idx++;
		pBuf += sizeof(type_double);
	}
	type_uint8 byCheckSum = 0;
	byCheckSum = pAnalogDatas->calculateChecksum(m_nFrameLength);// 根据接收到的数据计算出来的校验码
	type_uint8 byCheckSum1 = pAnalogDatas->getCheckSum(m_nFrameLength); // 对方送过来的校验码
	if (byCheckSum != byCheckSum1) {
		std::cout << std::endl << "收到服务端数据: 校验错误." << std::endl;
		reset_receive();
		return;
	}

	pBuf = m_pReceiveBuf + sizeof(ks07_08_Server_AnalogDatas);
	idx = 0;
	// 再执行一遍循环，获取数据
	uDataId = m_u32StartId;
	while (idx < m_u32CurrentFrameDataCount) {
		dValue = *(reinterpret_cast<type_double*>(pBuf)); // 第一遍循环中已经转换过字节序 
		if (uDataId < pDataVector->getSize()) {
			pDataVector->setData(uDataId + 1, dValue);
		}
		idx++;
		uDataId++;
		pBuf += sizeof(dValue);
	}
	//std::cout << std::endl << "服务器端返回数据: dataId = " << uDataId << "（序号从0开始）, value = " << dValue << std::endl;
	reset_receive();
}

void CClientProtocolObject::parseFrame_File() {
    /*
    *	|同步字0x68               	|   字节0		- 【1个字节】
    *	|帧类型                   	|   字节1		- 【1个字节】
	*	|帧序号						|   字节2~5		- 【4个字节】
	*   |本帧数据总长度				|	字节6~9		- 【4个字节】 
    *	|文件名                   	|   字节10~73	- 【MAX_FILE_NAME_LENGTH】
    *	|是否最后一帧              	|   字节74		- 【1个字节】
    *	|本帧数据在文件中的起始偏移量|  字节75~82	- 【8个字节】
    *	|本帧文件内容长度           |   字节83~86	- 【4个字节】
	*	|本帧文件内容              	|   字节87~		- 【N个字节】
  	*   |校验码						|   字节87+N	- 【1个字节】
	*/

	// 1. 读取数据
    if (!readData(m_nFrameLength)) {
        return;
    }
	if (m_nReceivedCount < m_nFrameLength) { // 数据还没有收全
		return;
	}
	// 5. 当数据收全后，解析并保存数据、切换工作状态
	ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pReceiveBuf);
	pFile->NetToHost();
	// todo
	{
		for (int i = 0; i < m_nFrameLength; i++) {
			//std::cout << (int)(m_pReceiveBuf[i]) << " "<< std::endl;
		}
	}
	type_uint8 byCheckSum = 0;
	byCheckSum = pFile->calculateChecksum(m_nFrameLength);// 根据接收到的数据计算出来的校验码
	type_uint8 byCheckSum1 = pFile->getCheckSum(m_nFrameLength); // 对方送过来的校验码
	if (byCheckSum != byCheckSum1) {
		std::cout << std::endl << "收到服务端文件数据: 校验错误。" << std::endl;
		reset_receive();
		return;
	}
	else {
		std::cout << std::endl << "收到服务端文件数据。校验正确。帧序号=" << pFile->header.n32FrameIndex << std::endl;
		if (pFile->byLastFrame) {
			std::cout << std::endl << "接收服务端文件完毕。" << std::endl;
		}
	}
	static bool bError = true;
	if (!bError) {
		CCommandAck* pCommand = new CCommandAck();
		pCommand->setFrameIndex(pFile->header.n32FrameIndex);
		pushCommand(pCommand);
	}
	if (bError) {
		bError = false;
	}

	std::string strFileName = "$PROJECT_DEV_HOME/test/chapter07/ks07_08/recv/";
	strFileName += pFile->szFileName;
	strFileName = ns_train::getPath(strFileName.c_str());
	std::string strDirectory = ns_train::getDirectory(strFileName.c_str());
	ns_train::mkDir(strDirectory);
	ns_train::CPPFile file;
	file.setFileName(strFileName);
	std::string strOpenMode = "ab";
	if (0 == pFile->u64Offset) {
		strOpenMode = "wb";
	}
	if (!file.open(strOpenMode.c_str())) {
		return;
	}
	file.seekToPosition(pFile->u64Offset, SEEK_SET);
	file.write(m_pReceiveBuf + sizeof(ks07_08_Server_File), pFile->u32DataLength);
	file.close();
	//std::cout << std::endl << "服务器端返回数据: dataId = " << uDataId << "（序号从0开始）, value = " << dValue << std::endl;
	reset_receive();
}

void CClientProtocolObject::makeFrameAndSend() {
	takeOneCommandAndExecute();
}