﻿/*! 
/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolobject.cpp
\brief 通信规约对象实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang)
\Date 2018/10
*/
#include "protocolobject.h"
#include "com_serial.h"
#include "base/basedll/base_class.h"
#include "ks07_08/protocolcommand.h"

#include <iostream>

static const int C_LIMIT_WAIT_TIME = 5;			 // 等待客户端Ack（确认帧）的时间

CProtocolObject::CProtocolObject() {
	m_pSerial = NULL;
	m_pReceiveBuf = NULL;
	m_pSendBuf = NULL;
	m_pReSendBuf = NULL;
	m_nReceivedCount = 0;
	m_nResendLength = 0;
	m_nResendCount = 0;
	m_eFrameStatus = EFRAME_SYNCHRONIZATION;
	m_byFrameType = 0;
	m_nFrameLength  = 0;
	m_bWaitAck = 0;
	m_bResend = false;
	m_nWaitAckTime = 5; // 5秒
}
CProtocolObject::~CProtocolObject() {
	freeBuffer();
}

void CProtocolObject::reset_receive() {
	m_nReceivedCount = 0;
	m_byFrameType = 0;
	m_eFrameStatus = EFRAME_SYNCHRONIZATION;
	m_nFrameLength = 0;
}
void CProtocolObject::reset_send() {
}

void CProtocolObject::makeBuffer() {

}

void CProtocolObject::freeBuffer() {
	if (NULL != m_pReceiveBuf) {
		delete[] m_pReceiveBuf;
		m_pReceiveBuf = NULL;
	}
	if (NULL != m_pSendBuf) {
		delete[] m_pSendBuf;
		m_pReceiveBuf = NULL;
	}
}

bool CProtocolObject::open() {
	return m_pSerial->open();
}

bool CProtocolObject::isOpen() {
	return m_pSerial->isOpen();
}
bool CProtocolObject::close() {
	return m_pSerial->close();
}
int CProtocolObject::read(char* data, int count) {
	return m_pSerial->read(data, count);
}

int CProtocolObject::write(char* data, int count) {
	return m_pSerial->write(data, count);
}

void CProtocolObject::setSerialObject(ns_train::CPPSerial* pSerial) {
	m_pSerial = pSerial;
}

ns_train::CPPSerial*  CProtocolObject::getSerialObject() const {
	return m_pSerial;
}

void CProtocolObject::recvAndParseFrame() {
	// 接收数据并解析
	switch (m_eFrameStatus) {
	case EFRAME_SYNCHRONIZATION:
		parseFrame_Synchronization();
		break;
	case EFRAME_GET_FRAMETYPE:
		parseFrame_GetFrameType();
		break;
	case EFRAME_GET_FRAMELENGTH:
		parseFrame_GetFrameLength();
		break;
	case EFRAME_PARSEDATA:
		parseFrame_Parse();
		break;
	default:
		reset_receive();
		return;
	}
}

void CProtocolObject::parseFrame_Synchronization() {
	reset_receive();	
}
void CProtocolObject::parseFrame_GetFrameType() {	
	m_eFrameStatus = EFRAME_GET_FRAMELENGTH;		// 切换到下一个工作状态
}

void CProtocolObject::parseFrame_GetFrameLength() {
	reset_receive();
}

void CProtocolObject::parseFrame_Parse() {
	reset_receive();
}

void CProtocolObject::makeFrameAndSend() {
}

void CProtocolObject::pushCommand(CCommandInterface* pCommand) {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	m_listCommands.push_back(pCommand);
}

CCommandInterface* CProtocolObject::takeCommand() {
	ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	CCommandInterface* pCommand = NULL;
	if (m_listCommands.size() > 0) {
		pCommand = *(m_listCommands.begin());
		m_listCommands.erase(m_listCommands.begin());
	}
	return pCommand;
}
bool CProtocolObject::readData(type_int32 nAllData) {
    // 2. 读取数据
    type_int32 readCount = m_pSerial->read(m_pReceiveBuf + m_nReceivedCount, nAllData - m_nReceivedCount);
    // 3. 没有数据则返回
    if (0 == readCount) { // 没有数据, 等下一个循环再尝试读取。
        return true;
    }
    // 4. 出现错误则将数据复位，返回false
    if (readCount < 0) { // 出现错误
        reset_receive();
        return false;
    }
    // 5. 更新接收到的字节数
    m_nReceivedCount += readCount;
    return true;
}
void CProtocolObject::reSend() {
	if (!m_bResend) {
		return;
	}
	ns_train::CPPTime tmNow;
	tmNow.setCurrentTime();
	if (m_nResendCount < 3) { // 重发次数还没有超限
		if ((tmNow.getTime() - m_tmBeginWaitAck.getTime()) < waitAckTime()) {
			return;
		}
	}
	else { // 重发次数超限，不再重发
		if ((tmNow.getTime() - m_tmBeginWaitAck.getTime()) < waitAckTime()) {
			return;
		}
		m_bResend = false;
		m_nResendLength = 0;
		m_nResendCount = 0;
		m_bWaitAck = false;
		reSendOverLimit();
		std::cout << "重发次数超限。" << std::endl;
		return;
	}
	int nCurrentFrameSend = 0;
	int nSend = 0;
	int nWrite = 0;
	for (nCurrentFrameSend = 0; nCurrentFrameSend < m_nResendLength;) {
		nSend = m_nResendLength - nCurrentFrameSend;
		nWrite = m_pSerial->write(m_pReSendBuf + nCurrentFrameSend, nSend);

		if (nWrite < 0) {
			m_pSerial->close();
			reset_send();
			return;
		}
		else {
			nCurrentFrameSend += nWrite;
		}
	}
	m_nResendCount++;
	std::cout << "超时，重发。" << m_nResendCount << "次" << std::endl;
	m_bWaitAck = true; // 需要等待客户端的Ack。
	m_tmBeginWaitAck.setCurrentTime(); // 更新“开始等待时间”

}

void CProtocolObject:: takeOneCommandAndExecute() {
	CCommandInterface* pCommand = takeCommand();
	if (NULL == pCommand) {
		return;
	}
	CProtocolCommand* pProtocolCommand = dynamic_cast<CProtocolCommand*>(pCommand);
	if (NULL != pProtocolCommand) {
		pProtocolCommand->setSendBuffer(m_pSendBuf);
		pProtocolCommand->setSerial(m_pSerial);
		pProtocolCommand->exec();
		delete pProtocolCommand;
	}
}
