﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: dialog.cpp
\brief 主界面。配置串口、展示通信数据。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/
#include "dialog.h"

#include <QCheckbox>
#include <QFileDialog>
#include <QStandardItemModel>
#include <QPushButton>
#include <QScrollBar>
#include <QTextEdit>
#include <QTimer>

#include "base/basedll/base_file.h"
#include "base/basedll/base_class.h"
#include "autoreplymodel.h"
#include "autoreplysetting.h"
#include "com_serial.h"
#include "tablemodel.h"

CDialog::CDialog(QWidget* parent) : QDialog(parent), m_nCurrentRow(0), m_nCurrentColumn(0), m_bTurned(false), m_bFirstShow(true){
	ui.setupUi(this);
	m_nReceivedCount = 0;
	setWindowFlag(Qt::WindowMaximizeButtonHint, true);
	setWindowTitle(QString::fromLocal8Bit("串口调试助手"));
	m_pSerial = NULL;
	ui.tableView->setAlternatingRowColors(true);

	initializeAutoReply();
	ui.chkSimulate->setEnabled(false);
	ui.btnSend->setEnabled(false);
	m_colorReceive = Qt::darkGreen;
	m_colorSend = Qt::red;

	//QStandardItemModel* pModel = new CTableModel(this);
	QStandardItemModel* pModel = new QStandardItemModel(this);
	ui.tableView->setModel(pModel);
	pModel->insertRows(0, 50);
	pModel->insertColumns(0, 40);
	int idx = 0;
	while (idx < pModel->columnCount()) {
		ui.tableView->setColumnWidth(idx, 40);
		idx++;
	}
	ui.comboBoxBaudrate->setCurrentIndex(3);
	ui.comboBoxDatabit->setCurrentIndex(3);

	m_pTimer = new QTimer(this);
	m_pTimer->setInterval(1000); // 设置定时器周期。单位:毫秒
	m_pTimer->start(); // 启动定时器

	connect(ui.btnOpenSerialPort, &QPushButton::clicked, this, &CDialog::slot_openSerialPort);	// 打开串口
	connect(ui.btnCloseSerialPort, &QPushButton::clicked, this, &CDialog::slot_closeSerialPort); // 关闭串口
	connect(ui.btnSend, &QPushButton::clicked, this, &CDialog::slot_sendData);	// 发送数据
	connect(ui.btnClear, &QPushButton::clicked, this, &CDialog::slot_clear);	// 清空数据
	connect(ui.btnSave2File, &QPushButton::clicked, this, &CDialog::slot_save2File);	// 将报文保存到文件
	connect(ui.btnAutoReply, &QPushButton::clicked, this, &CDialog::slot_autoReplySetting);	// 自动回复设置
	connect(ui.chkAutoReply, &QPushButton::toggled, this, &CDialog::slot_autoReplyChanged);	// 自动回复状态变化
	connect(m_pTimer, &QTimer::timeout, this, &CDialog::slot_timeOut); // 绑定槽函数
}
CDialog::~CDialog() { 
	if (NULL != m_pSerial) {
		m_pSerial->close();
	}
}

void CDialog::slot_openSerialPort() {
	// 初始化
	ns_train::s_serial_config serialConfig;
	strcpy(serialConfig.strDevicename, ui.lineEditSerialName->text().toLocal8Bit().data());
	serialConfig.u32Baudrate = ui.comboBoxBaudrate->currentText().toUShort();
	serialConfig.byDatabits = (type_uint8)ui.comboBoxDatabit->currentText().toUShort();
	switch (ui.comboBoxStopbit->currentIndex()) {
	case 0:
	default:
		serialConfig.chStopbit = 0; // 1位
		break;
	case 1:
		serialConfig.chStopbit = 1; // 1.5位
		break;
	case 2:
		serialConfig.chStopbit = 2; // 2位
		break;
	}
	switch (ui.comboBoxParity->currentIndex()) {
	case 0:
	default:
		serialConfig.chParity = 'N'; // 无校验
		break;
	case 1:
		serialConfig.chParity = 'O'; // 奇校验
		break;
	case 2:
		serialConfig.chParity = 'E'; // 偶校验
		break;
	}
	if (NULL != m_pSerial) {
		m_pSerial->close();
		delete m_pSerial;
	}
	m_pSerial = new ns_train::CPPSerial(serialConfig);
	ui.btnSend->setEnabled(m_pSerial->open());
	ui.chkSimulate->setEnabled(m_pSerial->open());
	ui.btnOpenSerialPort->setEnabled(!m_pSerial->open());
	ui.btnCloseSerialPort->setEnabled(m_pSerial->open());
}

void CDialog::slot_closeSerialPort() {
	if (NULL == m_pSerial) {
		return;
	}
	m_pSerial->close();
	delete m_pSerial;
	m_pSerial = NULL;
	ui.btnOpenSerialPort->setEnabled(true);
	ui.btnCloseSerialPort->setEnabled(false);
}

void CDialog::slot_timeOut() {
	if (NULL == m_pSerial) {
		return;
	}
	static const int c_nBufferLength = 12800;
	static char buf[c_nBufferLength] = { '\0' }; // 需要注意防止内存不够用
	int readCount = 0;
	if (ui.chkSimulate->isChecked()) {	// 测试代码
		static int nSendIndex = 0;
		QString str;
		switch (nSendIndex)	{
			case 0:	{ // 10 49 01 4A 16 
				// 第1组自动应答报文
				str = "10 49 01 4A 16";
				readCount = getDataFromString(str, buf, c_nBufferLength);
				break;
			}
			case 1: {
				str = "10 40 01 41 16";
				readCount = getDataFromString(str, buf, c_nBufferLength);
				break;
			}
			case 2: { 
				str = "68 09 09 68 73 01 64 01 06 01 00 00 14 F4 16";
				readCount = getDataFromString(str, buf, c_nBufferLength);
				break;
			}
			case 3: { 
				str = "10 5A 01 5B 16";
				readCount = getDataFromString(str, buf, c_nBufferLength);
				break;
			}
			case 4: { 
				str = "10 7A 01 7B 16";
				readCount = getDataFromString(str, buf, c_nBufferLength);
				break;
			}
			default: {
				readCount = 0;
				break;
			}
		}
		nSendIndex++;
		if (nSendIndex > 4) {
			nSendIndex = 0;
		}
	}
	else {
		readCount = m_pSerial->read(buf, c_nBufferLength);
		if (readCount <= 0) {
			return;
		}
	}
	showData(buf, readCount, true);
	if (ui.chkAutoReply->isChecked()) {
		if ((m_nReceivedCount + readCount) < RECEIVE_BUFFER_LENGTH) {
			memcpy(m_pBuffer, buf, readCount);
			m_nReceivedCount += readCount;
			autoReply();
		}
	}
}
void CDialog::roll() {
	QStandardItemModel *pModel = dynamic_cast<QStandardItemModel*>(ui.tableView->model());
	int nRowCount = pModel->rowCount();
	int nColumnCount = pModel->columnCount();
	if (m_nCurrentColumn >= nColumnCount) {
		m_nCurrentColumn = 0;
		m_nCurrentRow++;
		ui.tableView->scrollTo(pModel->index(m_nCurrentRow, m_nCurrentColumn));
	}
	if (m_nCurrentRow >= nRowCount) {
		m_nCurrentRow = 0;
		slot_clear();
		
		m_bTurned = !m_bTurned;
		if (m_bTurned) {
			m_colorReceive = Qt::blue;
			m_colorSend = Qt::magenta;
		}
		else {
			m_colorReceive = Qt::darkGreen;
			m_colorSend = Qt::red;
		}
	}
}
void CDialog::slot_clear() {
	QStandardItemModel *pModel = dynamic_cast<QStandardItemModel*>(ui.tableView->model());
	int nRowCount = pModel->rowCount();
	int nColumnCount = pModel->columnCount();
	pModel->clear();
	pModel->setRowCount(nRowCount);
	pModel->setColumnCount(nColumnCount);
	int idx = 0;
	while (idx < pModel->columnCount()) {
		ui.tableView->setColumnWidth(idx, 40);
		idx++;
	}
	m_nCurrentRow = 0;
	m_nCurrentColumn = 0;
	m_bLastShow = true;
}

void CDialog::slot_sendData() {
	if (NULL == m_pSerial) {
		return;
	}
	if (!m_pSerial->isOpen()) {
		return;
	}
	QString strSend = ui.textEditSend->toPlainText();// 以空格区分的二进制文本
	QStringList strList = strSend.split(" "); 
	QStringList::iterator ite = strList.begin();
	type_uint32 u32Count = 0;
	QString str;
	if (0 == strList.size()) {
		return;
	}
	char* pBuf = new char[strList.size()];
	bool bOk = false;
	while (ite != strList.end()) {
		str = (*ite).trimmed();
		if (0 == str.length()) {
			ite++;
			continue;
		}
		bOk = false;
		pBuf[u32Count++] = str.toUShort(&bOk, 16);
		ite++;
		if (!bOk) {
			u32Count--;
			ite--;
		}
	}
	m_pSerial->write(pBuf, u32Count);
	showData(pBuf, u32Count, false);
	m_nReceivedCount = 0; // 复位

	delete[] pBuf;
	pBuf = NULL;
}

void CDialog::slot_autoReplySetting() {
	CAutoReplySetting setting(this, m_pModel);
	setting.exec();
}
void CDialog::slot_autoReplyChanged(bool /*bChecked*/) {
	m_nReceivedCount = 0;
}

void CDialog::initializeAutoReply() {
	m_pModel = new CAutoReplyModel(this);
	m_pModel->insertRows(0, 10);
	m_pModel->insertColumns(0, 2);
	static const int c_nBufferLength = 12800;
	static char buf[c_nBufferLength] = { '\0' }; // 需要注意防止内存不够用
	int nCount = 0;
	QString str;
	QByteArray byteArrayReceive;
	QByteArray byteArraySend;

	// 第0组自动应答报文
	str = "10 49 01 4A 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArrayReceive = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(0, 0), byteArrayReceive);
	
	str = "10 0B 01 0C 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArraySend = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(0, 1), byteArraySend);

	// 第1组自动应答报文
	str = "10 40 01 41 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArrayReceive = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(1, 0), byteArrayReceive);

	str = "10 20 01 21 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArraySend = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(1, 1), byteArraySend);

	// 第2组自动应答报文
	str = "68 09 09 68 73 01 64 01 06 01 00 00 14 F4 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArrayReceive = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(2, 0), byteArrayReceive);

	str = "10 20 01 21 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArraySend = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(2, 1), byteArraySend);

	// 第3组自动应答报文
	str = "10 5A 01 5B 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArrayReceive = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(3, 0), byteArrayReceive);

	str = "68 09 09 68 28 01 46 01 04 01 00 00 00 75 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArraySend = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(3, 1), byteArraySend);

	// 第4组自动应答报文
	str = "10 7A 01 7B 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArrayReceive = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(4, 0), byteArrayReceive);

	str = "68 09 09 68 20 01 64 01 07 01 00 00 14 A2 16";
	nCount = getDataFromString(str, buf, c_nBufferLength);
	byteArraySend = QByteArray(buf, nCount);
	m_pModel->setData(m_pModel->index(4, 1), byteArraySend);
}

void CDialog::autoReply() {
	int row = 0;
	QByteArray byteArrayReceive;
	QByteArray byteArraySend;
	if (0 == m_nReceivedCount) {
		return;
	}
	while (row < m_pModel->rowCount()) {
		byteArrayReceive = m_pModel->data(m_pModel->index(row, 0), Qt::EditRole).toByteArray();
		if (0 == byteArrayReceive.size()) {
			return;
		}
		if (m_nReceivedCount >= byteArrayReceive.size()) {
			// 判断是否一致
			int idx = 0;
			for (; idx < byteArrayReceive.size(); idx++) {
				if (m_pBuffer[idx] != byteArrayReceive[idx]) {
					break;
				}
			}
			if (idx == byteArrayReceive.size()) {
				byteArraySend = m_pModel->data(m_pModel->index(row, 1), Qt::EditRole).toByteArray();
				type_int32 nCount = byteArraySend.size();
				char* pBuf = byteArraySend.data();
				m_pSerial->write(pBuf, nCount);
				showData(pBuf, nCount, false);
				m_nReceivedCount = 0; // 复位
				return;
			}
		}
		row++;
	}
}

int CDialog::getDataFromString(const QString& str, char* pBuffer, int nBufferLength) {
	QStringList strList = str.split(" ");
	QStringList::iterator ite = strList.begin();
	QString strTemp;
	QByteArray dataArray;
	quint8 data;
	int nCount = 0;
	if (strList.size() > nBufferLength) {
		return 0;
	}
	while (ite != strList.end()) {
		strTemp = (*ite).trimmed().toUpper();
		dataArray = strTemp.toLatin1();
		if (2 != dataArray.size()) {
			return 0;
		}
		data = 0;
		if ((dataArray[0] >= '0') && (dataArray[0] <= '9')) {
			data = (dataArray[0] - '0')*16;
		} 
		else if ((strTemp[0] >= 'A') && (strTemp[0] <= 'F')) {
			data = (dataArray[0] - 'A' + 10) * 16;
		}
		if ((strTemp[1] >= '0') && (strTemp[1] <= '9')) {
			data += (dataArray[1] - '0');
		}
		else if ((strTemp[1] >= 'A') && (strTemp[1] <= 'F')) {
			data += (dataArray[1] - 'A' + 10);
		}
		pBuffer[nCount++] = data;
		ite++;
	}
	return nCount;
}

void CDialog::slot_save2File() {
	QString szDir = ns_train::getDirectory("$TRAINDEVHOME/test/chapter07/ks07_06/").c_str();
	QString strFilter("text file(*.txt);;*(*.*)");

#ifdef WIN32
	QString fileName = QFileDialog::getSaveFileName(this, tr("select file to save"), szDir, strFilter);
#else
	QString fileName = QFileDialog::getSaveFileName(this, tr("select file to save"),
		szDir, strFilter, NULL, QFileDialog::DontUseNativeDialog);
#endif
	if (fileName.isEmpty()) {
		return;
	}
	fileName = ns_train::getFileName(fileName.toLocal8Bit().data()).c_str();
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Truncate)) {
		return;
	}
	QString str;
	
	QStandardItemModel *pModel = dynamic_cast<QStandardItemModel*>(ui.tableView->model());

	QVariant var;
	int nRowCount = pModel->rowCount();
	int nColumnCount = pModel->columnCount();
	QModelIndex index;
	QString strData;
	for (int iRow = 0; iRow < nRowCount; iRow++) {
		index = pModel->index(iRow, 0);
		var = pModel->data(index, Qt::UserRole);
		if (!var.isValid()) {
			continue;
		}
		if (var.toBool()) {
			file.write("Receive: ");
		}
		else {
			file.write("Send:\t ");
		}
		for (int iCol = 0; iCol < nColumnCount; iCol++) {
			index = pModel->index(iRow, iCol);
			var = pModel->data(index, Qt::EditRole);
			if (!var.isValid()) {
				continue;
			}
			strData = var.toString() + " ";
			file.write(strData.toLocal8Bit());
		}
		file.write("\n");
	}
	file.close();
}

void CDialog::showData(const char* pBuffer, int nDataCount, bool bReceive) {
	int nShow = 0;
	QString value;
	QModelIndex index;
	QColor color;
	if (!ui.chkRefresh->isChecked()) {
		return;
	}
	QStandardItemModel *pModel = dynamic_cast<QStandardItemModel*>(ui.tableView->model());
	if (m_bFirstShow) {
		m_bFirstShow = false;
	}
	else {
		if (m_bLastShow != bReceive) {
			m_nCurrentColumn = 0;	// 当从接收转为发送，或从发送转为接收时，需要进行换行。true : 该换行了, false : 不用换行
			m_nCurrentRow++;
			
			ui.tableView->scrollTo(pModel->index(m_nCurrentRow, m_nCurrentColumn));			
		}
	}
	m_bLastShow = bReceive;
	while (nShow < nDataCount) {
		roll();
		color = (bReceive ? m_colorReceive : m_colorSend);
		value.sprintf("%02X", static_cast<quint8>(pBuffer[nShow]));
		index = pModel->index(m_nCurrentRow, m_nCurrentColumn);
		pModel->setData(index, value);
		pModel->setData(index, color, Qt::TextColorRole);
		pModel->setData(index, bReceive, Qt::UserRole);
		++m_nCurrentColumn;
		nShow++;
	}
}