﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: dialog.h
\brief 数据处理线程类定义。模拟数据变化。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/
#pragma once

#include "ui_dialog.h"
#include <QDialog>
#include <QColor>

QT_BEGIN_NAMESPACE
class QTimer;
QT_END_NAMESPACE

namespace ns_train {
	class CPPSerial;
}
namespace Ui {
	class CDialog;
}
class CAutoReplyModel;

static const int RECEIVE_BUFFER_LENGTH = 1024;

class CDialog : public QDialog {
	Q_OBJECT
public:
	CDialog(QWidget* parent);	// 构造函数
	~CDialog();					// 析构函数

private slots:
	void slot_openSerialPort();		// 槽函数-打开串口
	void slot_closeSerialPort();	// 槽函数-关闭串口
	void slot_sendData();			// 槽函数-发送数据
	void slot_clear();				// 清空数据
	void slot_timeOut();			// 槽函数-定时器超时
	void slot_autoReplySetting();	// 自动回复设置
	void slot_autoReplyChanged(bool);	// 自动回复状态变化
	void slot_save2File();			// 保持报文到文件
private:
	/**
	* @brief 自动应答
	* @return 无
	*/
	void autoReply();
	/**
	* @brief 初始化自动应答
	* @return 无
	*/
	void initializeAutoReply();
	/**
	* @brief 滚屏
	* @return 无
	*/
	void roll();
	/**
	* @brief 显示数据
	* @param[in] pBuffer 数据缓冲区
	* @param[in] nDataCount 数据个数
	* @param[in] bReceive true:数据为接收数据, false:数据为发送数据
	* @return 无
	*/
	void showData(const char* pBuffer, int nDataCount, bool bReceive);

	/**
	* @brief 将报文字符串转换为数据
	* @param[in] str 报文字符串
	* @param[in] pBuffer 数据缓冲区
	* @param[in] nBufferLength 数据缓冲区长度
	* @return 数据个数
	*/	
	int getDataFromString(const QString& str, char* pBuffer, int nBufferLength);
private:
	Ui::CDialog ui;					// 界面
	ns_train::CPPSerial *m_pSerial; // 串口通信对象
	QTimer* m_pTimer;				// 定时器
	int m_nCurrentRow;				// 当前行
	int m_nCurrentColumn;			// 当前列
	CAutoReplyModel* m_pModel;		// 自动回复的报文数据
	char m_pBuffer[RECEIVE_BUFFER_LENGTH];	// 接收报文的暂存区
	int	m_nReceivedCount;			// 已经接收到的字节数
	bool m_bTurned;					// 滚屏标志
	QColor m_colorReceive;			// 接收报文的颜色
	QColor m_colorSend;				// 发送报文的颜色
	bool m_bFirstShow;				// 初次显示数据
	bool m_bLastShow;				// 上次显示的数据,true:接收, false:发送
};
