﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: app.cpp
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/4
*/
#include "app.h"
#include <iostream>
#include <string>
#include <string.h>
#include "base/basedll/service_api.h"
#include "base/basedll/base_api.h"
#include "base/basedll/base_class.h"
#include "com_serial.h"
#include "clientthread.h"
#include "datavector.h"
#include "dialog.h"
#include "protocolcommand.h"

CApp::CApp(int &argc, char **argv) : QApplication(argc, argv), m_bProcRun(true), m_bTerminal(true), m_bHelp(false), m_pSerial(NULL), m_pClientThread(NULL){
}

CApp::~CApp() {
}

bool CApp::getRunningFlag() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    return m_bProcRun;
}

void CApp::setRunningFlag(bool bRunningFlag) {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    m_bProcRun = bRunningFlag;
}

void CApp::prepare(void) {
    // 初始化
	ns_train::s_serial_config serialConfig;
#ifdef WIN32
	strcpy(serialConfig.strDevicename, "\\\\.\\COM1");
#else
	strcpy(serialConfig.strDevicename, "/dev/tty01");
#endif
	serialConfig.u32Baudrate = 115200;// 波特率
	serialConfig.byDatabits = 8;	// 8位数据位
	serialConfig.chParity = 'N';	// 无校验
	serialConfig.chStopbit = 0;		// 1位停止位
    serialConfig.u32RecvBufferLength = 4096;
    serialConfig.u32SendBufferLength = 4096;
	m_pSerial = new ns_train::CPPSerial(serialConfig);
    if (m_bTerminal) {
        std::cout << "打开串口:" << serialConfig.strDevicename << std::endl;
        std::cout << "波特率:" << (int)serialConfig.u32Baudrate << std::endl;
        std::cout << "数据位:" << (int)serialConfig.byDatabits << std::endl;
        std::cout << "校验:" << serialConfig.chParity << std::endl;
    }
}

bool CApp::initialize() {
    /* do initialize work */
    prepare();
    CDataVector* pDataVector = CDataVector::instance(); // 数据容器指针
    if (NULL != pDataVector) {
        pDataVector->initialize();
    }
    m_pClientThread = new CClientThread();
    m_pClientThread->start(m_pSerial);
	CDialog dialog(NULL);
	dialog.exec();
    return true;
}
/**
* @brief 模拟程序退出前的清理工作。
* @return true：成功，false:失败
*/
void CApp::beforeExit(void) {
    // 先退出线程，再释放套接字资源
    if (NULL != m_pClientThread) {
        m_pClientThread->exitThread();
        delete m_pClientThread;
        m_pClientThread = NULL;
    }
    if (NULL != m_pSerial) {
		m_pSerial->close();
		delete m_pSerial;
		m_pSerial = NULL;
	}
    if (!m_bHelp) {
        std::cout << ">>> 模块退出。" << std::endl;
        //char ch = '\0';
        //std::cin >> ch;
    }
}

void CApp::CommandProc() {
	if (!m_bTerminal) {
		return;
	}

#ifndef WIN32
	system("mesg n");	// 取消系统日志终端输出
#endif
	std::string strInput;
	std::cout << "-------------------------------------------------------" << std::endl;
	std::cout << ">>> 模块正常启动，请勿关闭!" << std::endl;
	CDataVector* pDataVector = CDataVector::instance(); // 数据容器指针
	while (m_bProcRun) {
		printMenu();
		std::cout << "<client -> command> : ";
		std::cin >> strInput;
		if (strInput.compare("help") == 0) {
			printMenu();
		}
		else if (strInput.compare("exit") == 0) {
			setRunningFlag(false);
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "模块正在退出 ..." << std::endl;
			return;
		}
		else if (strInput.compare("getdata") == 0) {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "请输入数据项序号（从0开始）:" << std::endl;
			/* do something.*/
			type_uint32 uDataId = 0;
			std::cin >> uDataId;
			CCommandGetData* pCommand = new CCommandGetData;
			pCommand->setDataId(uDataId);
			m_pClientThread->pushCommand(pCommand);
		}
		else if (strInput.compare("setdata") == 0) {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "请输入数据项序号（从0开始）、数据值：" << std::endl;
			/* do something.*/
			type_uint32 uDataId = 0;
			type_double dValue = 0.f;
			std::cin >> uDataId >> dValue;
			CCommandSetData* pCommand = new CCommandSetData;
			pCommand->setData(uDataId, dValue);
			m_pClientThread->pushCommand(pCommand);
		}
		else if (strInput.compare("data") == 0) {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << "数据项的值为:" << std::endl;
			/* do something.*/
			type_double dValue = 0.f;
			for (type_uint32 i = 0; i < pDataVector->getSize(); i++) {
				pDataVector->getData(i + 1, dValue); // 数据项的索引从1开始计数。
				std::cout << "data" << i + 1 << "=" << dValue << std::endl;
			}
		}
        else if (strInput.compare("getfile") == 0) {
            std::cout << "-------------------------------------------------------" << std::endl;
            std::cout << "请输入文件名（不含路径）:" << std::endl;
            /* do something.*/
            std::string str;
            std::cin >> str;
			CCommandGetFile* pCommand = new CCommandGetFile;
			pCommand->setFileName(str);
			m_pClientThread->pushCommand(pCommand);
        }
        else {
			std::cout << "-------------------------------------------------------" << std::endl;
			std::cout << ">>> 输入命令错误，请输入 \"help\"查询可用命令！" << std::endl;
			printMenu();
		}
	}
}

void CApp::printMenu(void) {
	if (!m_bTerminal) {
		return;
	}

	std::cout << "-------------------------------------------------------" << std::endl;
	std::cout << ">>> 1.帮助                           help" << std::endl;
	std::cout << ">>> 2.退出                           exit" << std::endl;
	std::cout << ">>> 3.更新服务器端数据               setdata" << std::endl;
	std::cout << ">>> 4.从服务器端获取数据             getdata" << std::endl;
	std::cout << ">>> 5.从服务器端获取文件             getfile" << std::endl;
	std::cout << ">>> 6.显示所有数据                   data" << std::endl;
}    

int CApp::exec(void) {
    return 0;
}
