﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: app.h
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/
#pragma once
#include "customtype.h"
#include "base_class.h"
#include "com_serial.h"

#include <QApplication>

namespace ns_train {
    class CPPSerial;
}
class CClientThread;
/**
* @brief 数据容器类。用来模拟对双精度浮点数数组的操作。
*/
class CApp : public QApplication {
public:
	/**
	* @brief  构造函数，定义为私有的目的是防止他人使用该类构造对象。
	* @param[in] argc 参数个数
	* @param[in] argv 参数数组
	* @return 无
	*/
	CApp(int &argc, char **argv);
	/**
	* @brief  析构函数，定义为私有的目的是防止他人用delete语句删除单体对象。
	* @return 无
	*/
	virtual ~CApp();
    /**
    * @brief 初始化。调用者需要显式调用。
    * @return true:成功,false:失败
    */
    bool initialize(void);
    /**
    * @brief 退出前的处理。
    * @return void
    */
    void beforeExit(void);
    /**
    * @brief 获取应用运行状态。
    * @return true:正在运行,false:退出运行
    */
    bool getRunningFlag(void);
    /**
    * @brief 设置应用运行状态。
    * @param[in] bRunningFlag true:正在运行,false:退出运行
    * @return void
    */
    void setRunningFlag(bool bRunningFlag);
    /**
    * @brief 执行。在该接口中实现应用程序的主要功能。
    * @return int
    */
    int exec(void);
    /**
    * @brief 打印帮助菜单。
    * @return void
    */
    void printMenu(void);  
    /**
    * @brief 交互命令处理。
    * @return void
    */
    void CommandProc(void);	 
private:

    /**
    * @brief 准备工作
    *
    * @return void
    */
    void prepare(void);
private:
    ns_train::CPPMutex m_mutex; // 用来保护对成员的访问
    bool m_bProcRun;            // 用来控制程序是否继续运行
    bool m_bTerminal;           // true:以终端方式启动，可以向终端输出信息。false:不允许向终端输出信息
    bool m_bHelp;               // true:用户需要查看帮助信息。false:无需输出帮助信息
    ns_train::CPPSerial* m_pSerial;	// 串口对象指针
    CClientThread* m_pClientThread;	// 客户端维护线程
};
