﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: dialog.cpp
\brief  CDialog
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
*/

#include "dialog.h"
#include "customtype.h"
#include "datavector.h"
#include "qcustomplot.h"

#include "ui_dialog.h" // 头文件名称来自: xxx.ui  ---> ui_xxx.h

#include <QComboBox>
#include <QSharedPointer>

CDialog::CDialog(QWidget* pParent) : QDialog(pParent), m_pUi(new Ui::CDialog()){
	m_pUi->setupUi(this);
	initialize();
	slot_currentIndexChanged(0);
	m_pDataTimer = new QTimer(this);
	m_pDataTimer->start(1000);

	connect(m_pUi->btnOk, &QPushButton::clicked, this, &CDialog::accept);
	void (QComboBox:: * currentIndexChangedSingal)(int) = &QComboBox::currentIndexChanged;//信号指针
	connect(m_pUi->comboBox, currentIndexChangedSingal, this, &CDialog::slot_currentIndexChanged);
	connect(m_pDataTimer, &QTimer::timeout, this, &CDialog::slot_timeout);
}

CDialog::~CDialog(){
	delete m_pUi;
}
void CDialog::slot_currentIndexChanged(int idx) {
	m_nId = m_pUi->comboBox->itemData(idx).toUInt();
	m_pUi->plotWidget->clearGraphs();
	addGraph();
}
void CDialog::slot_timeout() {
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
	// calculate and add a new data point to each graph:
	qreal dValue;
	pDataVector->getData(m_nId, dValue);
	qint64 tm = QDateTime::currentDateTime().toSecsSinceEpoch();
	m_pUi->plotWidget->graph(0)->addData(tm, dValue);

	// make key axis range scroll with the data:
	m_pUi->plotWidget->xAxis->rescale();
	m_pUi->plotWidget->graph(0)->rescaleValueAxis(false, true);

	m_pUi->plotWidget->xAxis->setRange(tm, 60, Qt::AlignRight);
	//qDebug() << m_pUi->plotWidget->xAxis->range().upper;
	m_pUi->plotWidget->replot(QCustomPlot::rpQueuedReplot);
}

void CDialog::initialize() {
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
	std::list<type_uint32> listIds;
	pDataVector->getDataIds(listIds);
	std::list<type_uint32>::iterator ite = listIds.begin();
	int idx = 0;
	while (ite != listIds.end()) {
		m_pUi->comboBox->insertItem(idx, QString::fromLocal8Bit(pDataVector->getDataName(*ite).c_str()), *ite);
		ite++;
	}
	addGraph();
}
	
void CDialog::addGraph() {
	m_pUi->plotWidget->addGraph();
	// 设定背景为黑色
	m_pUi->plotWidget->setBackground(QBrush(Qt::gray));
	// 设定右上角图例可见
	m_pUi->plotWidget->legend->setVisible(true);
	// 设定右上角图例的字体
	m_pUi->plotWidget->legend->setFont(QFont("Helvetica", 9));
	QVector<qreal> x(60), y(60);
	qint64 tm = QDateTime::currentDateTime().toSecsSinceEpoch();

	// 为折线添加数据
	for (int i = 0; i<60; i++) {
		x[i] = tm-60+i;
		y[i] = 0;
		qDebug() << i << "," << y[i];
	}

	// 设置画笔
	m_pUi->plotWidget->graph(0)->setPen(QPen(Qt::blue));
	// 设置右上角图形标注名称
	m_pUi->plotWidget->graph(0)->setName("curve");
	// 传入数据，setData的两个参数类型为double
	//m_pUi->plotWidget->graph(0)->setData(x, y);

	// 设置X轴文字标注
	m_pUi->plotWidget->xAxis->setLabel("angle");
	m_pUi->plotWidget->xAxis->setTicks(true);
	QSharedPointer<QCPAxisTickerDateTime> ticker(new QCPAxisTickerDateTime);
	// 创建坐标轴样式为时间格式；为时分秒结构。
	ticker->setDateTimeFormat("hh:mm:ss");
	m_pUi->plotWidget->xAxis->setTicker(ticker);
	m_pUi->plotWidget->xAxis->setLabel("time");
	m_pUi->plotWidget->xAxis->setTickLabelRotation(60);
	m_pUi->plotWidget->xAxis->setTickLabels(true);
	// 设置Y轴文字标注
	m_pUi->plotWidget->yAxis->setLabel("value");
	// 设置X轴坐标范围
	m_pUi->plotWidget->xAxis->setRange(0, 60);
	// 设置Y轴坐标范围
	//m_pUi->plotWidget->yAxis->setRange(-1, 1);
	m_pUi->plotWidget->yAxis->setTicks(true);
	// 在坐标轴右侧和上方画线，和X/Y轴一起形成一个矩形
	m_pUi->plotWidget->axisRect()->setupFullAxesBox();
}

