﻿/*!
* Copyright (C) 2020 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: datavector.cpp
\brief 示例文件
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2020/2
*/
#include "datavector.h"
//#include "observer.h"

SData::SData() {
    nId = 0;
    dataValue = 0.f;
    maxValue = -3.402823466e+38F; // 把最大值初始化为最小浮点数，因为只有这样，当更新dataValue时才有可能将新的最大值更新到maxValue中。
    minValue = 3.402823466e+38F;  // 把最大值初始化为最大浮点数，因为只有这样，当更新dataValue时才有可能将新的最小值更新到maxValue中。
}
ns_train::CPPMutex g_mutex;
CDataVector* CDataVector::instance() {
    ns_train::CPPMutexLocker mutexLocker(&g_mutex);
    static CDataVector dataVector;
    return &dataVector;
}

CDataVector::CDataVector() : m_pDataVector(NULL), m_nSize(0) {

}

CDataVector::~CDataVector() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    if (NULL != m_pDataVector) {
        delete m_pDataVector;
        m_pDataVector = NULL;
    }
}
type_uint32 CDataVector::getSize() {
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    return m_nSize;
}
bool CDataVector::initialize() {
    m_mutex.lock();
    m_nSize = 5; // 假设只有5个数据。此处模拟从数据库中读取记录。
    m_pDataVector = new SData[m_nSize];
    for (type_uint32 i = 0; i < m_nSize; i++) {
        m_pDataVector[i].nId = i+1;       // 有效Id值从1开始
        m_pDataVector[i].dataValue = 0.f; // 数据
    }
    m_mutex.unlock();
    return true;
}

bool CDataVector::getData(type_uint32 nId, type_double &dValue) {
    dValue = 0.f;
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    if ((nId < 1) || (nId > m_nSize)) {
        return false; // 数据项Id无效
    }
    if (NULL == m_pDataVector) {
        return false;
    }
    dValue = m_pDataVector[nId-1].dataValue;
    return true;
}

std::string CDataVector::getDataName(type_uint32 nId) {
    std::string strName;
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    if ((nId < 1) || (nId > m_nSize)) {
        return false; // 数据项Id无效
    }
    if (NULL == m_pDataVector) {
        return false;
    }
    switch ((nId-1) % 10) {
        case 0:
            strName="扭矩";
            break;
        case 1:
            strName="温度";
            break;
        case 2:
            strName="湿度";
            break;
        case 3:
            strName="CO2浓度";
            break;
        case 4:
            strName="压力";
            break;
        case 5:
            strName="海拔";
            break;
        case 6:
            strName="仰角";
            break;
        default:
            strName="Unknown";
            break;
    }
    return strName;
}

bool CDataVector::setData(type_uint32 nId, type_double dNewValue) {
    {
	    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
	    if ((nId < 1) || (nId > m_nSize)) {
	        return false; // 数据项Id无效
	    }
	    if (NULL == m_pDataVector) {
	        return false;
	    }
	    type_uint32 index = nId - 1;
	    m_pDataVector[index].dataValue = dNewValue;
	    if (m_pDataVector[index].maxValue < dNewValue) {
	        m_pDataVector[index].maxValue = dNewValue; // 更新最大值
	    }
	    if (m_pDataVector[index].minValue > dNewValue) {
	        m_pDataVector[index].minValue = dNewValue; // 更新最小值
	    }
    }
    //SData data;
    //data.nId = nId;
    //data.dataValue = dNewValue;
    //CMeasurementObserver::instance()->notifyChangedAnalog(data);
    return true;
}
int CDataVector::getDataIds(std::list<type_uint32>& listIds) {
    listIds.clear();
    ns_train::CPPMutexLocker mutexLocker(&m_mutex);
    for (type_uint32 i = 0; i < m_nSize; i++) {
        listIds.push_back(m_pDataVector[i].nId);
    }
    return static_cast<int>(listIds.size());
}
