﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: clientthread.cpp
\brief 服务器端线程类实现。该线程负责处理客户端连接及交互。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/
#include "observer.h"
#include <algorithm>
#include "service_api.h"

///////////////////////////////////////////////////////////////////////
CSubscriber::CSubscriber() {
    m_id = 0;
}
CSubscriber::~CSubscriber() {
}
void CSubscriber::notifyChangedAnalog(const SData& idValue) {
    emit sig_dataChanged(idValue);
}
///////////////////////////////////////////////////////////////////////
static ns_train::CPPMutex g_observerMutex;
CMeasurementObserver* CMeasurementObserver::instance() {
    ns_train::CPPMutexLocker mutexLocker(&g_observerMutex);
    static CMeasurementObserver observer;
    return &observer;
}
CMeasurementObserver::CMeasurementObserver() {

}
void CMeasurementObserver::notifyChangedAnalog(const SData&idvalue) {
    ns_train::CPPMutexLocker locker(&m_mutex);
    std::list<CSubscriber*>::iterator ite = m_lstSubscribers.begin();
    while (ite != m_lstSubscribers.end()) {
        if (idvalue.nId == (*ite)->getAnalogDataId()) {
            (*ite)->notifyChangedAnalog(idvalue);
        }
        ite++;
    }
}

size_t CMeasurementObserver::addSubscriber(CSubscriber* pSubscriber) {
    ns_train::CPPMutexLocker locker(&m_mutex);
    m_lstSubscribers.push_back(pSubscriber);
    return m_lstSubscribers.size();
}

size_t CMeasurementObserver::removeSubscriber(CSubscriber* pSubscriber) {
    ns_train::CPPMutexLocker locker(&m_mutex);
    std::list<CSubscriber*>::iterator ite = std::find(m_lstSubscribers.begin(), m_lstSubscribers.end(), pSubscriber);
    if (ite != m_lstSubscribers.end()) {
        m_lstSubscribers.erase(ite);
    }
    return m_lstSubscribers.size();
}