#########################################################################################
# 注意：此文件用于放置工程文件的公共(基本)设置，
# 在各 QT 工程文件中通过 include 函数包含，不应单独使用！
# 开发人员需要定义下面几个系统环境变量：
# PROJECT_DEV_HOME		系统开发主目录，放置与开发相关的各个子模块
# PROJECT_BUILD_TYPE	系统编译版本定义，指定编译版本类型：debug|release|all
# PROJECT_BUILD_BIT		系统程序编译位数定义，指定编译位数：32|64
#########################################################################################

# 需先通过环境变量 PROJECT_DEV_HOME 指定开发目录
# 由于 isEmpty 函数不能直接对环境变量进行判断，所以先将其放入一个临时变量中
isEmpty(PROJECT_HOME) {
	DEVHOME = $$(PROJECT_DEV_HOME)
}
!isEmpty(PROJECT_HOME) {
	DEVHOME = $$PROJECT_HOME
}
isEmpty(DEVHOME) {
	error('PROJECT_DEV_HOME'环境变量必须被定义.)
}
# 设置系统执行文件路径、库文件路径、头文件包含路径变量
PROJECT_BIN_PATH = $$DEVHOME/bin
PROJECT_LIB_PATH = $$DEVHOME/lib
PROJECT_OBJ_PATH = $$DEVHOME/obj
PROJECT_SRC_PATH = $$DEVHOME/src
PROJECT_UIC_PATH = $$DEVHOME/obj/uic
PROJECT_INCLUDE_PATH = $$DEVHOME/include

QMAKE_LIBDIR *= $$PROJECT_LIB_PATH
DEPENDPATH *=  .\
		$$PROJECT_INCLUDE_PATH
INCLUDEPATH *=  .\
		$$PROJECT_INCLUDE_PATH

# 加载编译版本配置
#########################################################################################
#
# 通过环境变量 PROJECT_BUILD_TYPE 指定编译版本类型：debug|release
# 分别对应 调试版、运行版
#
# 通过环境变量 PROJECT_BUILD_BIT 指定编译位数类型：32|64
# 分别对应 32 位、64 位
#
#####################################################################################################

# 配置系统使用的编译版本类型
# 如果未指定 PROJECT_BUILD_TYPE 环境变量，缺省按 debug 处理
# isEmpty() 是Qt提供的配置函数，用来在pro、pri配置文件中判断指定变量的值是否为空，为空表示未设置该变量。
BUILDTYPE = $$(PROJECT_BUILD_TYPE)
isEmpty(BUILDTYPE) {	
	message('PROJECT_BUILD_TYPE'环境变量未设置，使用缺省值debug.)
	BUILDTYPE = debug
}
# 对PROJECT_BUILD_TYPE设置进行简单的错误检查
# BUILDTYPE的值必须为单值
# count()用来判断指定变量的值被定义过几次
!count(BUILDTYPE,1){
	error(仅能定义使用一种有效的编译版本类型，请检查PROJECT_BUILD_TYPE环境变量设置.)
}
!equals(BUILDTYPE,all){
!equals(BUILDTYPE,debug){
!equals(BUILDTYPE,release){
	error(未被支持的的编译版本类型，请检查PROJECT_BUILD_TYPE环境变量设置.)
		}
	}
}

# 公共配置中不引入Qt，如果需要引入Qt，请在需要使用Qt的项目pro中进行配置。
CONFIG -= qt

# 不同编译版本相关的配置
equals(BUILDTYPE,debug){
	CONFIG += debug
	CONFIG -= release
}
equals(BUILDTYPE,release){
	CONFIG += release
	CONFIG -= debug
}
equals(BUILDTYPE,all){
	CONFIG += debug_and_release build_all
}
# 指定代码中宏定义
debug_and_release {
	CONFIG(debug, debug|release) {
	    DEFINES += PROJECT_DEBUG
	}
	CONFIG(release, debug|release) {
	    DEFINES += PROJECT_RELEASE
	}
} else {
	debug:DEFINES += PROJECT_DEBUG
	release:DEFINES += PROJECT_RELEASE
}

# 配置系统使用的编译位数类型
# 如果未指定 PROJECT_BUILD_BIT 环境变量，缺省按 32 位处理
BUILDBIT = $$(PROJECT_BUILD_BIT)
isEmpty(BUILDBIT) {	
	message('PROJECT_BUILD_BIT'环境变量未设置，使用缺省值 32 位.)
	BUILDBIT = 32
}

# 对PROJECT_BUILD_BIT设置进行简单的错误检查
# BUILDBIT的值必须为单值
!count(BUILDBIT,1){
	error(仅能定义使用一种有效的编译位数类型，请检查PROJECT_BUILD_BIT环境变量设置.)
}
!equals(BUILDBIT,32){
	!equals(BUILDBIT,64){
		error(未被支持的的编译位数类型，请检查PROJECT_BUILD_BIT环境变量设置.)
	}
}

# 不同编译版本相关的配置
equals(BUILDBIT,32){
	# 扩展 32 位配置项
	CONFIG *= x86

	DEFINES *= PROJECT_32
}
equals(BUILDBIT,64){
	# 扩展 64 位配置项
	CONFIG *= x64

	DEFINES *= PROJECT_64
}


# 加载编译器设置
#########################################################################################
#
# 不同平台的编译器设置
#
#####################################################################################################

# 获取编译 Qt 的编译器类型
PROJECT_QMAKESPEC = $$(QMAKESPEC)

#UNIX + gcc下声明使用预编译头文件
#GCC 3.4 及以后版本支持预编译头文件
unix{
	contains( PROJECT_QMAKESPEC, g++ ) {
		CONFIG *= precompile_header
	}
}
#WIN32下声明使用预编译头文件
win32{
	CONFIG *= precompile_header
}

#UNIX下编译设置
unix{
	DEFINES *= unix __unix
}

#WIN32下编译设置
win32{
	# 0x0500 -> 0x0501 操作系统最低 Windows XP，不再支持 Windows 2000
	DEFINES	*= WIN32 _WIN32_WINNT=0x0501

	# ui 生成的文件使用 utf-8 编码，编译时产生 4819 警告，去掉
	QMAKE_CXXFLAGS *= -wd4819

	# 去掉 strcpy 等编译警告
	QMAKE_CXXFLAGS *= -wd4996 -wd4251 -wd4275
}


#激活 STL、RTTI、EXCEPTIONS 支持
CONFIG *= stl exceptions rtti
#激活多线程、编译警告
CONFIG *= thread warn_on


# 指定不同编译版本中间文件目录
debug_and_release {
	CONFIG(debug, debug|release) {
	    PROJECT_OBJ_PATH = $$PROJECT_OBJ_PATH/debug
	}
	CONFIG(release, debug|release) {
	    PROJECT_OBJ_PATH = $$PROJECT_OBJ_PATH/release
	}
} else {
	debug:PROJECT_OBJ_PATH = $$PROJECT_OBJ_PATH/debug
	release:PROJECT_OBJ_PATH = $$PROJECT_OBJ_PATH/release
}
DEFINES *= QT_NO_OPENGL
